@@ -26,7 +26,7 @@
  * (c) 2001 James Courtier-Dutton <James@superbug.demon.co.uk>
  *
  * 
- * $Id: audio_alsa_out.c,v 1.151.2.2 2005/07/17 21:49:39 miguelfreitas Exp $
+ * $Id: audio_alsa_out.c,v 1.160 2006/05/03 19:46:06 dsalt Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -223,6 +223,7 @@ static void *ao_alsa_handle_event_thread(void *data) {
 	xine_event_t              event;
 	xine_audio_level_data_t   data;
 	xine_stream_t            *stream;
+	xine_list_iterator_t      ite;
 	
 	this->mixer.right_vol = right_vol;
 	this->mixer.left_vol  = left_vol;
@@ -242,8 +243,9 @@ static void *ao_alsa_handle_event_thread(void *data) {
 	event.data_length = sizeof(data);
 	
 	pthread_mutex_lock(&this->class->xine->streams_lock);
-	for(stream = xine_list_first_content(this->class->xine->streams); 
-	    stream; stream = xine_list_next_content(this->class->xine->streams)) {
+	for(ite = xine_list_front(this->class->xine->streams); 
+	    ite; ite = xine_list_next(this->class->xine->streams, ite)) {
+	  stream = xine_list_get_value(this->class->xine->streams, ite);
 	  event.stream = stream;
 	  xine_event_send(stream, &event);
 	}
@@ -270,7 +272,11 @@ static long ao_alsa_get_volume_from_percent(int val, long min, long max) {
  * Error callback, we need to control this, 
  * error message should be printed only in DEBUG mode.
  */
-static void error_callback(const char *file, int line, 
+static void
+#ifdef __GNUC__
+	    __attribute__((format (printf, 5, 6)))
+#endif
+	    error_callback(const char *file, int line, 
 			   const char *function, int err, const char *fmt, ...) {
 #ifdef DEBUG
   va_list   args;
@@ -471,6 +477,10 @@ static int ao_alsa_open(ao_driver_t *this_gen, uint32_t bits, uint32_t rate, int
 	     this->num_channels, err, snd_strerror(err));
     goto close;
   }
+#if SND_LIB_VERSION >= 0x010009
+  /* Restrict a configuration space to contain only real hardware rates */
+  err = snd_pcm_hw_params_set_rate_resample(this->audio_fd, params, 0);
+#endif
   /* set the stream rate [Hz] */
   dir=0;
   err = snd_pcm_hw_params_set_rate_near(this->audio_fd, params, &rate, &dir);
@@ -714,6 +724,19 @@ static void xrun(alsa_driver_t *this)
 }
 #endif
 
+/*
+ * resume from suspend
+ */
+static int resume(snd_pcm_t *pcm)
+{
+  int res;
+  while ((res = snd_pcm_resume(pcm)) == -EAGAIN)
+    sleep(1);
+  if (! res)
+    return 0;
+  return snd_pcm_prepare(pcm);
+}
+
 /*
  * Write audio data to output buffer (blocking using snd_pcm_wait)
  */
@@ -738,6 +761,12 @@ static int ao_alsa_write(ao_driver_t *this_gen, int16_t *data, uint32_t count) {
 #endif
   snd_pcm_status_alloca(&pcm_stat);
   state = snd_pcm_state(this->audio_fd);
+  if (state == SND_PCM_STATE_SUSPENDED) {
+    res = resume(this->audio_fd);
+    if (res < 0)
+      return 0;
+    state = snd_pcm_state(this->audio_fd);
+  }
   if (state == SND_PCM_STATE_XRUN) {
 #ifdef LOG_DEBUG
     printf("audio_alsa_out:write:XRUN before\n");
@@ -787,6 +816,12 @@ static int ao_alsa_write(ao_driver_t *this_gen, int16_t *data, uint32_t count) {
       printf("audio_alsa_out:write:result=%ld:%s\n",result, snd_strerror(result));
 #endif
       state = snd_pcm_state(this->audio_fd);
+      if (state == SND_PCM_STATE_SUSPENDED) {
+	res = resume(this->audio_fd);
+	if (res < 0)
+	  return 0;
+	continue;
+      }
       if ( (state != SND_PCM_STATE_PREPARED) &&
            (state != SND_PCM_STATE_RUNNING) &&
            (state != SND_PCM_STATE_DRAINING) ) {
@@ -1650,7 +1685,7 @@ static ao_info_t ao_info_alsa = {
  * exported plugin catalog entry
  */
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_AUDIO_OUT, AO_OUT_ALSA_IFACE_VERSION, "alsa", XINE_VERSION_CODE, &ao_info_alsa, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
