@@ -591,7 +591,7 @@ static void *init_class (xine_t *xine, void *data) {
   return this;
 }
 
-static ao_info_t ao_info_coreaudio = {
+static const ao_info_t ao_info_coreaudio = {
   1
 };
 
@@ -599,7 +599,7 @@ static ao_info_t ao_info_coreaudio = {
  * exported plugin catalog entry
  */
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_AUDIO_OUT, AO_OUT_COREAUDIO_IFACE_VERSION, "coreaudio", XINE_VERSION_CODE, &ao_info_coreaudio, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
