@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: audio_directx2_out.c,v 1.2.2.2 2005/07/17 21:49:40 miguelfreitas Exp $
+ * $Id: audio_directx2_out.c,v 1.7 2006/06/02 22:18:56 dsalt Exp $
  *
  *
  * xine audio output plugin using DirectX
@@ -129,7 +129,7 @@ typedef struct {
   int paused;                      /* paused mode */
   int finished;                    /* driver finished */
   int failed;                      /* don't open modal dialog again */
-  int count;                       /* number of current free parts */
+  int count;                       /* current free part number */
 
   pthread_t buffer_service;        /* service thread for operating with DSB */
   pthread_cond_t data_cond;        /* signals on data */
@@ -151,7 +151,11 @@ static int buffer_ready(dx2_driver_t *this);
 
 
 /* popup a dialog with error */
-static void error_message(const char *fmt, ...) {
+static void
+#ifdef __GNUC__
+	    __attribute__((format (printf, 1, 2)))
+#endif
+	    error_message(const char *fmt, ...) {
   char message[256];
   va_list ap;
 
@@ -176,7 +180,7 @@ static char *dsound_strerror(HRESULT err) {
     case DSERR_BUFFERLOST: return _("memory buffer has been lost and must be restored");
     case DSERR_CONTROLUNAVAIL: return _("requested buffer control is not available");
     case DSERR_GENERIC: return _("undetermined error inside DirectSound subsystem");
-#ifdef HWUNAVAIL
+#ifdef DSERR_HWUNAVAIL
     case DSERR_HWUNAVAIL: return _("DirectSound hardware device is unavailable");
 #endif
     case DSERR_INVALIDCALL: return _("function is not valid for the current state of the object");
@@ -1020,11 +1024,11 @@ static void *init_class(xine_t *xine, void *data) {
 }
 
 
-static ao_info_t ao_info_directx2 = {
+static const ao_info_t ao_info_directx2 = {
   10
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   { PLUGIN_AUDIO_OUT, AO_OUT_DIRECTX2_IFACE_VERSION, "directx2", XINE_VERSION_CODE, &ao_info_directx2, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
