@@ -20,7 +20,7 @@
  * audio_directx_out.c, direct sound audio output plugin for xine
  * by Matthew Grooms <elon@altavista.com>
  *
- * $Id: audio_directx_out.c,v 1.12 2004/09/02 19:56:40 valtri Exp $
+ * $Id: audio_directx_out.c,v 1.14 2006/06/02 22:18:56 dsalt Exp $
  */
 
 /*
@@ -856,14 +856,14 @@ static void *init_class (xine_t *xine, void *data) {
   return audiox;
 }
 
-static ao_info_t ao_info_directx = {
+static const ao_info_t ao_info_directx = {
   1                    /* priority        */
 };
 
 /*
  * exported plugin catalog entry
  */
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_AUDIO_OUT, AO_DIRECTX_IFACE_VERSION, "directx", XINE_VERSION_CODE, &ao_info_directx, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
