@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2001-2002 the xine project
+ * Copyright (C) 2001-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: audio_sun_out.c,v 1.27 2002/11/20 11:57:40 mroi Exp $
+ * $Id: audio_sun_out.c,v 1.43 2006/06/14 15:44:54 komadori Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -59,7 +59,7 @@
 #define	AUDIO_PRECISION_16	16
 #endif
 
-#define AO_SUN_IFACE_VERSION 6
+#define AO_SUN_IFACE_VERSION 8
 
 #define GAP_TOLERANCE         5000
 #define GAP_NONRT_TOLERANCE   AO_MAX_GAP
@@ -67,9 +67,9 @@
 
 
 typedef struct {
-  audio_driver_class_t driver_class;
+  audio_driver_class_t	driver_class;
 
-  config_values_t *config;
+  xine_t	       *xine;
 } sun_class_t;
 
 
@@ -77,6 +77,8 @@ typedef struct sun_driver_s {
 
   ao_driver_t    ao_driver;
 
+  xine_t	*xine;
+
   char		*audio_dev;
   int            audio_fd;
   int            capabilities;
@@ -96,6 +98,7 @@ typedef struct sun_driver_s {
   }		 use_rtsc;
 
   int		 convert_u8_s8;	       /* Builtin conversion 8-bit UNSIGNED->SIGNED */
+  int		 mixer_volume;
 
 #if	CS4231_WORKAROUND
   /*
@@ -124,7 +127,7 @@ typedef struct sun_driver_s {
  * try to figure out, if the soundcard driver provides usable (precise)
  * sample counter information
  */
-static int realtime_samplecounter_available(char *dev)
+static int realtime_samplecounter_available(xine_t *xine, char *dev)
 {
   int fd = -1;
   audio_info_t info;
@@ -147,9 +150,12 @@ static int realtime_samplecounter_available(char *dev)
   if (silence == NULL)
     goto error;
     
-  if ((fd = open(dev, O_WRONLY)) < 0)
+  if ((fd = open(dev, O_WRONLY|O_NONBLOCK)) < 0)
     goto error;
 
+  /* We wanted non blocking open but now put it back to normal */
+  fcntl(fd, F_SETFL, fcntl(fd, F_GETFL) & ~O_NONBLOCK);
+
   AUDIO_INITINFO(&info);
   info.play.sample_rate = 44100;
   info.play.channels = AUDIO_CHANNELS_STEREO;
@@ -157,17 +163,17 @@ static int realtime_samplecounter_available(char *dev)
   info.play.encoding = AUDIO_ENCODING_LINEAR;
   info.play.samples = 0;
   if (ioctl(fd, AUDIO_SETINFO, &info)) {
-    fprintf(stderr, "rtsc: SETINFO failed\n");
+    xprintf(xine, XINE_VERBOSITY_DEBUG, "rtsc: SETINFO failed\n");
     goto error;
   }
     
   if (write(fd, silence, len) != len) {
-    fprintf(stderr, "rtsc: write failed\n");
+    xprintf(xine, XINE_VERBOSITY_DEBUG, "rtsc: write failed\n");
     goto error;
   }
 
   if (ioctl(fd, AUDIO_GETINFO, &info)) {
-    fprintf(stderr, "rtsc: GETINFO1, %s\n", strerror(errno));
+    xprintf(xine, XINE_VERBOSITY_DEBUG, "rtsc: GETINFO1, %s\n", strerror(errno));
     goto error;
   }
 
@@ -189,11 +195,11 @@ static int realtime_samplecounter_available(char *dev)
       break;
 
     if (ioctl(fd, AUDIO_GETINFO, &info)) {
-	fprintf(stderr, "rtsc: GETINFO2 failed, %s\n", strerror(errno));
+	xprintf(xine, XINE_VERBOSITY_DEBUG, "rtsc: GETINFO2 failed, %s\n", strerror(errno));
 	goto error;
     }
     if (info.play.samples < last_samplecnt) {
-	fprintf(stderr, "rtsc: %u > %u?\n", last_samplecnt, info.play.samples);
+	xprintf(xine, XINE_VERBOSITY_DEBUG, "rtsc: %u > %u?\n", last_samplecnt, info.play.samples);
 	goto error;
     }
 
@@ -407,10 +413,10 @@ static int ao_sun_open(ao_driver_t *this_gen,
   int pass;
   int ok;
 
-  printf ("audio_sun_out: ao_sun_open rate=%d, mode=%d\n", rate, mode);
+  xprintf(this->xine, XINE_VERBOSITY_DEBUG, "audio_sun_out: ao_sun_open rate=%d, mode=%d\n", rate, mode);
 
   if ( (mode & this->capabilities) == 0 ) {
-    printf ("audio_sun_out: unsupported mode %08x\n", mode);
+    xprintf(this->xine, XINE_VERBOSITY_DEBUG, "audio_sun_out: unsupported mode %08x\n", mode);
     return 0;
   }
 
@@ -430,15 +436,15 @@ static int ao_sun_open(ao_driver_t *this_gen,
    * open audio device
    */
 
-  this->audio_fd=open(this->audio_dev,O_WRONLY|O_NONBLOCK);
+  this->audio_fd = open(this->audio_dev, O_WRONLY|O_NONBLOCK);
   if(this->audio_fd < 0) {
-    printf("audio_sun_out: Opening audio device %s failed: %s\n",
-	   this->audio_dev, strerror(errno));
+    xprintf(this->xine, XINE_VERBOSITY_LOG, 
+	    _("audio_sun_out: opening audio device %s failed: %s\n"), this->audio_dev, strerror(errno));
     return 0;
   }
   
   /* We wanted non blocking open but now put it back to normal */
-  fcntl(this->audio_fd, F_SETFL, fcntl(this->audio_fd, F_GETFL)&~O_NONBLOCK);
+  fcntl(this->audio_fd, F_SETFL, fcntl(this->audio_fd, F_GETFL) & ~O_NONBLOCK);
 
   /*
    * configure audio device
@@ -509,9 +515,9 @@ static int ao_sun_open(ao_driver_t *this_gen,
   }
 
   if (!ok) {
-      printf("audio_sun_out: Cannot configure audio device for "
-	     "%dhz, %d channel, %d bits: %s\n",
-	     rate, info.play.channels, bits, strerror(errno));
+      xprintf(this->xine, XINE_VERBOSITY_DEBUG,
+	      "audio_sun_out: Cannot configure audio device for %dhz, %d channel, %d bits: %s\n",
+	      rate, info.play.channels, bits, strerror(errno));
       close(this->audio_fd);
       this->audio_fd = -1;
       return 0;
@@ -537,7 +543,7 @@ static int ao_sun_open(ao_driver_t *this_gen,
 	   this->input_sample_rate, this->output_sample_rate);
   */
 
-  printf ("audio_sun_out: %d channels output\n",this->num_channels);
+  xprintf(this->xine, XINE_VERBOSITY_DEBUG, "audio_sun_out: %d channels output\n",this->num_channels);
   return this->output_sample_rate;
 }
 
@@ -562,8 +568,9 @@ static int ao_sun_delay(ao_driver_t *this_gen)
       (this->frames_in_buffer == 0 || info.play.samples > 0)) {
 
     if (info.play.samples < this->last_samplecnt) {
-	printf("*** broken sound driver, sample counter runs backwards, cur %u < prev %u\n",
-	       info.play.samples, this->last_samplecnt);
+	xprintf(this->xine, XINE_VERBOSITY_DEBUG, 
+		"audio_sun_out: broken sound driver, sample counter runs backwards, cur %u < prev %u\n",
+		info.play.samples, this->last_samplecnt);
     }
     this->last_samplecnt = info.play.samples;
 
@@ -767,9 +774,10 @@ static int ao_sun_get_property (ao_driver_t *this_gen, int property) {
   switch(property) {
   case AO_PROP_MIXER_VOL:
   case AO_PROP_PCM_VOL:
-    if (ioctl(this->audio_fd, AUDIO_GETINFO, &info) < 0)
-      return 0;
-    return info.play.gain * 100 / AUDIO_MAX_GAIN;
+    if (ioctl(this->audio_fd, AUDIO_GETINFO, &info) > -1) {
+      this->mixer_volume = info.play.gain * 100 / AUDIO_MAX_GAIN;
+    }
+    return this->mixer_volume;
 #if !defined(__NetBSD__)    /* audio_info.output_muted is missing on NetBSD */
   case AO_PROP_MUTE_VOL:
     if (ioctl(this->audio_fd, AUDIO_GETINFO, &info) < 0)
@@ -794,6 +802,7 @@ static int ao_sun_set_property (ao_driver_t *this_gen, int property, int value)
   switch(property) {
   case AO_PROP_MIXER_VOL:
   case AO_PROP_PCM_VOL:
+    this->mixer_volume = value;
     info.play.gain = value * AUDIO_MAX_GAIN / 100;
     if (ioctl(this->audio_fd, AUDIO_SETINFO, &info) < 0)
       return ~value;
@@ -862,44 +871,53 @@ static int ao_sun_ctrl(ao_driver_t *this_gen, int cmd, ...) {
 static ao_driver_t *ao_sun_open_plugin (audio_driver_class_t *class_gen, const void *data) {
 
   sun_class_t         *class = (sun_class_t *) class_gen;
-  config_values_t     *config = class->config;
+  config_values_t     *config = class->xine->config;
   sun_driver_t	      *this;
   char                *devname;
+  char                *audiodev;
   int                  audio_fd;
   int                  status;
   audio_info_t	       info;
 
-  this = (sun_driver_t *) malloc (sizeof (sun_driver_t));
+  this = (sun_driver_t *) xine_xmalloc (sizeof (sun_driver_t));
 
-  /* Fill the .xinerc file with options */ 
+  this->xine = class->xine;
+
+  audiodev = getenv("AUDIODEV");
+
+  /* This config entry is security critical, is it really necessary? */
   devname = config->register_string(config,
-				    "audio.sun_audio_device",
-				    "/dev/audio",
-				    _("device used for audio output with the 'Sun' audio plugin"),
-				    NULL,
-				    10, NULL,
+				    "audio.device.sun_audio_device",
+				    audiodev && *audiodev ? audiodev : "/dev/audio",
+				    _("Sun audio device name"),
+				    _("Specifies the file name for the Sun audio device "
+				      "to be used.\nThis setting is security critical, "
+				      "because when changed to a different file, xine "
+				      "can be used to fill this file with arbitrary content. "
+				      "So you should be careful that the value you enter "
+				      "really is a proper Sun audio device."),
+				    XINE_CONFIG_SECURITY, NULL,
 				    NULL);
 
   /*
    * find best device driver/channel
    */
 
-  printf ("audio_sun_out: Opening audio device %s...\n", devname);
+  xprintf(this->xine, XINE_VERBOSITY_DEBUG, "audio_sun_out: Opening audio device %s...\n", devname);
 
   /*
    * open the device
    */
 
-  audio_fd=open(this->audio_dev = devname, O_WRONLY|O_NONBLOCK);
+  this->audio_fd = open(this->audio_dev = devname, O_WRONLY|O_NONBLOCK);
 
-  if(audio_fd < 0) 
+  if(this->audio_fd < 0) 
   {
-    fprintf(stderr, "audio_sun_out: opening audio device %s failed:\n%s\n",
-	   devname, strerror(errno));
+    xprintf(this->xine, XINE_VERBOSITY_LOG,
+	    _("audio_sun_out: opening audio device %s failed: %s\n"), devname, strerror(errno));
 
     free (this);
     return NULL;
-
   }
 
   /*
@@ -910,32 +928,34 @@ static ao_driver_t *ao_sun_open_plugin (audio_driver_class_t *class_gen, const v
   info.play.encoding = AUDIO_ENCODING_LINEAR;
   info.play.precision = AUDIO_PRECISION_16;
   info.play.sample_rate = 44100;
-  status = ioctl(audio_fd, AUDIO_SETINFO, &info);
+  status = ioctl(this->audio_fd, AUDIO_SETINFO, &info);
+  
+  if (status < 0) {
+    xprintf(this->xine, XINE_VERBOSITY_LOG,
+	    _("audio_sun_out: audio ioctl on device %s failed: %s\n"), devname, strerror(errno));
+
+    free (this);
+    return NULL;
+  }
 
   /*
    * get capabilities
    */
 
-  this->capabilities = 0;
+  this->capabilities = AO_CAP_MODE_MONO | AO_CAP_MODE_STEREO | AO_CAP_8BITS
+  		     | AO_CAP_PCM_VOL | AO_CAP_MUTE_VOL;
 
-  printf ("audio_sun_out: supported modes are ");
+  /*
+   * get initial mixer volume
+   */
 
-  this->capabilities |= AO_CAP_MODE_MONO;
-  printf ("mono ");
-
-  this->capabilities |= AO_CAP_MODE_STEREO;
-  printf ("stereo ");
-
-  this->capabilities |= AO_CAP_8BITS;
-  printf ("8bit ");
-
-  this->capabilities |= AO_CAP_PCM_VOL | AO_CAP_MUTE_VOL;
-  printf ("\n");
-
-  close (audio_fd);
+  this->mixer_volume = ao_sun_get_property(&this->ao_driver, AO_PROP_MIXER_VOL);
 
+  close (this->audio_fd);
   this->audio_fd = -1;
-  this->use_rtsc = realtime_samplecounter_available(this->audio_dev);
+
+  this->xine = class->xine;
+  this->use_rtsc = realtime_samplecounter_available(this->xine, this->audio_dev);
   this->output_sample_rate = 0;
 
   this->ao_driver.get_capabilities	= ao_sun_get_capabilities;
@@ -976,20 +996,20 @@ static void ao_sun_dispose_class (audio_driver_class_t *this_gen) {
 static void *ao_sun_init_class (xine_t *xine, void *data) {
   sun_class_t         *this;
 
-  this = (sun_class_t *) malloc (sizeof (sun_class_t));
+  this = (sun_class_t *) xine_xmalloc (sizeof (sun_class_t));
 
   this->driver_class.open_plugin     = ao_sun_open_plugin;
   this->driver_class.get_identifier  = ao_sun_get_identifier;
   this->driver_class.get_description = ao_sun_get_description;
   this->driver_class.dispose         = ao_sun_dispose_class;
 
-  this->config = xine->config;
+  this->xine = xine;
 
   return this;
 }
 
 
-static ao_info_t ao_info_sun = {
+static const ao_info_t ao_info_sun = {
   10
 };
 
@@ -997,7 +1017,7 @@ static ao_info_t ao_info_sun = {
  * exported plugin catalog entry
  */
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_AUDIO_OUT, AO_SUN_IFACE_VERSION, "sun", XINE_VERSION_CODE, &ao_info_sun, ao_sun_init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
