@@ -36,7 +36,7 @@
  * * ANIM (Animations)
  *   - Animation works fine, without seeking.
  *
- * $Id: demux_iff.c,v 1.12 2004/08/21 21:21:12 manfredtremmel Exp $
+ * $Id: demux_iff.c,v 1.16 2006/06/02 22:18:56 dsalt Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -927,7 +927,7 @@ static int demux_iff_send_chunk(demux_plugin_t *this_gen) {
           buf->decoder_flags           |= BUF_FLAG_FRAME_END;
 
         xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
-                 "sending audio buf with %d bytes, %lld pts, %d duration\n",
+                 "sending audio buf with %" PRId32 " bytes, %" PRId64 " pts, %" PRId32 " duration\n",
                  buf->size, buf->pts, buf->decoder_info[0]);
         this->audio_fifo->put(this->audio_fifo, buf);
       }
@@ -1330,13 +1330,13 @@ static void *init_plugin (xine_t *xine, void *data) {
 /*
  * exported plugin catalog entry
  */
-demuxer_info_t demux_info_iff = {
+static const demuxer_info_t demux_info_iff = {
   10                       /* priority */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_DEMUX, 25, "iff", XINE_VERSION_CODE, &demux_info_iff, init_plugin },
+  { PLUGIN_DEMUX, 26, "iff", XINE_VERSION_CODE, &demux_info_iff, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
 
