@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003 the xine project
+ * Copyright (C) 2003-2005 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -19,7 +19,7 @@
  */
 
 /*
- * $Id: demux_image.c,v 1.15.2.1 2005/04/20 17:21:04 mroi Exp $
+ * $Id: demux_image.c,v 1.23 2006/06/02 22:18:56 dsalt Exp $
  *
  * image dummy demultiplexer
  */
@@ -42,8 +42,11 @@
 
 #include "xine_internal.h"
 #include "xineutils.h"
+#include "bswap.h"
 #include "demux.h"
 
+#define IMAGE_HEADER_LEN 4
+
 typedef struct demux_image_s {
   demux_plugin_t        demux_plugin;
 
@@ -51,6 +54,7 @@ typedef struct demux_image_s {
   fifo_buffer_t        *video_fifo;
   input_plugin_t       *input;
   int                   status;
+  int                   buf_type;
 } demux_image_t ;
 
 typedef struct {
@@ -64,37 +68,51 @@ static int demux_image_get_status (demux_plugin_t *this_gen) {
   return this->status;
 }
 
-static int demux_image_send_chunk (demux_plugin_t *this_gen) {
+static int demux_image_next (demux_plugin_t *this_gen, int preview) {
   demux_image_t *this = (demux_image_t *) this_gen;
   buf_element_t *buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
 
   buf->content = buf->mem;
-  buf->type = BUF_VIDEO_IMAGE;
+  buf->type = this->buf_type;
 
-  buf->size = this->input->read (this->input, buf->mem, buf->max_size-1);
+  buf->size = this->input->read (this->input, (char *)buf->mem, buf->max_size-1);
 
   if (buf->size <= 0) {
-    buf->free_buffer(buf);
-    xine_usec_sleep(250000);
+    buf->size = 0;
+    buf->decoder_flags |= BUF_FLAG_FRAME_END;
+    this->status = DEMUX_FINISHED;
   } else {
-    lprintf("got %i bytes\n", buf->size);
-    this->video_fifo->put (this->video_fifo, buf);
+    this->status = DEMUX_OK;
   }
-  this->status = DEMUX_OK;
+  
+  if (preview)
+    buf->decoder_flags = BUF_FLAG_PREVIEW;
+
+  this->video_fifo->put (this->video_fifo, buf);
+
   return this->status;
 }
 
+static int demux_image_send_chunk (demux_plugin_t *this_gen) {
+  return demux_image_next(this_gen, 0);
+}
+
 static void demux_image_send_headers (demux_plugin_t *this_gen) {
   demux_image_t *this = (demux_image_t *) this_gen;
 
   this->video_fifo  = this->stream->video_fifo;
 
+  _x_demux_control_start(this->stream);
+
+  this->input->seek (this->input, 0, SEEK_SET);
+
+  /* we can send everything here. this makes image decoder a lot easier */
+  while (demux_image_next(this_gen,1) == DEMUX_OK);
+
   this->status = DEMUX_OK;
 
   _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_VIDEO, 1);
   _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_AUDIO, 0);
-  
-  this->input->seek (this->input, 0, SEEK_SET);
 }
 
 static int demux_image_seek (demux_plugin_t *this_gen,
@@ -132,11 +150,23 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
 				    input_plugin_t *input) {
 
   demux_image_t *this;
+  int buf_type = BUF_VIDEO_IMAGE;
 
   switch (stream->content_detection_method) {
 
-  case METHOD_BY_CONTENT:
+  case METHOD_BY_CONTENT: {
+    char header[IMAGE_HEADER_LEN];
+    if (_x_demux_read_header(input, (unsigned char *)header, IMAGE_HEADER_LEN) != IMAGE_HEADER_LEN) {
+      return NULL;
+    }
+    if (memcmp (header, "GIF", 3) == 0 /* GIF */
+        || memcmp (header, "\377\330\377", 3) == 0 /* JPEG */
+	|| (BE_16(&header[0]) == 0xffd8) /* another JPEG */
+	|| memcmp (header, "\x89PNG", 4) == 0) { /* PNG */
+      break;
+    }
     return NULL;
+  }
   break;
 
   case METHOD_BY_EXTENSION: {
@@ -178,6 +208,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
   this->demux_plugin.demux_class       = class_gen;
   
   this->status = DEMUX_FINISHED;
+  this->buf_type = buf_type;
 
   lprintf("opened\n");
   return &this->demux_plugin;
@@ -196,7 +227,7 @@ static char *get_identifier (demux_class_t *this_gen) {
 }
 
 static char *get_extensions (demux_class_t *this_gen) {
-  return "png";
+  return "png gif jpg jpeg";
 }
 
 static char *get_mimetypes (demux_class_t *this_gen) {
@@ -229,11 +260,11 @@ static void *init_class (xine_t *xine, void *data) {
 /*
  * exported plugin catalog entry
  */
-demuxer_info_t demux_info_image = {
-  10                       /* priority */
+static const demuxer_info_t demux_info_image = {
+  11                       /* priority */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_DEMUX, 26, "image", XINE_VERSION_CODE, &demux_info_image, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
