@@ -19,7 +19,7 @@
  */
 
 /*
- * $Id: demux_ogg.c,v 1.160.2.2 2005/07/17 21:49:40 miguelfreitas Exp $
+ * $Id: demux_ogg.c,v 1.166 2006/06/02 22:18:56 dsalt Exp $
  *
  * demultiplexer for ogg streams
  *
@@ -2116,15 +2116,15 @@ static void *ogg_init_class (xine_t *xine, void *data) {
 /*
  * exported plugin catalog entry
  */
-demuxer_info_t demux_info_anx = {
+static const demuxer_info_t demux_info_anx = {
   20                       /* priority */
 };
 
-demuxer_info_t demux_info_ogg = {
+static const demuxer_info_t demux_info_ogg = {
   10                       /* priority */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_DEMUX, 26, "ogg", XINE_VERSION_CODE, &demux_info_ogg, ogg_init_class },
   { PLUGIN_DEMUX, 26, "anx", XINE_VERSION_CODE, &demux_info_anx, anx_init_class },
