@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -16,12 +16,14 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
+ */
+
+/*
  * TechnoTrend PVA File Demuxer by Mike Melanson (melanson@pcisys.net)
  * For more information regarding the PVA file format, refer to this PDF:
  *   http://www.technotrend.de/download/av_format_v1.pdf
  *
- * $Id: demux_pva.c,v 1.6 2003/01/26 15:56:21 tmmm Exp $
+ * $Id: demux_pva.c,v 1.23 2006/06/02 22:18:56 dsalt Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,6 +36,11 @@
 #include <string.h>
 #include <stdlib.h>
 
+/********** logging **********/
+#define LOG_MODULE "demux_pva"
+/* #define LOG_VERBOSE */
+/* #define LOG */
+
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "compat.h"
@@ -48,43 +55,22 @@
 #define PTS_VIDEO 1
 
 typedef struct {
-
   demux_plugin_t       demux_plugin;
 
   xine_stream_t       *stream;
-
-  config_values_t     *config;
-
   fifo_buffer_t       *video_fifo;
   fifo_buffer_t       *audio_fifo;
-
   input_plugin_t      *input;
-
-  pthread_t            thread;
-  int                  thread_running;
-  pthread_mutex_t      mutex;
-  int                  send_end_buffers;
-  
-  int                  send_newpts;
-  int                  buf_flag_seek;
-  int64_t              last_pts[2];
-
-  off_t                data_start;
-  off_t                data_size;
   int                  status;
 
-  char                 last_mrl[1024];
+  int                  send_newpts;
+  int64_t              last_pts[2];
 
+  off_t                data_size;
 } demux_pva_t;
 
 typedef struct {
-
   demux_class_t     demux_class;
-
-  /* class-wide, global variables here */
-
-  xine_t           *xine;
-  config_values_t  *config;
 } demux_pva_class_t;
 
 
@@ -92,8 +78,7 @@ typedef struct {
    i guess llabs may not be available everywhere */
 #define abs(x) ( ((x)<0) ? -(x) : (x) )
 
-static void check_newpts( demux_pva_t *this, int64_t pts, int video )
-{
+static void check_newpts( demux_pva_t *this, int64_t pts, int video ){
   int64_t diff;
 
   diff = pts - this->last_pts[video];
@@ -101,12 +86,8 @@ static void check_newpts( demux_pva_t *this, int64_t pts, int video )
   if( pts &&
       (this->send_newpts || (this->last_pts[video] && abs(diff)>WRAP_THRESHOLD) ) ) {
 
-    if (this->buf_flag_seek) {
-      xine_demux_control_newpts(this->stream, pts, BUF_FLAG_SEEK);
-      this->buf_flag_seek = 0;
-    } else {
-      xine_demux_control_newpts(this->stream, pts, 0);
-    }
+    _x_demux_control_newpts(this->stream, pts, 0);
+
     this->send_newpts = 0;
     this->last_pts[1-video] = 0;
   }
@@ -115,11 +96,8 @@ static void check_newpts( demux_pva_t *this, int64_t pts, int video )
     this->last_pts[video] = pts;
 }
 
-
-
 /* returns 1 if the PVA file was opened successfully, 0 otherwise */
 static int open_pva_file(demux_pva_t *this) {
-
   unsigned char preamble[PVA_PREAMBLE_SIZE];
 
   this->input->seek(this->input, 0, SEEK_SET);
@@ -145,8 +123,8 @@ static int open_pva_file(demux_pva_t *this) {
 }
 
 static int demux_pva_send_chunk(demux_plugin_t *this_gen) {
-
   demux_pva_t *this = (demux_pva_t *) this_gen;
+
   buf_element_t *buf;
   int chunk_size;
   unsigned char preamble[PVA_PREAMBLE_SIZE];
@@ -156,7 +134,7 @@ static int demux_pva_send_chunk(demux_plugin_t *this_gen) {
   unsigned int flags, header_len;
 
   if (this->input->read(this->input, preamble, PVA_PREAMBLE_SIZE) !=
-    PVA_PREAMBLE_SIZE) {
+      PVA_PREAMBLE_SIZE) {
     this->status = DEMUX_FINISHED;
     return this->status;
   }
@@ -169,8 +147,7 @@ static int demux_pva_send_chunk(demux_plugin_t *this_gen) {
 
   chunk_size = BE_16(&preamble[6]);
 
-  current_file_pos = this->input->get_current_pos(this->input) -
-    this->data_start;
+  current_file_pos = this->input->get_current_pos(this->input);
 
   if (preamble[2] == 1) {
     
@@ -193,8 +170,8 @@ static int demux_pva_send_chunk(demux_plugin_t *this_gen) {
       buf->type = BUF_VIDEO_MPEG;
       buf->pts = pts;
       pts = 0;
-      buf->extra_info->input_pos = current_file_pos;
-      buf->extra_info->input_length = this->data_size;
+      if( this->data_size )
+        buf->extra_info->input_normpos = (int) ((double) current_file_pos * 65535 / this->data_size);
       buf->extra_info->input_time = buf->pts / 90;
 
       if (chunk_size > buf->max_size)
@@ -256,7 +233,7 @@ static int demux_pva_send_chunk(demux_plugin_t *this_gen) {
         return this->status;
       }
 
-      pts = (preamble[0] & 0x0e) << 29 ;
+      pts = (int64_t)(preamble[0] & 0x0e) << 29 ;
       pts |= (BE_16(&preamble[1]) & 0xFFFE) << 14;
       pts |= (BE_16(&preamble[3]) & 0xFFFE) >> 1;
 
@@ -277,7 +254,9 @@ static int demux_pva_send_chunk(demux_plugin_t *this_gen) {
     buf->type = BUF_AUDIO_MPEG;
     buf->pts = pts;
 
-    buf->extra_info->input_pos = this->input->get_current_pos(this->input);
+    if( this->data_size )
+      buf->extra_info->input_normpos = (int) ((double) this->input->get_current_pos(this->input) * 
+                                              65535 / this->data_size);
 
     this->audio_fifo->put (this->audio_fifo, buf);
 
@@ -292,7 +271,6 @@ static int demux_pva_send_chunk(demux_plugin_t *this_gen) {
 }
 
 static void demux_pva_send_headers(demux_plugin_t *this_gen) {
-
   demux_pva_t *this = (demux_pva_t *) this_gen;
   buf_element_t *buf;
   int n;
@@ -303,11 +281,11 @@ static void demux_pva_send_headers(demux_plugin_t *this_gen) {
   this->status = DEMUX_OK;
 
   /* load stream information */
-  this->stream->stream_info[XINE_STREAM_INFO_HAS_VIDEO] = 1;
-  this->stream->stream_info[XINE_STREAM_INFO_HAS_AUDIO] = 1;
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_VIDEO, 1);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_AUDIO, 1);
 
   /* send start buffers */
-  xine_demux_control_start(this->stream);
+  _x_demux_control_start(this->stream);
 
   /* send init info to video decoder (cribbed from demux_elem.c) */
   buf = this->video_fifo->buffer_pool_alloc(this->video_fifo);
@@ -325,7 +303,9 @@ static void demux_pva_send_headers(demux_plugin_t *this_gen) {
   buf->size = n;
 
   buf->pts = 0;
-  buf->extra_info->input_pos = this->input->get_current_pos(this->input);
+  if( this->data_size )
+    buf->extra_info->input_normpos = (int) ((double) this->input->get_current_pos(this->input) * 
+                                              65535 / this->data_size);
   buf->type = BUF_VIDEO_MPEG;
 
   buf->decoder_flags = BUF_FLAG_PREVIEW;
@@ -350,24 +330,28 @@ static void demux_pva_send_headers(demux_plugin_t *this_gen) {
     buf->size = n;
 
     buf->pts = 0;
-    buf->extra_info->input_pos = this->input->get_current_pos(this->input);
+    if( this->data_size )
+      buf->extra_info->input_normpos = (int) ((double) this->input->get_current_pos(this->input) * 
+                                              65535 / this->data_size);
     buf->type = BUF_AUDIO_MPEG;
 
     buf->decoder_flags = BUF_FLAG_PREVIEW;
 
     this->video_fifo->put(this->audio_fifo, buf);
   }
-
 }
 
 #define SEEK_BUFFER_SIZE 1024
 static int demux_pva_seek (demux_plugin_t *this_gen,
-                               off_t start_pos, int start_time) {
+                               off_t start_pos, int start_time, int playing) {
 
   demux_pva_t *this = (demux_pva_t *) this_gen;
   unsigned char seek_buffer[SEEK_BUFFER_SIZE];
   int found = 0;
   int i;
+  
+  start_pos = (off_t) ( (double) start_pos / 65535 *
+              this->data_size );
 
   /* start from the start_pos */
   this->input->seek(this->input, start_pos, SEEK_SET);
@@ -402,14 +386,14 @@ static int demux_pva_seek (demux_plugin_t *this_gen,
   this->input->seek(this->input, -(SEEK_BUFFER_SIZE - i), SEEK_CUR);
 
   /* if thread is not running, initialize demuxer */
-  if( !this->stream->demux_thread_running ) {
+  if( !playing ) {
 
     this->send_newpts = 1;
 
     this->status = DEMUX_OK;
 
-  } else 
-    xine_demux_flush_engine(this->stream);
+  } else
+    _x_demux_flush_engine(this->stream);
 
   return this->status;
 }
@@ -426,7 +410,6 @@ static int demux_pva_get_status (demux_plugin_t *this_gen) {
 }
 
 static int demux_pva_get_stream_length (demux_plugin_t *this_gen) {
-
   return 0;
 }
 
@@ -440,13 +423,12 @@ static int demux_pva_get_optional_data(demux_plugin_t *this_gen,
 }
 
 static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *stream,
-                                    input_plugin_t *input_gen) {
+                                    input_plugin_t *input) {
 
-  input_plugin_t *input = (input_plugin_t *) input_gen;
   demux_pva_t    *this;
 
-  if (! (input->get_capabilities(input) & INPUT_CAP_SEEKABLE)) {
-    printf(_("demux_pva.c: input not seekable, can not handle!\n"));
+  if (!INPUT_IS_SEEKABLE(input)) {
+    xprintf(stream->xine, XINE_VERBOSITY_DEBUG, "input not seekable, can not handle!\n");
     return NULL;
   }
 
@@ -460,8 +442,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   this->demux_plugin.dispose           = demux_pva_dispose;
   this->demux_plugin.get_status        = demux_pva_get_status;
   this->demux_plugin.get_stream_length = demux_pva_get_stream_length;
-  this->demux_plugin.get_video_frame   = NULL;
-  this->demux_plugin.got_video_frame_cb= NULL;
   this->demux_plugin.get_capabilities  = demux_pva_get_capabilities;
   this->demux_plugin.get_optional_data = demux_pva_get_optional_data;
   this->demux_plugin.demux_class       = class_gen;
@@ -470,6 +450,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   switch (stream->content_detection_method) {
 
+  case METHOD_BY_EXTENSION: {
+    char *extensions, *mrl;
+
+    mrl = input->get_mrl (input);
+    extensions = class_gen->get_extensions (class_gen);
+
+    if (!_x_demux_check_extension (mrl, extensions)) {
+      free (this);
+      return NULL;
+    }
+  }
+  /* falling through is intended */
+
   case METHOD_BY_CONTENT:
   case METHOD_EXPLICIT:
 
@@ -479,39 +472,11 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
     }
   break;
 
-  case METHOD_BY_EXTENSION: {
-    char *ending, *mrl;
-
-    mrl = input->get_mrl (input);
-
-    ending = strrchr(mrl, '.');
-
-    if (!ending) {
-      free (this);
-      return NULL;
-    }
-
-    if (strncasecmp (ending, ".pva", 4)) {
-      free (this);
-      return NULL;
-    }
-
-    if (!open_pva_file(this)) {
-      free (this);
-      return NULL;
-    }
-
-  }
-
-  break;
-
   default:
     free (this);
     return NULL;
   }
 
-  strncpy (this->last_mrl, input->get_mrl (input), 1024);
-
   return &this->demux_plugin;
 }
 
@@ -532,19 +497,15 @@ static char *get_mimetypes (demux_class_t *this_gen) {
 }
 
 static void class_dispose (demux_class_t *this_gen) {
-
   demux_pva_class_t *this = (demux_pva_class_t *) this_gen;
 
   free (this);
 }
 
 static void *init_plugin (xine_t *xine, void *data) {
-
   demux_pva_class_t     *this;
 
-  this         = xine_xmalloc (sizeof (demux_pva_class_t));
-  this->config = xine->config;
-  this->xine   = xine;
+  this = xine_xmalloc (sizeof (demux_pva_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
@@ -559,9 +520,12 @@ static void *init_plugin (xine_t *xine, void *data) {
 /*
  * exported plugin catalog entry
  */
+static const demuxer_info_t demux_info_pva = {
+  10                       /* priority */
+};
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_DEMUX, 20, "pva", XINE_VERSION_CODE, NULL, init_plugin },
+  { PLUGIN_DEMUX, 26, "pva", XINE_VERSION_CODE, &demux_info_pva, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
