@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_ts.c,v 1.115 2004/12/20 20:00:27 jcdutton Exp $
+ * $Id: demux_ts.c,v 1.121 2006/06/28 20:20:54 dsalt Exp $
  *
  * Demultiplexer for MPEG2 Transport Streams.
  *
@@ -170,7 +170,7 @@
 #define BODY_SIZE (188 - 4)
 /* more PIDS are needed due "auto-detection". 40 spare media entries  */
 #define MAX_PIDS ((BODY_SIZE - 1 - 13) / 4) + 40 
-#define MAX_PMTS ((BODY_SIZE - 1 - 13) / 4)
+#define MAX_PMTS ((BODY_SIZE - 1 - 13) / 4) + 10
 #define SYNC_BYTE   0x47
 
 #define MIN_SYNCS 3
@@ -583,7 +583,8 @@ static void demux_ts_parse_pat (demux_ts_t*this, unsigned char *original_pkt,
     program_count = 0;
 
     while ((this->program_number[program_count] != INVALID_PROGRAM) &&
-           (this->program_number[program_count] != program_number) ) {
+           (this->program_number[program_count] != program_number)  &&
+           (program_count+1 < MAX_PMTS ) ) {
       program_count++;
     }
     this->program_number[program_count] = program_number;
@@ -868,6 +869,7 @@ static void demux_ts_buffer_pes(demux_ts_t*this, unsigned char *ts,
     if (!demux_ts_parse_pes_header(this->stream->xine, m, ts, len, this->stream)) {
       m->corrupted_pes = 1;
       m->buf->free_buffer(m->buf);
+      m->buf = NULL;
       xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
 	      "demux_ts: PID 0x%.4x: corrupted pes encountered\n", m->pid);
 
@@ -1104,9 +1106,9 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
 		      | this->pmt[program_count][2]) & 0x03ff;
   }
 
-  count=ts_payloadsize(pkt);
+  count=ts_payloadsize(originalPkt);
 
-  ptr = pkt+offset+(PKT_SIZE-count);
+  ptr = originalPkt+offset+(PKT_SIZE-count);
   len = count-offset;
   memcpy (this->pmt_write_ptr[program_count], ptr, len);
   this->pmt_write_ptr[program_count] +=len;
@@ -1674,7 +1676,8 @@ static void demux_ts_parse_packet (demux_ts_t*this) {
    */
    program_count=0;
    if(this->media_num<MAX_PMTS)
-      while ((this->program_number[program_count] != INVALID_PROGRAM) ) {
+      while ((this->program_number[program_count] != INVALID_PROGRAM) && 
+      		 (program_count < MAX_PMTS)) {
         if (pid == this->pmt_pid[program_count]) {
     
 #ifdef TS_LOG
@@ -1833,11 +1836,16 @@ static void demux_ts_dispose (demux_plugin_t *this_gen) {
   demux_ts_t*this = (demux_ts_t*)this_gen;
 
   for (i=0; i < MAX_PMTS; i++) {
-    if (this->pmt[i] != NULL) free(this->pmt[i]);
+    if (this->pmt[i] != NULL) {
+      free(this->pmt[i]);
+      this->pmt[i] = NULL;
+    }
   }
   for (i=0; i < MAX_PIDS; i++) {
-    if (this->media[i].buf != NULL) 
+    if (this->media[i].buf != NULL) { 
       this->media[i].buf->free_buffer(this->media[i].buf);
+      this->media[i].buf = NULL;
+    }
   }
 
   xine_event_dispose_queue (this->event_queue);
@@ -2192,12 +2200,12 @@ static void *init_class (xine_t *xine, void *data) {
 /*
  * exported plugin catalog entry
  */
-demuxer_info_t demux_info_ts = {
+static const demuxer_info_t demux_info_ts = {
   10                       /* priority */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 25, "mpeg-ts", XINE_VERSION_CODE, &demux_info_ts, init_class },
+  { PLUGIN_DEMUX, 26, "mpeg-ts", XINE_VERSION_CODE, &demux_info_ts, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
