@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: dxr3_decode_video.c,v 1.28 2003/01/13 18:09:58 mroi Exp $
+ * $Id: dxr3_decode_video.c,v 1.63 2006/06/18 20:29:04 dgp85 Exp $
  */
  
 /* dxr3 video decoder plugin.
@@ -28,9 +28,6 @@
 
 #include <sys/types.h>
 #include <sys/ioctl.h>
-#if defined(__sun)
-#include <sys/ioccom.h>
-#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -38,17 +35,24 @@
 #include <fcntl.h>
 #include <errno.h>
 
+#define LOG_MODULE "dxr3_decode_video"
+/* #define LOG_VERBOSE */
+/* #define LOG */
+
+#define LOG_VID 0
+#define LOG_PTS 0
+
 #include "xine_internal.h"
 #include "buffer.h"
-#include "dxr3_scr.h"
 #include "video_out_dxr3.h"
 #include "dxr3.h"
 
-#define LOG_VID 1
-#define LOG_PTS 0
+/* once activated, we wait for this amount of missing pan&scan info
+ * before disabling it again */
+#define PAN_SCAN_WINDOW_SIZE 50
 
 /* the number of frames to pass after an out-of-sync situation
-   before locking the stream again */
+ * before locking the stream again */
 #define RESYNC_WINDOW_SIZE 50
 
 /* we adjust vpts_offset in metronom, when skip_count reaches this value */
@@ -57,9 +61,6 @@
 /* the number of frames to pass before we stop duration correction */
 #define FORCE_DURATION_WINDOW_SIZE 100
 
-/* offset for mpeg header parsing */
-#define HEADER_OFFSET 0
-
 
 /* plugin class initialization function */
 static void     *dxr3_init_plugin(xine_t *xine, void *);
@@ -68,14 +69,14 @@ static void     *dxr3_init_plugin(xine_t *xine, void *);
 /* plugin catalog information */
 static uint32_t supported_types[] = { BUF_VIDEO_MPEG, 0 };
 
-static decoder_info_t dxr3_video_decoder_info = {
+static const decoder_info_t dxr3_video_decoder_info = {
   supported_types,     /* supported types */
   10                   /* priority        */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_VIDEO_DECODER, 14, "dxr3-mpeg2", XINE_VERSION_CODE, &dxr3_video_decoder_info, &dxr3_init_plugin },
+  { PLUGIN_VIDEO_DECODER, 18, "dxr3-mpeg2", XINE_VERSION_CODE, &dxr3_video_decoder_info, &dxr3_init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
 
@@ -106,11 +107,9 @@ typedef struct dxr3_decoder_s {
   video_decoder_t        video_decoder;
   dxr3_decoder_class_t  *class;
   xine_stream_t         *stream;
-  dxr3_scr_t            *scr;
-  dxr3_driver_t         *dxr3_vo;              /* we need to talk to the dxr3 video out */
+  dxr3_scr_t            *scr;                  /* shortcut to the scr plugin in the dxr3 video out */
   
-  char                   devname[128];
-  char                   devnum[3];
+  int                    devnum;
   int                    fd_control;
   int                    fd_video;             /* to access the dxr3 devices */
   
@@ -118,16 +117,24 @@ typedef struct dxr3_decoder_s {
   int                    sequence_open;
   int                    width;
   int                    height;
-  int                    aspect;
+  double                 ratio;
+  int                    aspect_code;
   int                    frame_rate_code;
   int                    repeat_first_field;   /* mpeg stream header data */
   
-  int                    force_aspect;         /* when input plugin has better info, we are forced
-                                                * to use a certain aspect */
+  int                    force_aspect;         /* when input plugin has better info, we are forced */
+  int                    force_pan_scan;       /* to use a certain aspect or to do pan&scan */
+  
+  int                    use_panscan;
+  int                    panscan_smart_change;
+  int                    afd_smart_change;
+  int                    afd_code;             /* use pan&scan info if present in stream */
+  
   int                    last_width;
   int                    last_height;
-  int                    last_aspect;          /* used to detect changes for event sending */
+  int                    last_aspect_code;     /* used to detect changes for event sending */
   
+  unsigned int           dts_offset[3];
   int                    sync_every_frame;
   int                    sync_retry;
   int                    enhanced_mode;
@@ -141,22 +148,54 @@ typedef struct dxr3_decoder_s {
 } dxr3_decoder_t;
 
 /* helper functions */
-static int       dxr3_present(xine_stream_t *stream);
-static int       dxr3_mvcommand(int fd_control, int command);
-static void      parse_mpeg_header(dxr3_decoder_t *this, uint8_t *buffer);
-static int       get_duration(dxr3_decoder_t *this);
+static inline int  dxr3_present(xine_stream_t *stream);
+static inline int  dxr3_mvcommand(int fd_control, int command);
+static        void parse_mpeg_header(dxr3_decoder_t *this, uint8_t *buffer);
+static        int  get_duration(dxr3_decoder_t *this);
+static        void frame_format_change(dxr3_decoder_t *this);
 
 /* config callbacks */
+static void      dxr3_update_panscan(void *this_gen, xine_cfg_entry_t *entry);
 static void      dxr3_update_sync_mode(void *this_gen, xine_cfg_entry_t *entry);
 static void      dxr3_update_enhanced_mode(void *this_gen, xine_cfg_entry_t *entry);
 static void      dxr3_update_correct_durations(void *this_gen, xine_cfg_entry_t *entry);
 
+/* inline helper implementations */
+static inline int dxr3_present(xine_stream_t *stream)
+{
+  plugin_node_t *node;
+  video_driver_class_t *vo_class;
+  int present = 0;
+  
+  if (stream->video_driver && stream->video_driver->node) {
+    node = (plugin_node_t *)stream->video_driver->node;
+    if (node->plugin_class) {
+      vo_class = (video_driver_class_t *)node->plugin_class;
+      if (vo_class->get_identifier)
+        present = (strcmp(vo_class->get_identifier(vo_class), DXR3_VO_ID) == 0);
+    }
+  }
+  llprintf(LOG_VID, "dxr3 %s\n", present ? "present" : "not present");
+  return present;
+}
+
+static inline int dxr3_mvcommand(int fd_control, int command)
+{
+  em8300_register_t reg;
+  
+  reg.microcode_register = 1;
+  reg.reg = 0;
+  reg.val = command;
+  
+  return ioctl(fd_control, EM8300_IOCTL_WRITEREG, &reg);
+}
+
 
 static void *dxr3_init_plugin(xine_t *xine, void *data)
 {
   dxr3_decoder_class_t *this;
   
-  this = (dxr3_decoder_class_t *)malloc(sizeof (dxr3_decoder_class_t));
+  this = (dxr3_decoder_class_t *)xine_xmalloc(sizeof (dxr3_decoder_class_t));
   if (!this) return NULL;
   
   this->video_decoder_class.open_plugin     = dxr3_open_plugin;
@@ -174,17 +213,16 @@ static void *dxr3_init_plugin(xine_t *xine, void *data)
 
 static video_decoder_t *dxr3_open_plugin(video_decoder_class_t *class_gen, xine_stream_t *stream)
 {
+  static char *panscan_types[] = { "only when forced", "use MPEG hint", "use DVB hint", NULL };
   dxr3_decoder_t *this;
   dxr3_decoder_class_t *class = (dxr3_decoder_class_t *)class_gen;
   config_values_t *cfg;
-  const char *confstr;
-  int dashpos;
   char tmpstr[128];
   
   if (class->instance) return NULL;
   if (!dxr3_present(stream)) return NULL;
   
-  this = (dxr3_decoder_t *)malloc(sizeof (dxr3_decoder_t));
+  this = (dxr3_decoder_t *)xine_xmalloc(sizeof (dxr3_decoder_t));
   if (!this) return NULL;
   
   cfg = stream->xine->config;
@@ -198,71 +236,63 @@ static video_decoder_t *dxr3_open_plugin(video_decoder_class_t *class_gen, xine_
   this->class                       = class;
   this->stream                      = stream;
   this->scr                         = NULL;
-  this->dxr3_vo                     = (dxr3_driver_t *)stream->video_driver;
   
-  confstr = cfg->register_string(cfg, CONF_LOOKUP, CONF_DEFAULT, CONF_NAME, CONF_HELP, 0, NULL, NULL);
-  strncpy(this->devname, confstr, 128);
-  this->devname[127] = '\0';
-  dashpos = strlen(this->devname) - 2; /* the dash in the new device naming scheme would be here */
-  if (this->devname[dashpos] == '-') {
-    /* use new device naming scheme with trailing number */
-    strncpy(this->devnum, &this->devname[dashpos], 3);
-    this->devname[dashpos] = '\0';
-  } else {
-    /* use old device naming scheme without trailing number */
-    /* FIXME: remove this when everyone uses em8300 >=0.12.0 */
-    this->devnum[0] = '\0';
-  }
+  this->devnum = cfg->register_num(cfg, CONF_KEY, 0, CONF_NAME, CONF_HELP, 10, NULL, NULL);
   
-  snprintf(tmpstr, sizeof(tmpstr), "%s%s", this->devname, this->devnum);
-#if LOG_VID
-  printf("dxr3_decode_video: Entering video init, devname=%s.\n",tmpstr);
-#endif
+  snprintf(tmpstr, sizeof(tmpstr), "/dev/em8300-%d", this->devnum);
+  llprintf(LOG_VID, "Entering video init, devname=%s.\n",tmpstr);
   
   /* open later, because dxr3_video_out might have it open until we request a frame */
   this->fd_video = -1;
   
   if ((this->fd_control = open(tmpstr, O_WRONLY)) < 0) {
-    printf("dxr3_decode_video: Failed to open control device %s (%s)\n",
-      tmpstr, strerror(errno));
+    xprintf(this->stream->xine, XINE_VERBOSITY_LOG, 
+	    _("dxr3_decode_video: Failed to open control device %s (%s)\n"), tmpstr, strerror(errno));
     free(this);
     return NULL;
   }
   
-  this->have_header_info      = 0;
-  this->sequence_open         = 0;
-  this->repeat_first_field    = 0;
+  this->use_panscan           = cfg->register_enum(cfg,
+    "dxr3.use_panscan", 0, panscan_types, _("use Pan & Scan info"),
+    _("\"Pan & Scan\" is a special display mode which is sometimes used in MPEG "
+      "encoded material. You can specify here, how to handle such content.\n\n"
+      "only when forced\n"
+      "Use Pan & Scan only, when the content you are playing enforces it.\n\n"
+      "use MPEG hint\n"
+      "Enable Pan & Scan based on information embedded in the MPEG video stream.\n\n"
+      "use DVB hint\n"
+      "Enable Pan & Scan based on information embedded in DVB streams. This makes "
+      "use of the Active Format Descriptor (AFD) used in some European DVB channels."),
+    10, dxr3_update_panscan, this);
   
-  this->force_aspect          = 0;
-  
-  this->last_width            = 0;
-  this->last_height           = 0;
-  this->last_aspect           = 0;
-  
-  this->sync_retry            = 0;
-  this->resync_window         = 0;
-  this->skip_count            = 0;
+  this->dts_offset[0]         = 21600;
+  this->dts_offset[1]         = 21600;
+  this->dts_offset[2]         = 21600;
   
   this->force_duration_window = -FORCE_DURATION_WINDOW_SIZE;
   this->last_vpts             = this->class->clock->get_current_time(this->class->clock);
-  this->avg_duration          = 0;
   
   this->sync_every_frame      = cfg->register_bool(cfg,
-    "dxr3.sync_every_frame", 0, _("Try to sync video every frame"),
-    _("This is relevant for progressive video only (most PAL films)."), 20,
-    dxr3_update_sync_mode, this);
+    "dxr3.playback.sync_every_frame", 0, _("try to sync video every frame"),
+    _("Tries to set a synchronization timestamp for every frame. "
+      "Normally this is not necessary, because sync is sufficent "
+      "even when the timestamp is set only every now and then.\n"
+      "This is relevant for progressive video only (most PAL films)."),
+    20, dxr3_update_sync_mode, this);
   this->enhanced_mode         = cfg->register_bool(cfg,
-    "dxr3.alt_play_mode", 1, _("Use alternate Play mode"),
-    _("Enabling this option will utilise a smoother play mode."), 10,
-    dxr3_update_enhanced_mode, this);
+    "dxr3.playback.alt_play_mode", 1, _("use smooth play mode"),
+    _("Enabling this option will utilise a smoother play mode."),
+    20, dxr3_update_enhanced_mode, this);
   this->correct_durations     = cfg->register_bool(cfg,
-    "dxr3.correct_durations", 0, _("Correct frame durations in broken streams"),
-    _("Enable this for streams with wrong frame durations."), 10,
-    dxr3_update_correct_durations, this);
+    "dxr3.playback.correct_durations", 0, _("correct frame durations in broken streams"),
+    _("Enables a small logic that corrects the frame durations of "
+      "some mpeg streams with wrong framerate codes. Currently a "
+      "correction for NTSC streams erroneously labeled as PAL "
+      "streams is implemented. Enable only, when you encounter such streams."),
+    0, dxr3_update_correct_durations, this);
   
-  if (!this->dxr3_vo->overlay_enabled)
-    /* set a/v offset to compensate dxr3 internal delay */
-    this->stream->metronom->set_option(this->stream->metronom, METRONOM_AV_OFFSET, -21600);
+  /* the dxr3 needs a longer prebuffering to have time for its internal decoding */
+  this->stream->metronom->set_option(this->stream->metronom, METRONOM_PREBUFFER, 90000);
   
   stream->video_out->open(stream->video_out, stream);
   
@@ -302,24 +332,16 @@ static void dxr3_decode_data(video_decoder_t *this_gen, buf_element_t *buf)
   /* handle aspect hints from xine-dvdnav */
   if (buf->decoder_flags & BUF_FLAG_SPECIAL) {
     if (buf->decoder_info[1] == BUF_SPECIAL_ASPECT) {
-      this->aspect = this->force_aspect = buf->decoder_info[2];
-      if (buf->decoder_info[3] == 0x1 && this->force_aspect == XINE_VO_ASPECT_ANAMORPHIC)
-        /* letterboxing is denied, we have to do pan&scan */
-        this->aspect = this->force_aspect = XINE_VO_ASPECT_PAN_SCAN;
-      /* when aspect changed, we have to send an event for dxr3 spu decoder */
-      if (!this->last_aspect || this->last_aspect != this->aspect) {
-        xine_event_t event;
-	xine_format_change_data_t data;
-        event.type        = XINE_EVENT_FRAME_FORMAT_CHANGE;
-	event.stream      = this->stream;
-	event.data        = &data;
-	event.data_length = sizeof(data);
-        data.width        = this->last_width;
-        data.height       = this->last_height;
-        data.aspect       = this->aspect;
-        xine_event_send(this->stream, &event);
-        this->last_aspect = this->aspect;
-      }
+      this->aspect_code = this->force_aspect = buf->decoder_info[2];
+      if (buf->decoder_info[3] == 0x1 && this->force_aspect == 3)
+	/* letterboxing is denied, we have to do pan&scan */
+	this->force_pan_scan = 1;
+      else
+	this->force_pan_scan = 0;
+
+      frame_format_change(this);
+      
+      this->last_aspect_code = this->aspect_code;
     }
     return;
   }
@@ -337,27 +359,83 @@ static void dxr3_decode_data(video_decoder_t *this_gen, buf_element_t *buf)
     }
     /* header code of some kind found */
     shift = 0xffffff00;
+    
+    if (byte == 0xb2) {
+      /* check for AFD data */
+      if (buffer + 5 < buf->content + buf->size) {
+	if (buffer[0] == 0x44 && buffer[1] == 0x54 && buffer[2] == 0x47) {
+	  this->afd_code = buffer[5] & 0x0f;
+	  if (this->aspect_code == 3)
+	    /* 4:3 image in 16:9 frame -> zoomit! */
+	    this->afd_smart_change = PAN_SCAN_WINDOW_SIZE;
+	}
+      }
+      continue;
+    }
     if (byte == 0xb3) {
       /* sequence data */
-      parse_mpeg_header(this, buffer);
+      if (buffer + 3 < buf->content + buf->size)
+        parse_mpeg_header(this, buffer);
       this->sequence_open = 1;
       continue;
     }
     if (byte == 0xb5) {
       /* extension data */
-      if ((buffer[0] & 0xf0) == 0x80)
-        this->repeat_first_field = (buffer[3] >> 1) & 1;
+      /* parse the extension type and use what is necessary...
+       * types are: sequence(1), sequence_display(2), quant_matrix(3),
+       * copyright(4), picture_display(7), picture_coding(8), ... */
+      if (buffer + 4 < buf->content + buf->size) {
+        switch (buffer[0] >> 4) {
+	case 2:
+	case 7:
+	  /* picture_display and sequence_display are pan&scan info */
+	  if (this->use_panscan) this->panscan_smart_change = PAN_SCAN_WINDOW_SIZE;
+	  break;
+	case 8:
+	  this->repeat_first_field = (buffer[3] >> 1) & 1;
+	  /* clearing the progessive flag gets rid of the slight shaking with
+	   * TV-out in the lower third of the image; but we have to set this
+	   * flag, when a still frame is coming along, otherwise the card will
+	   * drop one of the fields; therefore we check for the fifo size */
+	  if (!((dxr3_driver_t *)this->stream->video_driver)->overlay_enabled) {
+	    if (this->stream->video_fifo->fifo_size > this->stream->video_fifo->buffer_pool_capacity / 2)
+	      buffer[4] &= ~(1 << 7);
+	    else
+	      buffer[4] |=  (1 << 7);
+	  }
+	  break;
+	}
+      }
       /* check if we can keep syncing */
       if (this->repeat_first_field && this->sync_retry)  /* reset counter */
         this->sync_retry = 500;
       if (this->repeat_first_field && this->sync_every_frame) {
-#if LOG_VID
-        printf("dxr3: non-progressive video detected. "
-          "disabling sync_every_frame.\n");
-#endif
+        llprintf(LOG_VID, "non-progressive video detected. disabling sync_every_frame.\n");
         this->sync_every_frame = 0;
         this->sync_retry = 500; /* see you later */
       }
+      /* check for pan&scan state */
+      if (this->use_panscan && (this->panscan_smart_change > 0 || this->afd_smart_change > 0)) {
+	this->panscan_smart_change--;
+	this->afd_smart_change--;
+	if (this->panscan_smart_change > 0 || this->afd_smart_change > 0) {
+	  /* only pan&scan if source is anamorphic */
+	  if (this->aspect_code == 3) {
+	    if (this->afd_smart_change && this->use_panscan == 2 && this->afd_code == 9)
+	      this->force_pan_scan = 1; /* panscan info available -> zoom */
+	    else if (this->afd_smart_change && this->use_panscan == 2 && this->afd_code != 9)
+	      this->force_pan_scan = 0; /* force no panscan - image is 16:9 */
+	    else if (this->use_panscan == 1 && this->panscan_smart_change)
+	      this->force_pan_scan = 1; /* panscan info available, ignore AFD mode */
+	    else if (!this->afd_smart_change && this->panscan_smart_change)
+	      this->force_pan_scan = 1;
+	    frame_format_change(this);
+	  }
+	} else {
+	  this->force_pan_scan = 0;
+	  frame_format_change(this);
+	}
+      }
       continue;
     }
     if (byte == 0xb7)
@@ -373,8 +451,8 @@ static void dxr3_decode_data(video_decoder_t *this_gen, buf_element_t *buf)
     
     /* pretend like we have decoded a frame */
     img = this->stream->video_out->get_frame(this->stream->video_out,
-      this->width, this->height, this->aspect,
-      XINE_IMGFMT_DXR3, VO_BOTH_FIELDS);
+      this->width, this->height, this->ratio,
+      XINE_IMGFMT_DXR3, VO_BOTH_FIELDS | (this->force_pan_scan ? VO_PAN_SCAN_FLAG : 0));
     img->pts       = buf->pts;
     img->bad_frame = 0;
     img->duration  = get_duration(this);
@@ -387,28 +465,25 @@ static void dxr3_decode_data(video_decoder_t *this_gen, buf_element_t *buf)
       if (this->correct_durations) {
         /* calculate an average frame duration from metronom's vpts values */
         this->avg_duration = this->avg_duration * 0.9 + (vpts - this->last_vpts) * 0.1;
-#if LOG_PTS
-        printf("dxr3_decode_video: average frame duration %d\n", this->avg_duration);
-#endif
+        llprintf(LOG_PTS, "average frame duration %d\n", this->avg_duration);
       }
       
       if (this->skip_count) this->skip_count--;
       
       if (this->resync_window == 0 && this->scr && this->enhanced_mode &&
-        !this->scr->scanning) {
+	  !this->scr->scanning) {
         /* we are in sync, so we can lock the stream now */
-#if LOG_VID
-        printf("dxr3_decode_video: in sync, stream locked\n");
-#endif
+        llprintf(LOG_VID, "in sync, stream locked\n");
         dxr3_mvcommand(this->fd_control, MVCOMMAND_SYNC);
         this->resync_window = -RESYNC_WINDOW_SIZE;
+	pthread_mutex_lock(&this->scr->mutex);
+	this->scr->sync = 1;
+	pthread_mutex_unlock(&this->scr->mutex);
       }
       if (this->resync_window != 0 && this->resync_window > -RESYNC_WINDOW_SIZE)
         this->resync_window--;
     } else { /* metronom says skip, so don't set vpts */
-#if LOG_VID
-      printf("dxr3_decode_video: %d frames to skip\n", skip);
-#endif
+      llprintf(LOG_VID, "%d frames to skip\n", skip);
       vpts = 0;
       this->avg_duration = 0;
       
@@ -426,13 +501,14 @@ static void dxr3_decode_data(video_decoder_t *this_gen, buf_element_t *buf)
       
       if (this->scr && this->scr->scanning) this->resync_window = 0;
       if (this->resync_window == 0 && this->scr && this->enhanced_mode &&
-        !this->scr->scanning) {
+	  !this->scr->scanning) {
         /* switch off sync mode in the card to allow resyncing */
-#if LOG_VID
-        printf("dxr3_decode_video: out of sync, allowing stream resync\n");
-#endif
+        llprintf(LOG_VID, "out of sync, allowing stream resync\n");
         dxr3_mvcommand(this->fd_control, MVCOMMAND_START);
         this->resync_window = RESYNC_WINDOW_SIZE;
+	pthread_mutex_lock(&this->scr->mutex);
+	this->scr->sync = 0;
+	pthread_mutex_unlock(&this->scr->mutex);
       }
       if (this->resync_window != 0 && this->resync_window < RESYNC_WINDOW_SIZE)
         this->resync_window++;
@@ -447,64 +523,67 @@ static void dxr3_decode_data(video_decoder_t *this_gen, buf_element_t *buf)
      */
     if (this->sync_retry) {
       if (!--this->sync_retry) {
-#if LOG_VID
-        printf("dxr3_decode_video: retrying sync_every_frame");
-#endif
+        llprintf(LOG_VID, "retrying sync_every_frame");
         this->sync_every_frame = 1;
       }
     }
   }
   if (buf->decoder_flags & BUF_FLAG_PREVIEW) return;
   
-  /* ensure video device is open 
+  /* ensure video device is open
    * (we open it late because on occasion the dxr3 video out driver
    * wants to open it)
+   * also ensure the scr is running
    */
   if (this->fd_video < 0) {
+    metronom_clock_t *clock = this->class->clock;
     char tmpstr[128];
-    snprintf (tmpstr, sizeof(tmpstr), "%s_mv%s", this->devname, this->devnum);
-    if ((this->fd_video = open(tmpstr, O_WRONLY | O_NONBLOCK)) < 0) {
-      printf("dxr3_decode_video: Failed to open video device %s (%s)\n",
-        tmpstr, strerror(errno)); 
-      return;
-    }
-  }
-  
-  /* We may want to issue a SETPTS, so make sure the scr plugin
-   * is running and registered. Unfortuantely wa cannot do this
-   * earlier, because the dxr3's internal scr gets confused
-   * when started with a closed video device. Maybe this is a
-   * driver bug and gets fixed somewhen. FIXME: We might then
-   * want to move this code to dxr3_init.
-   */
-  if (!this->scr) {
     int64_t time;
     
-    time = this->class->clock->get_current_time(this->class->clock);
+    /* open the device for the decoder */
+    snprintf (tmpstr, sizeof(tmpstr), "/dev/em8300_mv-%d", this->devnum);
+    if ((this->fd_video = open(tmpstr, O_WRONLY)) < 0) {
+      xprintf(this->stream->xine, XINE_VERBOSITY_LOG, 
+	      _("dxr3_decode_video: Failed to open video device %s (%s)\n"), tmpstr, strerror(errno)); 
+      return;
+    }
     
-    this->scr = dxr3_scr_init(this->stream);
+    /* We may want to issue a SETPTS, so make sure the scr plugin
+     * is running and registered. Unfortuantely wa cannot do this
+     * earlier, because the dxr3's internal scr gets confused
+     * when started with a closed video device. Maybe this is a
+     * driver bug and gets fixed somewhen. FIXME: We might then
+     * want to do this entirely in the video out.
+     */
+    this->scr = ((dxr3_driver_t *)this->stream->video_driver)->class->scr;
+    time = clock->get_current_time(clock);
     this->scr->scr_plugin.start(&this->scr->scr_plugin, time);
-    this->class->clock->register_scr(
-      this->class->clock, &this->scr->scr_plugin);
-    if (this->class->clock->scr_master == &this->scr->scr_plugin)
-#if LOG_VID
-      printf("dxr3_decode_video: dxr3_scr plugin is master\n");
-#endif
-    else
-#if LOG_VID
-      printf("dxr3_decode_video: dxr3scr plugin is NOT master\n");
-#endif
+    clock->register_scr(clock, &this->scr->scr_plugin);
   }
   
   /* update the pts timestamp in the card, which tags the data we write to it */
   if (vpts) {
     int64_t delay;
     
+    /* The PTS values written to the DXR3 must be modified based on the difference
+     * between stream's PTS and DTS (decoder timestamp). We receive this
+     * difference via decoder_info */
+    buf->decoder_info[0] <<= 1;
+    if (buf->pts) {
+      if ((this->dts_offset[0] == buf->decoder_info[0]) &&
+	  (this->dts_offset[1] == buf->decoder_info[0]))
+	this->dts_offset[2] = buf->decoder_info[0];
+      else {
+	this->dts_offset[1] = this->dts_offset[0];
+	this->dts_offset[0] = buf->decoder_info[0];
+      }
+      llprintf(LOG_PTS, "PTS to DTS correction: %d\n", this->dts_offset[1]);
+    }
+    vpts -= this->dts_offset[2];
+    
     delay = vpts - this->class->clock->get_current_time(
       this->class->clock);
-#if LOG_PTS
-    printf("dxr3_decode_video: SETPTS got %lld\n", vpts);
-#endif
+    llprintf(LOG_PTS, "SETPTS got %" PRId64 "\n", vpts);
     /* SETPTS only if less then one second in the future and
      * either buffer has pts or sync_every_frame is set */
     if ((delay > 0) && (delay < 90000) &&
@@ -512,20 +591,19 @@ static void dxr3_decode_data(video_decoder_t *this_gen, buf_element_t *buf)
       uint32_t vpts32 = vpts;
       /* update the dxr3's current pts value */
       if (ioctl(this->fd_video, EM8300_IOCTL_VIDEO_SETPTS, &vpts32))
-        printf("dxr3_decode_video: set video pts failed (%s)\n",
-          strerror(errno));
+        xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
+		"dxr3_decode_video: set video pts failed (%s)\n", strerror(errno));
     }
+    
     if (delay >= 90000)   /* frame more than 1 sec ahead */
-      printf("dxr3_decode_video: WARNING: vpts %lld is %.02f seconds ahead of time!\n",
-        vpts, delay/90000.0); 
+      xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
+	      "dxr3_decode_video: WARNING: vpts %" PRId64 " is %.02f seconds ahead of time!\n",
+	      vpts, delay/90000.0);
     if (delay < 0)
-      printf("dxr3_decode_video: WARNING: overdue frame.\n");
+      xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "dxr3_decode_video: WARNING: overdue frame.\n");
   }
-#if LOG_PTS
-  else if (buf->pts) {
-    printf("dxr3_decode_video: skip buf->pts = %lld (no vpts)\n", buf->pts);
-  }
-#endif
+  else if (buf->pts)
+    llprintf(LOG_PTS, "skip buf->pts = %" PRId64 " (no vpts)\n", buf->pts);
   
   /* now write the content to the dxr3 mpeg device and, in a dramatic
    * break with open source tradition, check the return value
@@ -533,17 +611,18 @@ static void dxr3_decode_data(video_decoder_t *this_gen, buf_element_t *buf)
   written = write(this->fd_video, buf->content, buf->size);
   if (written < 0) {
     if (errno == EAGAIN) {
-      printf("dxr3_decode_video: write to device would block. flushing\n");
+      xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+	      _("dxr3_decode_video: write to device would block. flushing\n"));
       dxr3_flush(this_gen);
     } else {
-      printf("dxr3_decode_video: video device write failed (%s)\n",
-        strerror(errno));
+      xprintf(this->stream->xine, XINE_VERBOSITY_LOG, 
+	      _("dxr3_decode_video: video device write failed (%s)\n"), strerror(errno));
     }
     return;
   }
   if (written != buf->size)
-    printf("dxr3_decode_video: Could only write %d of %d video bytes.\n",
-      written, buf->size);
+    xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
+	    "dxr3_decode_video: Could only write %zd of %d video bytes.\n", written, buf->size);
 }
 
 static void dxr3_reset(video_decoder_t *this_gen)
@@ -561,8 +640,8 @@ static void dxr3_flush(video_decoder_t *this_gen)
 {
   dxr3_decoder_t *this = (dxr3_decoder_t *)this_gen;
   
-  if (this->sequence_open &&
-      this->stream->stream_info[XINE_STREAM_INFO_VIDEO_HAS_STILL]) {
+  if (this->sequence_open && ++this->sequence_open > 5 &&
+      _x_stream_info_get(this->stream, XINE_STREAM_INFO_VIDEO_HAS_STILL)) {
     /* The dxr3 needs a sequence end code for still menus to work correctly
      * (the highlights won't move without), but some dvds have stills
      * with no sequence end code. Since it is very likely that flush() is called
@@ -570,7 +649,7 @@ static void dxr3_flush(video_decoder_t *this_gen)
     static uint8_t end_buffer[4] = { 0x00, 0x00, 0x01, 0xb7 };
     write(this->fd_video, &end_buffer, 4);
     this->sequence_open = 0;
-    printf("dxr3_decode_video: WARNING: added missing end sequence\n");
+    xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "dxr3_decode_video: WARNING: added missing end sequence\n");
   }
 }
 
@@ -579,10 +658,8 @@ static void dxr3_dispose(video_decoder_t *this_gen)
   dxr3_decoder_t *this = (dxr3_decoder_t *)this_gen;
   metronom_clock_t *clock = this->class->clock;
   
-  if (this->scr) {
+  if (this->scr)
     clock->unregister_scr(clock, &this->scr->scr_plugin);
-    this->scr->scr_plugin.exit(&this->scr->scr_plugin);
-  }
   
   dxr3_mvcommand(this->fd_control, MVCOMMAND_FLUSHBUF);
   
@@ -596,89 +673,30 @@ static void dxr3_dispose(video_decoder_t *this_gen)
 }
 
 
-static int dxr3_present(xine_stream_t *stream)
-{
-  plugin_node_t *node;
-  video_driver_class_t *vo_class;
-  int present = 0;
-  
-  if (stream->video_driver && stream->video_driver->node) {
-    node = (plugin_node_t *)stream->video_driver->node;
-    if (node->plugin_class) {
-      vo_class = (video_driver_class_t *)node->plugin_class;
-      if (vo_class->get_identifier)
-        present = (strcmp(vo_class->get_identifier(vo_class), DXR3_VO_ID) == 0);
-    }
-  }
-#ifdef LOG_VID
-  printf("dxr3_decode_video: dxr3 %s\n", present ? "present" : "not present");
-#endif
-  return present;
-}
-
-static int dxr3_mvcommand(int fd_control, int command)
-{
-  em8300_register_t regs;
-  
-  regs.microcode_register = 1;
-  regs.reg = 0;
-  regs.val = command;
-  
-  return ioctl(fd_control, EM8300_IOCTL_WRITEREG, &regs);
-}
-
 static void parse_mpeg_header(dxr3_decoder_t *this, uint8_t * buffer)
 {
-  this->frame_rate_code = buffer[HEADER_OFFSET+3] & 15;
-  this->height          = (buffer[HEADER_OFFSET+0] << 16) |
-                          (buffer[HEADER_OFFSET+1] <<  8) |
-                          buffer[HEADER_OFFSET+2];
+  this->frame_rate_code = buffer[3] & 15;
+  this->height          = (buffer[0] << 16) |
+                          (buffer[1] <<  8) |
+                          (buffer[2] <<  0);
   this->width           = ((this->height >> 12) + 15) & ~15;
   this->height          = ((this->height & 0xfff) + 15) & ~15;
-  this->aspect          = buffer[HEADER_OFFSET+3] >> 4;
+  this->aspect_code     = buffer[3] >> 4;
   
   this->have_header_info = 1;
   
-  if (this->force_aspect) this->aspect = this->force_aspect;
+  if (this->force_aspect) this->aspect_code = this->force_aspect;
   
   /* when width, height or aspect changes,
    * we have to send an event for dxr3 spu decoder */
-  if (!this->last_width || !this->last_height || !this->last_aspect ||
+  if (!this->last_width || !this->last_height || !this->last_aspect_code ||
       (this->last_width != this->width) ||
       (this->last_height != this->height) ||
-      (this->last_aspect != this->aspect)) {
-    xine_event_t event;
-    xine_format_change_data_t data;
-    event.type        = XINE_EVENT_FRAME_FORMAT_CHANGE;
-    event.stream      = this->stream;
-    event.data        = &data;
-    event.data_length = sizeof(data);
-    data.width        = this->width;
-    data.height       = this->height;
-    data.aspect       = this->aspect;
-    xine_event_send(this->stream, &event);
+      (this->last_aspect_code != this->aspect_code)) {
+    frame_format_change(this);
     this->last_width = this->width;
     this->last_height = this->height;
-    this->last_aspect = this->aspect;
-    
-    /* update stream metadata */
-    this->stream->stream_info[XINE_STREAM_INFO_VIDEO_WIDTH]  = this->width;
-    this->stream->stream_info[XINE_STREAM_INFO_VIDEO_HEIGHT] = this->height;
-    switch (this->aspect) {
-    case XINE_VO_ASPECT_SQUARE:
-      this->stream->stream_info[XINE_STREAM_INFO_VIDEO_RATIO] = 10000;
-      break;
-    case XINE_VO_ASPECT_4_3:
-      this->stream->stream_info[XINE_STREAM_INFO_VIDEO_RATIO] = 10000 * 4.0 / 3.0;
-      break;
-    case XINE_VO_ASPECT_PAN_SCAN:
-    case XINE_VO_ASPECT_ANAMORPHIC:
-      this->stream->stream_info[XINE_STREAM_INFO_VIDEO_RATIO] = 10000 * 16.0 / 9.0;
-      break;
-    case XINE_VO_ASPECT_DVB:
-      this->stream->stream_info[XINE_STREAM_INFO_VIDEO_RATIO] = 10000 * 2.11;
-      break;
-    }
+    this->last_aspect_code = this->aspect_code;
   }
 }
 
@@ -688,7 +706,7 @@ static int get_duration(dxr3_decoder_t *this)
   
   switch (this->frame_rate_code) {
   case 1: /* 23.976 */
-    duration = 3913;
+    duration = 3754;  /* actually it's 3753.75 */
     break;
   case 2: /* 24.000 */
     duration = 3750;
@@ -706,22 +724,22 @@ static int get_duration(dxr3_decoder_t *this)
     duration = 1800;
     break;
   case 7: /* 59.940 */
-    duration = 1525;
+    duration = 1502;  /* actually it's 1501.5 */
     break;
   case 8: /* 60.000 */
-    duration = 1509;
+    duration = 1500;
     break;
   default:
-    printf("dxr3_decode_video: WARNING: unknown frame rate code %d: using PAL\n",
-      this->frame_rate_code);
-    duration = 3600;  /* PAL 25fps */
+    xprintf(this->stream->xine, XINE_VERBOSITY_LOG, 
+	    _("dxr3_decode_video: WARNING: unknown frame rate code %d\n"), this->frame_rate_code);
+    duration = 0;
     break;
   }
   
   /* update stream metadata */
-  this->stream->stream_info[XINE_STREAM_INFO_FRAME_DURATION] = duration;
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_FRAME_DURATION, duration);
   
-  if (this->correct_durations) {
+  if (this->correct_durations && duration) {
     /* we set an initial average frame duration here */
     if (!this->avg_duration) this->avg_duration = duration;
   
@@ -740,7 +758,8 @@ static int get_duration(dxr3_decoder_t *this)
         /* we just entered a force_duration window, so we start the correction */
         metronom_t *metronom = this->stream->metronom;
         int64_t cur_offset;
-        printf("dxr3_decode_video: WARNING: correcting frame rate code from PAL to NTSC\n");
+        xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+		_("dxr3_decode_video: WARNING: correcting frame rate code from PAL to NTSC\n"));
         /* those weird streams need an offset, too */
         cur_offset = metronom->get_option(metronom, METRONOM_AV_OFFSET);
         metronom->set_option(metronom, METRONOM_AV_OFFSET, cur_offset - 28800);
@@ -765,23 +784,62 @@ static int get_duration(dxr3_decoder_t *this)
   return duration;
 }
 
+static void frame_format_change(dxr3_decoder_t *this)
+{
+  /* inform the dxr3 SPU decoder about the current format,
+   * so that it can choose the correctly matching SPU */
+  xine_event_t event;
+  xine_format_change_data_t data;
+  event.type        = XINE_EVENT_FRAME_FORMAT_CHANGE;
+  event.stream      = this->stream;
+  event.data        = &data;
+  event.data_length = sizeof(data);
+  data.width        = this->width;
+  data.height       = this->height;
+  data.aspect       = this->aspect_code;
+  data.pan_scan     = this->force_pan_scan;
+  xine_event_send(this->stream, &event);
+  
+  /* update ratio */
+  switch (this->aspect_code) {
+  case 2:
+    this->ratio = 4.0 / 3.0;
+    break;
+  case 3:
+    this->ratio = 16.0 / 9.0;
+    break;
+  case 4:
+    this->ratio = 2.11;
+    break;
+  default:
+    if (this->have_header_info)
+      this->ratio = (double)this->width / (double)this->height;
+  }
+  
+  /* update stream metadata */
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_WIDTH,  this->width);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_HEIGHT, this->height);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_RATIO,  10000 * this->ratio);
+  
+  _x_meta_info_set_utf8(this->stream, XINE_META_INFO_VIDEOCODEC, "MPEG (DXR3)");
+}
+
+static void dxr3_update_panscan(void *this_gen, xine_cfg_entry_t *entry)
+{
+  ((dxr3_decoder_t *)this_gen)->use_panscan = entry->num_value;
+}
+
 static void dxr3_update_sync_mode(void *this_gen, xine_cfg_entry_t *entry)
 {
   ((dxr3_decoder_t *)this_gen)->sync_every_frame = entry->num_value;
-  printf("dxr3_decode_video: setting sync_every_frame to %s\n", 
-    (entry->num_value ? "on" : "off"));
 }
 
 static void dxr3_update_enhanced_mode(void *this_gen, xine_cfg_entry_t *entry)
 {
   ((dxr3_decoder_t *)this_gen)->enhanced_mode = entry->num_value;
-  printf("dxr3_decode_video: setting enhanced mode to %s\n", 
-    (entry->num_value ? "on" : "off"));
 }
 
 static void dxr3_update_correct_durations(void *this_gen, xine_cfg_entry_t *entry)
 {
   ((dxr3_decoder_t *)this_gen)->correct_durations = entry->num_value;
-  printf("dxr3_decode_video: setting correct_durations mode to %s\n", 
-    (entry->num_value ? "on" : "off"));
 }
