@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2005 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -20,7 +20,7 @@
  * Compact Disc Digital Audio (CDDA) Input Plugin 
  *   by Mike Melanson (melanson@pcisys.net)
  *
- * $Id: input_cdda.c,v 1.69.2.2 2005/07/17 21:49:58 miguelfreitas Exp $
+ * $Id: input_cdda.c,v 1.88 2006/07/08 14:08:48 dsalt Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,7 +32,12 @@
 #include <unistd.h>
 
 #include <sys/types.h>
+#ifdef HAVE_SYS_PARAM_H
+#include <sys/param.h>
+#endif
+#ifdef HAVE_DIRENT_H
 #include <dirent.h>
+#endif
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -74,7 +79,7 @@
 #define CDDB_SERVER             "freedb.freedb.org"
 #define CDDB_PORT               8880
 #define CDDB_PROTOCOL           6
-#define CDDB_TIMEOUT            2000
+#define CDDB_TIMEOUT            5000
 
 /* CD-relevant defines and data structures */
 #define CD_SECONDS_PER_MINUTE   60
@@ -106,7 +111,7 @@ typedef struct _cdrom_toc {
  *************************************************************************/
 
 #define MAX_TRACKS     99
-#define CACHED_FRAMES  100
+#define CACHED_FRAMES  500
 
 typedef struct {
   int                  start;
@@ -684,18 +689,26 @@ static int read_cdrom_frames(cdda_input_plugin_t *this_gen, int frame, int num_f
   unsigned char *data) {
 
   int fd = this_gen->fd;
+#if  __FreeBSD_version < 501106
   struct ioc_read_audio cdda;
+#endif
 
   while( num_frames ) {
+#if  __FreeBSD_version < 501106
     cdda.address_format = CD_MSF_FORMAT;
     cdda.address.msf.minute = frame / CD_SECONDS_PER_MINUTE / CD_FRAMES_PER_SECOND;
     cdda.address.msf.second = (frame / CD_FRAMES_PER_SECOND) % CD_SECONDS_PER_MINUTE;
     cdda.address.msf.frame = frame % CD_FRAMES_PER_SECOND;
     cdda.nframes = 1;
     cdda.buffer = data;
+#endif
 
+#if  __FreeBSD_version >= 501106
+    if (pread(fd, data, CD_RAW_FRAME_SIZE, frame * CD_RAW_FRAME_SIZE) != CD_RAW_FRAME_SIZE) {
+#else
     /* read a frame */
     if(ioctl(fd, CDIOCREADAUDIO, &cdda) < 0) {
+#endif
       perror("CDIOCREADAUDIO");
       return -1;
     }
@@ -895,7 +908,11 @@ static int parse_url (char *urlbuf, char** host, int *port) {
 }
 #endif
 
-static int network_command( xine_stream_t *stream, int socket, char *data_buf, char *msg, ...)
+static int
+#ifdef __GNUC__
+__attribute__((format (printf, 4, 5)))
+#endif
+network_command( xine_stream_t *stream, int socket, char *data_buf, char *msg, ...)
 {
   char     buf[_BUFSIZ];
   va_list  args;
@@ -1380,60 +1397,90 @@ static int _cdda_load_cached_cddb_infos(cdda_input_plugin_t *this) {
 	  return 0;
 	}
 	else {
-	  char buffer[256], *ln, *pt;
+	  char buffer[256], *ln;
 	  char buf[256];
 	  int  tnum;
+	  char *dtitle = NULL;
 	  
-	  while((ln = fgets(buffer, 255, fd)) != NULL) {
+	  while ((ln = fgets(buffer, 255, fd)) != NULL) {
 
 	    buffer[strlen(buffer) - 1] = '\0';
 	    
-	    if(sscanf(buffer, "DTITLE=%s", &buf[0]) == 1) {
-	      char *artist, *title;
+	    if (sscanf(buffer, "DTITLE=%s", &buf[0]) == 1) {
+	      char *pt, *artist, *title;
 
 	      pt = strrchr(buffer, '=');
-	      if(pt)
+	      if (pt) {
 		pt++;
 
-	      artist = pt;
-	      title = strchr(pt, '/');
-	      if(title) {
-		*title++ = 0;
-	      }
-	      else {
-		title = artist;
-		artist = NULL;
-	      }
+	        if (dtitle != NULL)
+	        {
+		  dtitle = (char *) realloc(dtitle, strlen(dtitle) + strlen(pt) + 1);
+		  strcat(dtitle, pt);
+		  pt = dtitle;
+	        }
+	        dtitle = strdup(pt);
+ 
+	        artist = pt;
+	        title = strstr(pt, " / ");
+	        if (title) {
+		  *title++ = '\0';
+		  title += 2;
+	        }
+	        else {
+		  title = artist;
+		  artist = NULL;
+	        }
 
-	      if(artist)
-		this->cddb.disc_artist = strdup(artist);
+	        if (artist)
+		  this->cddb.disc_artist = strdup(artist);
 
-	      this->cddb.disc_title = strdup(title);
+	        this->cddb.disc_title = strdup(title);
+	      }
 	    }
-	    else if(sscanf(buffer, "TTITLE%d=%s", &tnum, &buf[0]) == 2) {
+	    else if (sscanf(buffer, "DYEAR=%s", &buf[0]) == 1) {
+	      char *pt;
+
 	      pt = strrchr(buffer, '=');
-	      if(pt)
+	      pt++;
+	      if (pt != NULL && strlen(pt) == 4)
+		this->cddb.disc_year = strdup(pt);
+	    }
+	    else if (sscanf(buffer, "TTITLE%d=%s", &tnum, &buf[0]) == 2) {
+	      char *pt;
+
+	      pt = strrchr(buffer, '=');
+	      if (pt)
 		pt++;
-	      this->cddb.track[tnum].title = strdup(pt);
+	      if (this->cddb.track[tnum].title == NULL)
+		this->cddb.track[tnum].title = strdup(pt); 
+	      else
+	      {
+		this->cddb.track[tnum].title
+		  = (char *) realloc(this->cddb.track[tnum].title, strlen(this->cddb.track[tnum].title) + strlen(pt) + 1);
+		strcat(this->cddb.track[tnum].title, pt);
+	      }
 	    }
 	    else {
-	      if(!strncmp(buffer, "EXTD=", 5)) {
+	      if (!strncmp(buffer, "EXTD=", 5)) {
 		char *y;
 		int   nyear;
 		
 		y = strstr(buffer, "YEAR:");
 		if(y) {
-		  if(sscanf(y+5, "%4d", &nyear) == 1) {
+		  if (sscanf(y+5, "%4d", &nyear) == 1) {
 		    char year[5];
 
 		    snprintf(year, 5, "%d", nyear);
-		    this->cddb.disc_year = strdup(year);
+		    if (this->cddb.disc_year == NULL)
+		      this->cddb.disc_year = strdup(year);
 		  }
 		}
 	      }
 	    }
 	  }
 	  fclose(fd);
+	  free(dtitle);
 	}
 	
 	closedir(dir);
@@ -1473,7 +1520,7 @@ static void _cdda_save_cached_cddb_infos(cdda_input_plugin_t *this, char *fileco
     return;
   }
   else {
-    fprintf(fd, filecontent);
+    fprintf(fd, "%s", filecontent);
     fclose(fd);
   }
   
@@ -1518,6 +1565,7 @@ static void _cdda_cddb_socket_close(cdda_input_plugin_t *this) {
 static int _cdda_cddb_retrieve(cdda_input_plugin_t *this) {
   cdda_input_class_t *this_class = (cdda_input_class_t *)this->class;
   char buffer[2048], buffercache[32768], *m, *p;
+  char *dtitle = NULL;
   int err, i;
 
   if(this == NULL) {
@@ -1691,7 +1739,7 @@ static int _cdda_cddb_retrieve(cdda_input_plugin_t *this) {
 			
     this->cddb.have_cddb_info = 1;
     memset(&buffercache, 0, sizeof(buffercache));
-						
+
     while (strcmp(buffer, ".")) {
       char buf[2048];
       int tnum;
@@ -1706,10 +1754,21 @@ static int _cdda_cddb_retrieve(cdda_input_plugin_t *this) {
 
         pt = strrchr(buffer, '=');
         if (pt) {
-          artist = pt+1;
-          title = strchr(pt+1, '/');
+          pt++;
+
+          if (dtitle != NULL)
+          {
+            dtitle = (char *) realloc(dtitle, strlen(dtitle) + strlen(pt) + 1);
+            strcat(dtitle, pt);
+            pt = dtitle;
+          }
+          dtitle = strdup(pt);
+
+          artist = pt;
+          title = strstr(pt, " / ");
           if (title) {
-            *title++ = 0;
+            *title++ = '\0';
+            title += 2;
           }
           else {
             title = artist;
@@ -1722,12 +1781,28 @@ static int _cdda_cddb_retrieve(cdda_input_plugin_t *this) {
           this->cddb.disc_title = strdup(title);
         }
       }
+      else if(sscanf(buffer, "DYEAR=%s", &buf[0]) == 1) {
+        char *pt;
+
+        pt = strrchr(buffer, '=');
+        pt++;
+        if (pt != NULL && strlen(pt) == 4)
+          this->cddb.disc_year = strdup(pt);
+      }
       else if (sscanf(buffer, "TTITLE%d=%s", &tnum, &buf[0]) == 2) {
         char *pt;
 
         pt = strrchr(buffer, '=');
         if (pt) {
-          this->cddb.track[tnum].title = strdup(pt+1);
+	  pt++;
+          if (this->cddb.track[tnum].title == NULL)
+            this->cddb.track[tnum].title = strdup(pt);
+          else
+          {
+            this->cddb.track[tnum].title
+              = (char *) realloc(this->cddb.track[tnum].title, strlen(this->cddb.track[tnum].title) + strlen(pt) + 1);
+            strcat(this->cddb.track[tnum].title, pt);
+          }
         }
       }
       else {
@@ -1741,13 +1816,15 @@ static int _cdda_cddb_retrieve(cdda_input_plugin_t *this) {
               char year[5];
 
               snprintf(year, 5, "%d", nyear);
-              this->cddb.disc_year = strdup(year);
+              if (this->cddb.disc_year == NULL)
+                this->cddb.disc_year = strdup(year);
             }
           }
         }
       }
     }
-
+    free(dtitle);
+    
     /* Save cddb info and close socket */
     _cdda_save_cached_cddb_infos(this, buffercache);
     _cdda_cddb_socket_close(this);
@@ -2098,11 +2175,20 @@ static uint32_t cdda_plugin_get_capabilities (input_plugin_t *this_gen) {
 
 static off_t cdda_plugin_read (input_plugin_t *this_gen, char *buf, off_t len) {
 
+  /* only allow reading in block-sized chunks */
+
+  return 0;
+}
+
+static buf_element_t *cdda_plugin_read_block (input_plugin_t *this_gen, fifo_buffer_t *fifo, 
+  off_t nlen) {
+
   cdda_input_plugin_t *this = (cdda_input_plugin_t *) this_gen;
+  buf_element_t *buf;
+  unsigned char frame_data[CD_RAW_FRAME_SIZE];
   int err = 0;
 
-  /* only allow reading in block-sized chunks */
-  if (len != CD_RAW_FRAME_SIZE)
+  if (nlen != CD_RAW_FRAME_SIZE)
     return 0;
 
   if (this->current_frame > this->last_frame)
@@ -2136,26 +2222,14 @@ static off_t cdda_plugin_read (input_plugin_t *this_gen, char *buf, off_t len) {
   if( err < 0 )
     return 0;
     
-  memcpy(buf, this->cache[this->current_frame-this->cache_first], CD_RAW_FRAME_SIZE);
+  memcpy(frame_data, this->cache[this->current_frame-this->cache_first], CD_RAW_FRAME_SIZE);
   this->current_frame++;
 
-  return CD_RAW_FRAME_SIZE;
-}
-
-static buf_element_t *cdda_plugin_read_block (input_plugin_t *this_gen, fifo_buffer_t *fifo, 
-  off_t nlen) {
-
-  buf_element_t *buf;
-
   buf = fifo->buffer_pool_alloc(fifo);
   buf->content = buf->mem;
   buf->type = BUF_DEMUX_BLOCK;
-
-  buf->size = cdda_plugin_read(this_gen, buf->content, nlen);
-  if (buf->size == 0) {
-    buf->free_buffer(buf);
-    buf = NULL;
-  }
+  buf->size = CD_RAW_FRAME_SIZE;
+  memcpy(buf->mem, frame_data, CD_RAW_FRAME_SIZE);
 
   return buf;
 }
@@ -2394,6 +2468,8 @@ static char ** cdda_class_get_autoplay_list (input_class_t *this_gen,
      * let's create a dummy instance in such a condition */
     ip = (cdda_input_plugin_t *)xine_xmalloc(sizeof(cdda_input_plugin_t));
     ip->stream = NULL;
+    ip->fd = -1;
+    ip->net_fd = -1;
   }
 
 #ifndef WIN32
@@ -2548,6 +2624,16 @@ static char *cdda_class_get_description (input_class_t *this_gen) {
 
 static void cdda_class_dispose (input_class_t *this_gen) {
   cdda_input_class_t  *this = (cdda_input_class_t *) this_gen;
+  config_values_t     *config = this->xine->config;
+
+  config->unregister_callback(config, "media.audio_cd.device");
+  config->unregister_callback(config, "media.audio_cd.use_cddb");
+  config->unregister_callback(config, "media.audio_cd.cddb_server");
+  config->unregister_callback(config, "media.audio_cd.cddb_port");
+  config->unregister_callback(config, "media.audio_cd.cddb_cachedir");
+#ifdef CDROM_SELECT_SPEED
+  config->unregister_callback(config, "media.audio_cd.drive_slowdown");
+#endif
 
   free (this->mrls);
   free (this);
@@ -2639,9 +2725,9 @@ static void *init_plugin (xine_t *xine, void *data) {
   return this;
 }
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 16, "CD", XINE_VERSION_CODE, NULL, init_plugin },
+  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 17, "CD", XINE_VERSION_CODE, NULL, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
 
