@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project, 
+ * Copyright (C) 2000-2005 the xine project, 
  *                         Rich Wareham <richwareham@users.sourceforge.net>
  * 
  * This file is part of xine, a free video player.
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_dvd.c,v 1.195.2.3 2005/07/17 21:49:58 miguelfreitas Exp $
+ * $Id: input_dvd.c,v 1.207 2006/05/03 19:46:07 dsalt Exp $
  *
  */
 
@@ -47,7 +47,6 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <dirent.h>
 
 #include <unistd.h>
 #include <fcntl.h>
@@ -896,6 +895,20 @@ static off_t dvd_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin
   return dvd_plugin_get_current_pos(this_gen);
 }
 
+static off_t dvd_plugin_seek_time (input_plugin_t *this_gen, int time_offset, int origin) {
+  dvd_input_plugin_t *this = (dvd_input_plugin_t*)this_gen;
+ 
+  trace_print("Called\n");
+  
+  if(!this || !this->dvdnav || origin != SEEK_SET) {
+    return -1;
+  }
+ 
+  dvdnav_time_search(this->dvdnav, time_offset * 90);
+  
+  return dvd_plugin_get_current_pos(this_gen);
+}
+
 static off_t dvd_plugin_get_length (input_plugin_t *this_gen) {
   dvd_input_plugin_t *this = (dvd_input_plugin_t*)this_gen;
   uint32_t pos=0;
@@ -1628,6 +1641,7 @@ static input_plugin_t *dvd_class_get_instance (input_class_t *class_gen, xine_st
   this->input_plugin.read               = dvd_plugin_read;
   this->input_plugin.read_block         = dvd_plugin_read_block;
   this->input_plugin.seek               = dvd_plugin_seek;
+  this->input_plugin.seek_time          = dvd_plugin_seek_time;
   this->input_plugin.get_current_pos    = dvd_plugin_get_current_pos;
   this->input_plugin.get_length         = dvd_plugin_get_length;
   this->input_plugin.get_blocksize      = dvd_plugin_get_blocksize;
@@ -1709,7 +1723,15 @@ static char **dvd_class_get_autoplay_list (input_class_t *this_gen,
 
 static void dvd_class_dispose(input_class_t *this_gen) {
   dvd_input_class_t *this = (dvd_input_class_t*)this_gen;
-  
+  config_values_t *config = this->xine->config;
+
+  config->unregister_callback(config, "media.dvd.device");
+  config->unregister_callback(config, "media.dvd.region");
+  config->unregister_callback(config, "media.dvd.language");
+  config->unregister_callback(config, "media.dvd.readahead");
+  config->unregister_callback(config, "media.dvd.seek_behaviour");
+  config->unregister_callback(config, "media.dvd.play_single_chapter");
+
   free(this->eject_device);
   free(this);
 }
@@ -1750,6 +1772,7 @@ static void *init_class (xine_t *xine, void *data) {
   this->input_class.eject_media        = dvd_class_eject_media;
   
   this->config                         = config;
+  this->xine                           = xine;
 
   this->ip                             = NULL;
 
@@ -1883,8 +1906,8 @@ static void *init_class (xine_t *xine, void *data) {
 }
 
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 16, "DVD", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 17, "DVD", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
