@@ -19,7 +19,7 @@
  *
  * input plugin for http network streams
  *
- * $Id: input_http.c,v 1.102.2.4 2005/07/17 21:49:59 miguelfreitas Exp $
+ * $Id: input_http.c,v 1.119 2006/06/20 01:46:41 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -286,7 +286,7 @@ static int http_plugin_read_metainf (http_input_plugin_t *this) {
   xine_ui_data_t data;
     
   /* get the length of the metadata */
-  if (_x_io_tcp_read (this->stream, this->fh, &len, 1) != 1)
+  if (_x_io_tcp_read (this->stream, this->fh, (char*)&len, 1) != 1)
     return 0;
 
   lprintf ("http_plugin_read_metainf: len=%d\n", len);
@@ -300,9 +300,14 @@ static int http_plugin_read_metainf (http_input_plugin_t *this) {
     lprintf ("http_plugin_read_metainf: %s\n", metadata_buf);
 
     /* Extract the title of the current song */
-    if ((songtitle = strstr(metadata_buf, "StreamTitle='"))) {
-      songtitle += 13; /* skip "StreamTitle='" */
-      if ((title_end = strchr(songtitle, '\''))) {
+    if ((songtitle = strstr(metadata_buf, "StreamTitle="))) {
+      char terminator[] = { ';', 0, 0 };
+      songtitle += 12; /* skip "StreamTitle=" */
+      if (*songtitle == '\'' || *songtitle == '"') {
+        terminator[0] = *songtitle++;
+        terminator[1] = ';';
+      }
+      if ((title_end = strstr(songtitle, terminator))) {
         *title_end = '\0';
         
         if ((!this->shoutcast_songtitle ||
@@ -434,7 +439,7 @@ static int resync_nsv(http_input_plugin_t *this) {
   lprintf("resyncing NSV stream\n");
   while ((pos < 3) && (read_bytes < (1024*1024))) {
   
-    if (http_plugin_read_int(this, &c, 1) != 1)
+    if (http_plugin_read_int(this, (char*)&c, 1) != 1)
       return 1;
 
     this->preview[pos] = c;
@@ -481,7 +486,7 @@ static buf_element_t *http_plugin_read_block (input_plugin_t *this_gen, fifo_buf
   buf->content = buf->mem;
   buf->type = BUF_DEMUX_BLOCK;
 
-  total_bytes = http_plugin_read (this_gen, buf->content, todo);
+  total_bytes = http_plugin_read (this_gen, (char*)buf->content, todo);
 
   if (total_bytes != todo) {
     buf->free_buffer (buf);
@@ -657,7 +662,7 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
   if (this->port == 0)
     this->port = DEFAULT_HTTP_PORT;
   
-  if ((this->user && strlen(this->user)) && use_proxy) {
+  if (this->user && strlen(this->user)) {
     if (http_plugin_basicauth (this->user, this->password, this->auth, BUFSIZE)) {
       _x_message(this->stream, XINE_MSG_CONNECTION_REFUSED, "basic auth error", NULL);
       return 0;
@@ -785,18 +790,20 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
 
       if (linenum == 1) {
         int httpver, httpsub;
-	char httpstatus[51];
+	char httpstatus[51] = { 0, };
 
-	if (sscanf(this->buf, "HTTP/%d.%d %d %50[^\015\012]", &httpver, &httpsub,
-		   &httpcode, httpstatus) != 4)	{
-	  
-	  /* icecast 1 ? */
-	  if (sscanf(this->buf, "ICY %d %50[^\015\012]", &httpcode, httpstatus) != 2)	{
+	if (
+	    (sscanf(this->buf, "HTTP/%d.%d %d %50[^\015\012]", &httpver, &httpsub,
+		    &httpcode, httpstatus) != 4) &&
+	    (sscanf(this->buf, "HTTP/%d.%d %d", &httpver, &httpsub,
+		    &httpcode) != 3) &&
+	    (sscanf(this->buf, "ICY %d %50[^\015\012]", /* icecast 1 ? */
+		    &httpcode, httpstatus) != 2)
+	   ) {
 	    _x_message(this->stream, XINE_MSG_CONNECTION_REFUSED, "invalid http answer", NULL);
 	    xine_log (this->stream->xine, XINE_LOG_MSG, 
 		      _("input_http: invalid http answer\n"));
 	    return 0;
-	  }
 	}
 
 	if (httpcode >= 300 && httpcode < 400) {
@@ -895,6 +902,12 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
 	len = 0;
     } else
       len ++;
+    if ( len >= BUFSIZE ) {
+       _x_message(this->stream, XINE_MSG_PERMISSION_ERROR, this->mrl, NULL);
+       xine_log (this->stream->xine, XINE_LOG_MSG,
+         _("input_http: buffer exhausted after %d bytes."), BUFSIZE);
+       return 0;
+    }
   }
 
   lprintf ("end of headers\n");
@@ -931,15 +944,23 @@ static input_plugin_t *http_class_get_instance (input_class_t *cls_gen, xine_str
   /* http_input_class_t  *cls = (http_input_class_t *) cls_gen;*/
   http_input_plugin_t *this;
   
-  if (strncasecmp (mrl, "http://", 7) && (strncasecmp (mrl, "unsv://", 7))) {
+  if (strncasecmp (mrl, "http://", 7) && 
+      strncasecmp (mrl, "unsv://", 7) &&
+      strncasecmp (mrl, "peercast://pls/", 15)) {
     return NULL;
   }
   this = (http_input_plugin_t *) xine_xmalloc(sizeof(http_input_plugin_t));
 
-  this->mrl     = strdup(mrl);
-  this->stream  = stream;
-  this->fh      = -1;
-  this->nbc     = nbc_init (this->stream);
+  if (!strncasecmp (mrl, "peercast://pls/", 15)) {
+    this->mrl = xine_xmalloc (30 + strlen(mrl) - 15);
+    sprintf (this->mrl, "http://127.0.0.1:7144/stream/%s", mrl+15);
+  } else {    
+    this->mrl = strdup (mrl);
+  }
+  
+  this->stream = stream;
+  this->fh     = -1;
+  this->nbc    = nbc_init (this->stream);
   
   this->input_plugin.open              = http_plugin_open;
   this->input_plugin.get_capabilities  = http_plugin_get_capabilities;
@@ -1057,8 +1078,8 @@ static void *init_class (xine_t *xine, void *data) {
  * exported plugin catalog entry
  */
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 16, "http", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 17, "http", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
