@@ -20,7 +20,7 @@
  * Read from a tcp network stream over a lan (put a tweaked mp1e encoder the
  * other end and you can watch tv anywhere in the house ..)
  *
- * $Id: input_net.c,v 1.60 2004/09/17 19:21:46 valtri Exp $
+ * $Id: input_net.c,v 1.67 2006/06/20 01:46:41 dgp85 Exp $
  *
  * how to set up mp1e for use with this plugin:
  * 
@@ -206,7 +206,6 @@ static int host_connect(const char *host, int port, xine_t *xine) {
   struct addrinfo hints, *res, *tmpaddr;
   int error;
   char strport[16];
-  int             i;
   int             s;
 	
   memset(&hints, 0, sizeof(hints));
@@ -254,7 +253,7 @@ static off_t net_plugin_read (input_plugin_t *this_gen,
   net_input_plugin_t *this = (net_input_plugin_t *) this_gen;
   off_t n, total;
 
-  lprintf("reading %d bytes...\n", len);
+  lprintf("reading %" PRIdMAX " bytes...\n", (intmax_t)len);
 
   total=0;
   if (this->curpos < this->preview_size) {
@@ -262,7 +261,7 @@ static off_t net_plugin_read (input_plugin_t *this_gen,
     if (n > (len - total))
       n = len - total;
 
-    lprintf("%lld bytes from preview (which has %lld bytes)\n", n, this->preview_size);
+    lprintf("%" PRIdMAX " bytes from preview (which has %" PRIdMAX " bytes)\n", (intmax_t)n, (intmax_t)this->preview_size);
 
     memcpy (&buf[total], &this->preview[this->curpos], n);
     this->curpos += n;
@@ -272,7 +271,7 @@ static off_t net_plugin_read (input_plugin_t *this_gen,
   if( (len-total) > 0 ) {
     n = _x_read_abort (this->stream, this->fh, &buf[total], len-total);
 
-    xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "input_net: got %" PRIiMAX " bytes (%" PRIiMAX "/%" PRIiMAX " bytes read)\n", (intmax_t)n, (intmax_t)total, (intmax_t)len);
+    xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "input_net: got %" PRIdMAX " bytes (%" PRIdMAX "/%" PRIdMAX " bytes read)\n", (intmax_t)n, (intmax_t)total, (intmax_t)len);
   
     if (n < 0) {
       _x_message(this->stream, XINE_MSG_READ_ERROR, this->host_port, NULL);
@@ -294,7 +293,7 @@ static buf_element_t *net_plugin_read_block (input_plugin_t *this_gen,
   buf->content = buf->mem;
   buf->type = BUF_DEMUX_BLOCK;
   
-  total_bytes = net_plugin_read (this_gen, buf->content, todo);
+  total_bytes = net_plugin_read (this_gen, (char*)buf->content, todo);
 
   if (total_bytes != todo) {
     buf->free_buffer (buf);
@@ -349,7 +348,7 @@ static off_t net_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin
         this->curpos = offset;
       else
         xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
-                "input_net: cannot seek back! (%" PRIiMAX " > %" PRIiMAX ")\n",
+                "input_net: cannot seek back! (%" PRIdMAX " > %" PRIdMAX ")\n",
                 (intmax_t)this->curpos, (intmax_t)offset);
 
     } else {
@@ -544,9 +543,9 @@ static void *init_class (xine_t *xine, void *data) {
  * exported plugin catalog entry
  */
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT, 15, "tcp", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT, 17, "tcp", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
 
