@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mms.c,v 1.52 2004/12/14 23:13:09 tmattern Exp $
+ * $Id: mms.c,v 1.59 2006/06/20 01:46:41 dgp85 Exp $
  *
  * MMS over TCP protocol
  *   based on work from major mms
@@ -45,7 +45,7 @@
 #include <stdlib.h>
 #include <time.h>
 
-#if defined(HAVE_ICONV) && defined(HAVE_LANGINFO_CODESET)
+#if defined(HAVE_ICONV) && defined(HAVE_NL_LANGINFO)
 #define USE_ICONV
 #include <iconv.h>
 #include <locale.h>
@@ -58,7 +58,6 @@
 /*
 #define LOG 
 */
-
 #include "xine_internal.h"
 #include "xineutils.h"
 
@@ -140,7 +139,7 @@ struct mms_s {
   int           stream_ids[ASF_MAX_NUM_STREAMS];
   int           stream_types[ASF_MAX_NUM_STREAMS];
   int           asf_packet_len;
-  uint32_t      file_len;
+  uint64_t      file_len;
   char          guid[37];
   uint32_t      bitrates[ASF_MAX_NUM_STREAMS];
   uint32_t      bitrates_pos[ASF_MAX_NUM_STREAMS];
@@ -154,8 +153,8 @@ struct mms_s {
 };
 
 
-static void mms_buffer_init (mms_buffer_t *mms_buffer, uint8_t *buffer) {
-  mms_buffer->buffer = buffer;
+static void mms_buffer_init (mms_buffer_t *mms_buffer, char *buffer) {
+  mms_buffer->buffer = (uint8_t*)buffer;
   mms_buffer->pos = 0;
 }
 
@@ -271,22 +270,25 @@ static int send_command (mms_t *this, int command,
   mms_buffer_init(&command_buffer, this->scmd);
   mms_buffer_put_32 (&command_buffer, 0x00000001);   /* start sequence */
   mms_buffer_put_32 (&command_buffer, 0xB00BFACE);   /* #-)) */
-  mms_buffer_put_32 (&command_buffer, length + 32);
+  mms_buffer_put_32 (&command_buffer, len8 * 8 + 32);
   mms_buffer_put_32 (&command_buffer, 0x20534d4d);   /* protocol type "MMS " */
   mms_buffer_put_32 (&command_buffer, len8 + 4);
   mms_buffer_put_32 (&command_buffer, this->seq_num);
   this->seq_num++;
   mms_buffer_put_32 (&command_buffer, 0x0);          /* timestamp */
   mms_buffer_put_32 (&command_buffer, 0x0);
-  mms_buffer_put_32 (&command_buffer, len8+2);
+  mms_buffer_put_32 (&command_buffer, len8 + 2);
   mms_buffer_put_32 (&command_buffer, 0x00030000 | command); /* dir | command */
   /* end of the 40 byte command header */
   
   mms_buffer_put_32 (&command_buffer, prefix1);
   mms_buffer_put_32 (&command_buffer, prefix2);
 
-  n = _x_io_tcp_write (this->stream, this->s, this->scmd, length + CMD_HEADER_LEN + CMD_PREFIX_LEN);
-  if (n != (length + CMD_HEADER_LEN + CMD_PREFIX_LEN)) {
+  if (length & 7)
+    memset(this->scmd + length + CMD_HEADER_LEN + CMD_PREFIX_LEN, 0, 8 - (length & 7));
+
+  n = _x_io_tcp_write (this->stream, this->s, this->scmd, len8 * 8 + CMD_HEADER_LEN + CMD_PREFIX_LEN);
+  if (n != (len8 * 8 + CMD_HEADER_LEN + CMD_PREFIX_LEN)) {
     return 0;
   }
 
@@ -357,14 +359,14 @@ static int get_packet_header (mms_t *this, mms_packet_header_t *header) {
   header->packet_seq     = 0;
   header->flags          = 0;
   header->packet_id_type = 0;
-  len = _x_io_tcp_read (this->stream, this->s, this->buf, 8);
+  len = _x_io_tcp_read (this->stream, this->s, (char*)this->buf, 8);
   if (len != 8)
     goto error;
     
   if (LE_32(this->buf + 4) == 0xb00bface) {
     /* command packet */
     header->flags = this->buf[3];
-    len = _x_io_tcp_read (this->stream, this->s, this->buf + 8, 4);
+    len = _x_io_tcp_read (this->stream, this->s, (char*)(this->buf + 8), 4);
     if (len != 4)
       goto error;
     
@@ -401,12 +403,12 @@ static int get_packet_command (mms_t *this, uint32_t packet_len) {
   /* always enter this loop */
   lprintf("packet_len: %d bytes\n", packet_len);
 
-  len = _x_io_tcp_read (this->stream, this->s, this->buf + 12, packet_len) ;
+  len = _x_io_tcp_read (this->stream, this->s, (char*)(this->buf + 12), packet_len) ;
   if (len != packet_len) {
     return 0;
   }
 
-  print_command (this->buf, len);
+  print_command ((char*)this->buf, len);
   
   /* check protocol type ("MMS ") */
   if (LE_32(this->buf + 12) != 0x20534D4D) {
@@ -496,7 +498,7 @@ static int get_asf_header (mms_t *this) {
       case MMS_PACKET_ASF_HEADER:
       case MMS_PACKET_ASF_PACKET:
         len = _x_io_tcp_read (this->stream, this->s,
-                              this->asf_header + this->asf_header_len, header.packet_len);
+                              (char*)(this->asf_header + this->asf_header_len), header.packet_len);
         if (len != header.packet_len) {
           xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
                   "libmms: get_header failed\n");
@@ -515,7 +517,7 @@ static int get_asf_header (mms_t *this) {
 
 static void interp_asf_header (mms_t *this) {
 
-  int i;
+  unsigned int i;
  
   this->asf_packet_len = 0;
   this->num_stream_ids = 0;
@@ -540,9 +542,9 @@ static void interp_asf_header (mms_t *this) {
       case GUID_ASF_FILE_PROPERTIES:
 
         this->asf_packet_len = LE_32(this->asf_header + i + 92 - 24);
-        this->file_len   = LE_32(this->asf_header + i + 40 - 24);
-        lprintf ("file object, packet length = %d (%d)\n",
-                 this->asf_packet_len, LE_32(this->asf_header + i + 96 - 24));
+        this->file_len       = LE_64(this->asf_header + i + 40 - 24);
+        lprintf ("file object, file_length = %lld, packet length = %d",
+		 this->file_len, this->asf_packet_len);
         break;
 
       case GUID_ASF_STREAM_PROPERTIES:
@@ -560,6 +562,8 @@ static void interp_asf_header (mms_t *this) {
               break;
     
             case GUID_ASF_VIDEO_MEDIA:
+            case GUID_ASF_JFIF_MEDIA:
+            case GUID_ASF_DEGRADABLE_JPEG_MEDIA:
               type = ASF_STREAM_TYPE_VIDEO;
               this->has_video = 1;
               break;
@@ -579,9 +583,13 @@ static void interp_asf_header (mms_t *this) {
           lprintf ("stream object, stream id: %d, type: %d, encrypted: %d\n",
                    stream_id, type, encrypted);
           
-          this->stream_types[stream_id] = type;
-          this->stream_ids[this->num_stream_ids] = stream_id;
-          this->num_stream_ids++;
+          if (this->num_stream_ids < ASF_MAX_NUM_STREAMS && stream_id < ASF_MAX_NUM_STREAMS) {
+            this->stream_types[stream_id] = type;
+            this->stream_ids[this->num_stream_ids] = stream_id;
+            this->num_stream_ids++;
+          } else {
+            lprintf ("too many streams, skipping\n");
+          }
       
         }
         break;
@@ -619,7 +627,7 @@ static void interp_asf_header (mms_t *this) {
   }
 }
 
-const static char *const mmst_proto_s[] = { "mms", "mmst", NULL };
+static const char *const mmst_proto_s[] = { "mms", "mmst", NULL };
 
 static int mmst_valid_proto (char *proto) {
   int i = 0;
@@ -702,15 +710,15 @@ static void mms_gen_guid(char guid[]) {
 /*
  * return 0 on error
  */
-int static mms_choose_best_streams(mms_t *this) {
+static int mms_choose_best_streams(mms_t *this) {
   int     i;
   int     video_stream = 0;
   int     audio_stream = 0;
-  int     max_arate    = 0;
-  int     min_vrate    = 0;
-  int     min_bw_left  = 0;
+  unsigned int max_arate = 0;
+  unsigned int min_vrate = 0;
+  unsigned int min_bw_left = 0;
   int     stream_id;
-  int     bandwitdh_left;
+  unsigned int bandwitdh_left;
   int     res;
 
   /* command 0x33 */
@@ -732,10 +740,7 @@ int static mms_choose_best_streams(mms_t *this) {
   }
   
   /* choose a video stream adapted to the user bandwidth */
-  bandwitdh_left = this->bandwidth - max_arate;
-  if (bandwitdh_left < 0) {
-    bandwitdh_left = 0;
-  }
+  bandwitdh_left = (int)( this->bandwidth - max_arate ) < 0 ? 0 : this->bandwidth - max_arate;
   lprintf("bandwitdh %d, left %d\n", this->bandwidth, bandwitdh_left);
 
   min_bw_left = bandwitdh_left;
@@ -927,13 +932,23 @@ mms_t *mms_connect (xine_stream_t *stream, const char *url, int bandwidth) {
   /* command 0x5 */
   {
     mms_buffer_t command_buffer;
+    char *path;
+    int pathlen;
+
+    /* remove the first '/' */
+    path = this->uri;
+    pathlen = strlen(path);
+    if (pathlen > 1) {
+      path++;
+      pathlen--;
+    }
     
     lprintf("send command 0x05\n");
     mms_buffer_init(&command_buffer, this->scmd_body);
     mms_buffer_put_32 (&command_buffer, 0x00000000); /* ?? */
     mms_buffer_put_32 (&command_buffer, 0x00000000); /* ?? */
-    string_utf16 (url_conv, this->scmd_body + command_buffer.pos, this->uri, strlen(this->uri));
-    if (!send_command (this, 5, 1, 0xffffffff, strlen(this->uri) * 2 + 12))
+    string_utf16 (url_conv, this->scmd_body + command_buffer.pos, path, pathlen);
+    if (!send_command (this, 5, 1, 0xffffffff, pathlen * 2 + 12))
       goto fail;
   }
   
@@ -1170,11 +1185,8 @@ static int get_media_packet (mms_t *this) {
                    "libmms: invalid asf packet len: %d bytes\n", header.packet_len);
           return 0;
         }
-    
-        /* simulate a seek */
-        this->current_pos = (off_t)this->asf_header_len + (off_t)header.packet_seq * (off_t)this->asf_packet_len;
-    
-        len = _x_io_tcp_read (this->stream, this->s, this->buf, header.packet_len);
+
+        len = _x_io_tcp_read (this->stream, this->s, (char*)this->buf, header.packet_len);
         if (len != header.packet_len) {
           xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
                    "libmms: read failed\n");
@@ -1195,9 +1207,9 @@ static int get_media_packet (mms_t *this) {
 }
 
 
-int mms_peek_header (mms_t *this, char *data, int maxsize) {
+size_t mms_peek_header (mms_t *this, char *data, size_t maxsize) {
 
-  int len;
+  size_t len;
 
   len = (this->asf_header_len < maxsize) ? this->asf_header_len : maxsize;
 
