@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mmsh.c,v 1.33 2004/12/14 23:13:09 tmattern Exp $
+ * $Id: mmsh.c,v 1.37 2006/06/20 01:46:41 dgp85 Exp $
  *
  * MMS over HTTP protocol
  *   written by Thibaut Mattern
@@ -82,61 +82,61 @@
 
 #define SCRATCH_SIZE             1024
 
-static const char* mmsh_FirstRequest =
-    "GET %s HTTP/1.0\r\n"
-    "Accept: */*\r\n"
-    USERAGENT
-    "Host: %s:%d\r\n"
-    "Pragma: no-cache,rate=1.000000,stream-time=0,stream-offset=0:0,request-context=%u,max-duration=0\r\n"
-    CLIENTGUID
-    "Connection: Close\r\n\r\n";
+#define mmsh_FirstRequest \
+    "GET %s HTTP/1.0\r\n" \
+    "Accept: */*\r\n" \
+    USERAGENT \
+    "Host: %s:%d\r\n" \
+    "Pragma: no-cache,rate=1.000000,stream-time=0,stream-offset=0:0,request-context=%u,max-duration=0\r\n" \
+    CLIENTGUID \
+    "Connection: Close\r\n\r\n"
 
-static const char* mmsh_SeekableRequest =
-    "GET %s HTTP/1.0\r\n"
-    "Accept: */*\r\n"
-    USERAGENT
-    "Host: %s:%d\r\n"
-    "Pragma: no-cache,rate=1.000000,stream-time=%u,stream-offset=%u:%u,request-context=%u,max-duration=%u\r\n"
-    CLIENTGUID
-    "Pragma: xPlayStrm=1\r\n"
-    "Pragma: stream-switch-count=%d\r\n"
-    "Pragma: stream-switch-entry=%s\r\n" /*  ffff:1:0 ffff:2:0 */
-    "Connection: Close\r\n\r\n";
+#define mmsh_SeekableRequest \
+    "GET %s HTTP/1.0\r\n" \
+    "Accept: */*\r\n" \
+    USERAGENT \
+    "Host: %s:%d\r\n" \
+    "Pragma: no-cache,rate=1.000000,stream-time=%u,stream-offset=%u:%u,request-context=%u,max-duration=%u\r\n" \
+    CLIENTGUID \
+    "Pragma: xPlayStrm=1\r\n" \
+    "Pragma: stream-switch-coun t=%d\r\n" \
+    "Pragma: stream-switch-entry=%s\r\n" /*  ffff:1:0 ffff:2:0 */ \
+    "Connection: Close\r\n\r\n"
 
-static const char* mmsh_LiveRequest =
-    "GET %s HTTP/1.0\r\n"
-    "Accept: */*\r\n"
-    USERAGENT
-    "Host: %s:%d\r\n"
-    "Pragma: no-cache,rate=1.000000,request-context=%u\r\n"
-    "Pragma: xPlayStrm=1\r\n"
-    CLIENTGUID
-    "Pragma: stream-switch-count=%d\r\n"
-    "Pragma: stream-switch-entry=%s\r\n"
-    "Connection: Close\r\n\r\n";
+#define mmsh_LiveRequest \
+    "GET %s HTTP/1.0\r\n" \
+    "Accept: */*\r\n" \
+    USERAGENT \
+    "Host: %s:%d\r\n" \
+    "Pragma: no-cache,rate=1.000000,request-context=%u\r\n" \
+    "Pragma: xPlayStrm=1\r\n" \
+    CLIENTGUID \
+    "Pragma: stream-switch-count=%d\r\n" \
+    "Pragma: stream-switch-entry=%s\r\n" \
+    "Connection: Close\r\n\r\n"
 
 /* Unused requests */
 #if 0
-static const char* mmsh_PostRequest =
-    "POST %s HTTP/1.0\r\n"
-    "Accept: */*\r\n"
-    USERAGENT
-    "Host: %s\r\n"
-    "Pragma: client-id=%u\r\n"
+#define mmsh_PostRequest \
+    "POST %s HTTP/1.0\r\n" \
+    "Accept: */*\r\n" \
+    USERAGENT \
+    "Host: %s\r\n" \
+    "Pragma: client-id=%u\r\n" \
 /*    "Pragma: log-line=no-cache,rate=1.000000,stream-time=%u,stream-offset=%u:%u,request-context=2,max-duration=%u\r\n"
- */
-    "Pragma: Content-Length: 0\r\n"
-    CLIENTGUID
-    "\r\n";
+ */ \
+    "Pragma: Content-Length: 0\r\n" \
+    CLIENTGUID \
+    "\r\n"
 
-static const char* mmsh_RangeRequest =
-    "GET %s HTTP/1.0\r\n"
-    "Accept: */*\r\n"
-    USERAGENT
-    "Host: %s:%d\r\n"
-    "Range: bytes=%Lu-\r\n"
-    CLIENTGUID
-    "Connection: Close\r\n\r\n";
+#define mmsh_RangeRequest \
+    "GET %s HTTP/1.0\r\n" \
+    "Accept: */*\r\n" \
+    USERAGENT \
+    "Host: %s:%d\r\n" \
+    "Range: bytes=%Lu-\r\n" \
+    CLIENTGUID \
+    "Connection: Close\r\n\r\n"
 #endif
 
 
@@ -183,7 +183,7 @@ struct mmsh_s {
   int           stream_ids[ASF_MAX_NUM_STREAMS];
   int           stream_types[ASF_MAX_NUM_STREAMS];
   int           packet_length;
-  uint32_t      file_length;
+  int64_t       file_length;
   char          guid[37];
   uint32_t      bitrates[ASF_MAX_NUM_STREAMS];
   uint32_t      bitrates_pos[ASF_MAX_NUM_STREAMS];
@@ -247,7 +247,7 @@ static int get_answer (mmsh_t *this) {
 
   while (!done) {
 
-    if (_x_io_tcp_read(this->stream, this->s, &(this->buf[len]), 1) != 1) {
+    if (_x_io_tcp_read(this->stream, this->s, (char*)&(this->buf[len]), 1) != 1) {
       xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
                "libmmsh: alert: end of stream\n");
       return 0;
@@ -271,7 +271,7 @@ static int get_answer (mmsh_t *this) {
         int httpver, httpsub, httpcode;
         char httpstatus[51];
 
-        if (sscanf(this->buf, "HTTP/%d.%d %d %50[^\015\012]", &httpver, &httpsub,
+        if (sscanf((char*)this->buf, "HTTP/%d.%d %d %50[^\015\012]", &httpver, &httpsub,
             &httpcode, httpstatus) != 4) {
           xine_log (this->stream->xine, XINE_LOG_MSG,
 		    _("libmmsh: bad response format\n"));
@@ -293,14 +293,14 @@ static int get_answer (mmsh_t *this) {
         }
       } else {
 
-        if (!strncasecmp(this->buf, "Location: ", 10)) {
+        if (!strncasecmp((char*)this->buf, "Location: ", 10)) {
           xine_log (this->stream->xine, XINE_LOG_MSG,
 		    _("libmmsh: Location redirection not implemented\n"));
           return 0;
         }
         
-        if (!strncasecmp(this->buf, "Pragma:", 7)) {
-          features = strstr(this->buf + 7, "features=");
+        if (!strncasecmp((char*)this->buf, "Pragma:", 7)) {
+          features = strstr((char*)(this->buf + 7), "features=");
           if (features) {
             if (strstr(features, "seekable")) {
               lprintf("seekable stream\n");
@@ -341,7 +341,7 @@ static int get_chunk_header (mmsh_t *this) {
   lprintf ("get_chunk_header\n");
 
   /* read chunk header */
-  read_len = _x_io_tcp_read(this->stream, this->s, chunk_header, CHUNK_HEADER_LENGTH);
+  read_len = _x_io_tcp_read(this->stream, this->s, (char*)chunk_header, CHUNK_HEADER_LENGTH);
   if (read_len != CHUNK_HEADER_LENGTH) {
     xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
              "libmmsh: chunk header read failed, %d != %d\n", read_len, CHUNK_HEADER_LENGTH);
@@ -368,7 +368,7 @@ static int get_chunk_header (mmsh_t *this) {
   }
   /* read extended header */
   if (ext_header_len > 0) {
-    read_len = _x_io_tcp_read(this->stream, this->s, ext_header, ext_header_len);
+    read_len = _x_io_tcp_read(this->stream, this->s, (char*)ext_header, ext_header_len);
     if (read_len != ext_header_len) {
       xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
                "extended header read failed, %d != %d\n", read_len, ext_header_len);
@@ -430,7 +430,7 @@ static int get_header (mmsh_t *this) {
                    "libmmsh: the asf header exceed %d bytes\n", ASF_HEADER_SIZE);
           return 0;
         } else {
-          len = _x_io_tcp_read(this->stream, this->s, this->asf_header + this->asf_header_len,
+          len = _x_io_tcp_read(this->stream, this->s, (char*)(this->asf_header + this->asf_header_len),
                              this->chunk_length);
           this->asf_header_len += len;
           if (len != this->chunk_length) {
@@ -448,11 +448,10 @@ static int get_header (mmsh_t *this) {
 
   if (this->chunk_type == CHUNK_TYPE_DATA) {
     /* read the first data chunk */
-    len = _x_io_tcp_read(this->stream, this->s, this->buf, this->chunk_length);
+    len = _x_io_tcp_read(this->stream, this->s, (char*)this->buf, this->chunk_length);
     if (len != this->chunk_length) {
       return 0;
     } else {
-      this->buf_size = this->packet_length;
       return 1;
     }
   } else {
@@ -463,7 +462,7 @@ static int get_header (mmsh_t *this) {
 
 static void interp_header (mmsh_t *this) {
 
-  int i;
+  unsigned int i;
 
   lprintf ("interp_header, header_len=%d\n", this->asf_header_len);
 
@@ -492,9 +491,9 @@ static void interp_header (mmsh_t *this) {
       case GUID_ASF_FILE_PROPERTIES:
 
         this->packet_length = LE_32(this->asf_header + i + 92 - 24);
-        this->file_length   = LE_32(this->asf_header + i + 40 - 24);
-        lprintf ("file object, packet length = %d (%d)\n",
-		 this->packet_length, LE_32(this->asf_header + i + 96 - 24));
+        this->file_length   = LE_64(this->asf_header + i + 40 - 24);
+        /*lprintf ("file object, file_length = %lld, packet length = %d",
+		 this->file_length, this->packet_count);*/
         break;
 
       case GUID_ASF_STREAM_PROPERTIES:
@@ -510,16 +509,18 @@ static void interp_header (mmsh_t *this) {
               type = ASF_STREAM_TYPE_AUDIO;
               this->has_audio = 1;
               break;
-
+    
             case GUID_ASF_VIDEO_MEDIA:
+            case GUID_ASF_JFIF_MEDIA:
+            case GUID_ASF_DEGRADABLE_JPEG_MEDIA:
               type = ASF_STREAM_TYPE_VIDEO;
               this->has_video = 1;
               break;
-
+          
             case GUID_ASF_COMMAND_MEDIA:
               type = ASF_STREAM_TYPE_CONTROL;
               break;
-
+        
             default:
               type = ASF_STREAM_TYPE_UNKNOWN;
           }
@@ -574,7 +575,7 @@ static void interp_header (mmsh_t *this) {
   }
 }
 
-const static char *const mmsh_proto_s[] = { "mms", "mmsh", NULL };
+static const char *const mmsh_proto_s[] = { "mms", "mmsh", NULL };
 
 static int mmsh_valid_proto (char *proto) {
   int i = 0;
@@ -793,6 +794,7 @@ static int mmsh_connect_int(mmsh_t *this, int bandwidth) {
   if (!get_header(this))
     goto fail;
   interp_header(this);
+  this->buf_size = this->packet_length;
   
   for (i = 0; i < this->num_stream_ids; i++) {
     if ((this->stream_ids[i] != audio_stream) &&
@@ -928,12 +930,15 @@ static int get_media_packet (mmsh_t *this) {
       
       case CHUNK_TYPE_RESET:
         /* next chunk is an ASF header */
+      
         if (this->chunk_length != 0) {
           /* that's strange, don't know what to do */
           return 0;
         }
         if (!get_header(this))
           return 0;
+        interp_header(this);
+        this->buf_size = this->packet_length;
         return 2;
       
       default:
@@ -942,10 +947,16 @@ static int get_media_packet (mmsh_t *this) {
         return 0;
     }
 
-    len = _x_io_tcp_read (this->stream, this->s, this->buf, this->chunk_length);
+    len = _x_io_tcp_read (this->stream, this->s, (char*)this->buf, this->chunk_length);
       
     if (len == this->chunk_length) {
       /* explicit padding with 0 */
+      if (this->chunk_length > this->packet_length) {
+        xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
+                 "libmmsh: chunk_length(%d) > packet_length(%d)\n",
+                 this->chunk_length, this->packet_length);
+        return 0;
+      }
       memset(this->buf + this->chunk_length, 0,
              this->packet_length - this->chunk_length);
       this->buf_size = this->packet_length;
@@ -960,8 +971,8 @@ static int get_media_packet (mmsh_t *this) {
   }
 }
 
-int mmsh_peek_header (mmsh_t *this, char *data, int maxsize) {
-  int len;
+size_t mmsh_peek_header (mmsh_t *this, char *data, size_t maxsize) {
+  size_t len;
 
   lprintf("mmsh_peek_header\n");
 
