@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.73.2.1 2005/07/17 21:50:01 miguelfreitas Exp $
+ * $Id: xine_decoder.c,v 1.79 2006/06/02 22:18:57 dsalt Exp $
  *
  * stuff needed to turn liba52 into a xine decoder plugin
  */
@@ -64,6 +64,8 @@
 #include "buffer.h"
 #include "xineutils.h"
 
+#include "crc.c"
+
 #undef DEBUG_A52
 #ifdef DEBUG_A52
 int a52file;
@@ -852,12 +854,12 @@ static uint32_t audio_types[] = {
   0
  };
 
-static decoder_info_t dec_info_audio = {
+static const decoder_info_t dec_info_audio = {
   audio_types,         /* supported types */
   5                    /* priority        */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_AUDIO_DECODER | PLUGIN_MUST_PRELOAD, 15, "a/52", XINE_VERSION_CODE, &dec_info_audio, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
