@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2005 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.60 2004/12/16 13:59:10 mroi Exp $
+ * $Id: xine_decoder.c,v 1.65 2006/06/02 22:18:57 dsalt Exp $
  *
  * 04-09-2001 DTS passtrough  (C) Joachim Koenig 
  * 09-12-2001 DTS passthrough inprovements (C) James Courtier-Dutton
@@ -67,9 +67,6 @@ typedef struct {
   int64_t          pts;
 
   int              audio_caps;  
-  uint32_t         rate;
-  uint32_t         bits_per_sample;
-  uint32_t         number_of_channels;
   int              sync_state;
   int              ac5_length, ac5_pcm_length, frame_todo;
   uint32_t         syncdword;
@@ -157,8 +154,7 @@ static void dts_decode_frame (dts_decoder_t *this, int64_t pts, int preview_mode
       /* SPDIF digital output */
       if (!this->output_open) {
         this->output_open = (this->stream->audio_out->open (this->stream->audio_out, this->stream,
-                                                            this->bits_per_sample, 
-                                                            this->rate,
+                                                            16, this->dts_sample_rate, 
                                                             AO_CAP_MODE_AC5));
       }
       
@@ -225,38 +221,37 @@ static void dts_decode_frame (dts_decoder_t *this, int64_t pts, int preview_mode
       }
     } else {
       /* Software decode */
-      int       i, dts_flags;
+      int       i, dts_output_flags;
       int16_t  *int_samples = audio_buffer->mem;
       int       number_of_dts_blocks;
 
       level_t   level = 1.0;
       sample_t *samples;
 
-      this->have_lfe = this->dts_flags & DTS_LFE;
-      dts_flags = this->dts_flags_map[this->dts_flags & DTS_CHANNEL_MASK];
-      if (this->have_lfe)
-        if (this->audio_caps & AO_CAP_MODE_5_1CHANNEL) {
-          output_mode = AO_CAP_MODE_5_1CHANNEL;
-          dts_flags |= DTS_LFE;
-        } else if (this->audio_caps & AO_CAP_MODE_4_1CHANNEL) {
-          output_mode = AO_CAP_MODE_4_1CHANNEL;
-          dts_flags |= DTS_LFE;
-        } else {
-          xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "libdts: WHAT DO I DO!!!\n");
-          output_mode = this->ao_flags_map[dts_flags & DTS_CHANNEL_MASK];
-        }
-      else
-        output_mode = this->ao_flags_map[dts_flags & DTS_CHANNEL_MASK];
+      dts_output_flags = this->dts_flags_map[this->dts_flags & DTS_CHANNEL_MASK];
 
-      if(dts_frame(this->dts_state, data_in, &dts_flags, &level, 0)) {
+      if(dts_frame(this->dts_state, data_in, &dts_output_flags, &level, 0)) {
         xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "libdts: dts_frame error\n");
         return;
       }
+
+      this->have_lfe = dts_output_flags & DTS_LFE;
+      if (this->have_lfe)
+        if (this->audio_caps & AO_CAP_MODE_5_1CHANNEL) {
+          output_mode = AO_CAP_MODE_5_1CHANNEL;
+        } else if (this->audio_caps & AO_CAP_MODE_4_1CHANNEL) {
+          output_mode = AO_CAP_MODE_4_1CHANNEL;
+        } else {
+          xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "libdts: WHAT DO I DO!!!\n");
+          output_mode = this->ao_flags_map[dts_output_flags & DTS_CHANNEL_MASK];
+        }
+      else
+        output_mode = this->ao_flags_map[dts_output_flags & DTS_CHANNEL_MASK];
+
       if (!this->output_open) {
-        this->output_open = (this->stream->audio_out->open (this->stream->audio_out, this->stream,
-                                                this->bits_per_sample, 
-                                                this->rate,
-                                                output_mode));
+        this->output_open = this->stream->audio_out->open (this->stream->audio_out, this->stream,
+                                                           16, this->dts_sample_rate, 
+                                                           output_mode);
       }
       
       if (!this->output_open) 
@@ -410,26 +405,34 @@ static void dts_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
                 old_dts_sample_rate != this->dts_sample_rate ||
 		old_dts_bit_rate    != this->dts_bit_rate) {
 
-              this->rate = this->dts_sample_rate;
-              if (((this->dts_flags & DTS_CHANNEL_MASK) == DTS_3F2R) && (this->dts_flags & DTS_LFE))
-                _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "DTS 5.1");
-              else if ((((this->dts_flags & DTS_CHANNEL_MASK) == DTS_2F2R) && (this->dts_flags & DTS_LFE)) ||
-                       (((this->dts_flags & DTS_CHANNEL_MASK) == DTS_3F1R) && (this->dts_flags & DTS_LFE)))
-                _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "DTS 4.1");
-              else if ((this->dts_flags & DTS_CHANNEL_MASK) == DTS_3F2R) 
-                _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "DTS 5.0");
-              else if (((this->dts_flags & DTS_CHANNEL_MASK) == DTS_2F2R) ||
-                       ((this->dts_flags & DTS_CHANNEL_MASK) == DTS_3F1R))
-                _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "DTS 4.0");
-              else if (((this->dts_flags & DTS_CHANNEL_MASK) == DTS_2F1R) ||
-                       ((this->dts_flags & DTS_CHANNEL_MASK) == DTS_3F))
-                _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "DTS 3.0");
-              else if ((this->dts_flags & DTS_CHANNEL_MASK) == DTS_STEREO)
-                _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "DTS 2.0 (stereo)");
-              else if ((this->dts_flags & DTS_CHANNEL_MASK) == DTS_MONO)
-                _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "DTS 1.0");
-              else
-                _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "DTS");
+              switch (this->dts_flags & DTS_CHANNEL_MASK) {
+                case DTS_3F2R:
+                  if (this->dts_flags & DTS_LFE)
+                    _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "DTS 5.1");
+                  else
+                    _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "DTS 5.0");
+                  break;
+                case DTS_3F1R:
+                case DTS_2F2R:
+                  if (this->dts_flags & DTS_LFE)
+                    _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "DTS 4.1");
+                  else
+                    _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "DTS 4.0");
+                  break;
+                case DTS_2F1R:
+                case DTS_3F:
+                  _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "DTS 3.0");
+                  break;
+                case DTS_STEREO:
+                  _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "DTS 2.0 (stereo)");
+                  break;
+                case DTS_MONO:
+                  _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "DTS 1.0");
+                  break;
+                default:
+                  _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "DTS");
+                  break;
+              }
 
               _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_BITRATE, this->dts_bit_rate);
               _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_SAMPLERATE, this->dts_sample_rate);
@@ -562,9 +565,7 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
   this->stream        = stream;
   this->class         = class_gen;
   this->output_open   = 0;
-  this->rate          = 48000;
-  this->bits_per_sample=16;
-  this->number_of_channels=2;
+
   return &this->audio_decoder;
 }
 
@@ -601,12 +602,12 @@ static uint32_t audio_types[] = {
   BUF_AUDIO_DTS, 0
  };
 
-static decoder_info_t dec_info_audio = {
+static const decoder_info_t dec_info_audio = {
   audio_types,         /* supported types */
   1                    /* priority        */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_AUDIO_DECODER, 15, "dts", XINE_VERSION_CODE, &dec_info_audio, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
