@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -38,13 +38,17 @@
 
 #include <FLAC/seekable_stream_decoder.h>
 
+#define LOG_MODULE "demux_flac"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
+
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "../demuxers/demux.h"
 
-/*
-#define LOG 1
-*/
+#include "demux_flac.h"
 
 /* FLAC Demuxer plugin */
 typedef struct demux_flac_s {
@@ -95,9 +99,7 @@ flac_read_callback (const FLAC__SeekableStreamDecoder *decoder,
     input_plugin_t *input = this->input; 
     off_t offset = *bytes;
 
-#ifdef LOG
-    printf("FLAC_DMXR: flac_read_callback\n");
-#endif
+    lprintf("flac_read_callback\n");
     
     /* This should only be called when flac is reading the metadata
      * of the flac stream.
@@ -105,26 +107,23 @@ flac_read_callback (const FLAC__SeekableStreamDecoder *decoder,
     
     offset = input->read (input, buffer, offset);
 
-#ifdef LOG
-    printf("FLAC_DMXR: Read %lld / %u bytes into buffer\n", offset, *bytes);
-#endif
+    lprintf("Read %lld / %u bytes into buffer\n", offset, *bytes);
 
-    *bytes = offset;
     /* This is the way to detect EOF with xine input plugins */
-    if ( (offset != *bytes) && (*bytes != 0) )
+    if ( offset <= 0 && *bytes != 0 )
     {
-#ifdef LOG
-        printf("FLAC_DMXR: Marking EOF\n");
-#endif
-        this->status = DEMUX_FINISHED;
-        return FLAC__SEEKABLE_STREAM_DECODER_READ_STATUS_ERROR;
+      *bytes = offset;
+      lprintf("Marking EOF\n");
+      
+      this->status = DEMUX_FINISHED;
+      return FLAC__SEEKABLE_STREAM_DECODER_READ_STATUS_ERROR;
     }
     else
     {
-#ifdef LOG
-        printf("FLAC_DMXR: Read was perfect\n");
-#endif
-        return FLAC__SEEKABLE_STREAM_DECODER_READ_STATUS_OK;
+      *bytes = offset;
+      lprintf("Read was perfect\n");
+    
+      return FLAC__SEEKABLE_STREAM_DECODER_READ_STATUS_OK;
     }
 }
 
@@ -136,9 +135,7 @@ flac_seek_callback (const FLAC__SeekableStreamDecoder *decoder,
     input_plugin_t *input = ((demux_flac_t *)client_data)->input;
     off_t offset;
 
-#ifdef LOG
-    printf("FLAC_DMXR: flac_seek_callback\n");
-#endif
+    lprintf("flac_seek_callback\n");
 
     offset = input->seek (input, absolute_byte_offset, SEEK_SET);
 
@@ -156,9 +153,8 @@ flac_tell_callback (const FLAC__SeekableStreamDecoder *decoder,
     input_plugin_t *input = ((demux_flac_t *)client_data)->input;
     off_t offset;
 
-#ifdef LOG
-    printf("FLAC_DMXR: flac_tell_callback\n");
-#endif
+    lprintf("flac_tell_callback\n");
+
     offset = input->get_current_pos (input);
 
     *absolute_byte_offset = offset;
@@ -174,9 +170,8 @@ flac_length_callback (const FLAC__SeekableStreamDecoder *decoder,
     input_plugin_t *input = ((demux_flac_t *)client_data)->input;
     off_t offset;
 
-#ifdef LOG
-    printf("FLAC_DMXR: flac_length_callback\n");
-#endif
+    lprintf("flac_length_callback\n");
+
     offset = input->get_length (input);
 
     /* FIXME, can flac handle -1 as offset ? */
@@ -188,23 +183,20 @@ flac_eof_callback (const FLAC__SeekableStreamDecoder *decoder,
                     void *client_data)
 {
     demux_flac_t *this = (demux_flac_t *)client_data;
-#ifdef LOG
-    printf("FLAC_DMXR: flac_eof_callback\n");
-#endif
+
+    lprintf("flac_eof_callback\n");
 
     if (this->status == DEMUX_FINISHED)
     {
-#ifdef LOG
-        printf("FLAC_DMXR: flac_eof_callback: True!\n");
-#endif
-        return true;
+      lprintf("flac_eof_callback: True!\n");
+
+      return true;
     }
     else
     {
-#ifdef LOG
-        printf("FLAC_DMXR: flac_eof_callback: False!\n");
-#endif
-        return false;
+      lprintf("flac_eof_callback: False!\n");
+
+      return false;
     }
 }
 
@@ -218,11 +210,9 @@ flac_write_callback (const FLAC__SeekableStreamDecoder *decoder,
      * is seeking. We do the decoding in the decoder
      */
     
-#ifdef LOG
-    printf("FLAC_DMXR: Error: Write callback was called!\n");
-#endif
+  lprintf("Error: Write callback was called!\n");
 
-    return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
+  return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
 }
 
 static void 
@@ -232,22 +222,20 @@ flac_metadata_callback (const FLAC__SeekableStreamDecoder *decoder,
 {
     demux_flac_t *this = (demux_flac_t *)client_data;
   
-#ifdef LOG
-    printf("FLAC_DMXR: IN: Metadata callback\n");
-#endif
+    lprintf("IN: Metadata callback\n");
+
     /* This should be called when we first look at a flac stream,
      * We get information about the stream here.
      */
      if (metadata->type == FLAC__METADATA_TYPE_STREAMINFO) {
-#ifdef LOG
-        printf("FLAC_DMXR: Got METADATA!\n");
-#endif
-        this->total_samples   = metadata->data.stream_info.total_samples;
-        this->bits_per_sample = metadata->data.stream_info.bits_per_sample;
-        this->channels        = metadata->data.stream_info.channels;
-        this->sample_rate     = metadata->data.stream_info.sample_rate;
-        this->length_in_msec  = (this->total_samples * 10 /
-                                (this->sample_rate / 100))/1000;
+       lprintf("Got METADATA!\n");
+
+       this->total_samples   = metadata->data.stream_info.total_samples;
+       this->bits_per_sample = metadata->data.stream_info.bits_per_sample;
+       this->channels        = metadata->data.stream_info.channels;
+       this->sample_rate     = metadata->data.stream_info.sample_rate;
+       this->length_in_msec  = (this->total_samples * 1000) /
+                                this->sample_rate;
      }
      return;
 }
@@ -257,22 +245,27 @@ flac_error_callback (const FLAC__SeekableStreamDecoder *decoder,
                      FLAC__StreamDecoderErrorStatus status,
                      void *client_data)
 {
+    demux_flac_t *this = (demux_flac_t *)client_data;
     /* This will be called if there is an error when flac is seeking
      * in the stream.
      */
 
-#ifdef LOG
-    printf("FLAC_DMXR: flac_error_callback\n");
-    if (status == FLAC__STREAM_DECODER_ERROR_STATUS_LOST_SYNC)
-        printf("FLAC_DMXR: Decoder lost synchronization.\n");
-    else if (status == FLAC__STREAM_DECODER_ERROR_STATUS_BAD_HEADER)
-        printf("FLAC_DMXR: Decoder encounted a corrupted frame header.\n");
-    else if (status == FLAC__STREAM_DECODER_ERROR_STATUS_FRAME_CRC_MISMATCH)
-        printf("FLAC_DMXR: Frame's data did not match the CRC in the footer.\n");
-    else
-        printf("FLAC_DMXR: unknown error.\n");
-#endif
+    xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "demux_flac: flac_error_callback\n");
 
+    if (status == FLAC__STREAM_DECODER_ERROR_STATUS_LOST_SYNC)
+      xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
+	      "demux_flac: Decoder lost synchronization.\n");
+    else if (status == FLAC__STREAM_DECODER_ERROR_STATUS_BAD_HEADER)
+      xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
+	      "demux_flac: Decoder encounted a corrupted frame header.\n");
+    else if (status == FLAC__STREAM_DECODER_ERROR_STATUS_FRAME_CRC_MISMATCH)
+      xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
+	      "demux_flac: Frame's data did not match the CRC in the footer.\n");
+    else
+      xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "demux_flac: unknown error.\n");
+    
+    this->status = DEMUX_FINISHED;
+    
     return;
 }
 
@@ -281,7 +274,7 @@ static int
 demux_flac_send_chunk (demux_plugin_t *this_gen) {
     demux_flac_t *this = (demux_flac_t *) this_gen;
     buf_element_t *buf = NULL;
-    off_t current_file_pos;
+    off_t current_file_pos, file_size = 0;
     int64_t current_pts;
     unsigned int remaining_sample_bytes = 0;
 
@@ -289,27 +282,41 @@ demux_flac_send_chunk (demux_plugin_t *this_gen) {
 
     current_file_pos = this->input->get_current_pos (this->input) 
                         - this->data_start;
+    if( (this->data_size - this->data_start) > 0 )
+        file_size = (this->data_size - this->data_start);
 
     current_pts = current_file_pos;
-    current_pts *= 90000;
-    if (this->sample_rate != 0)
-    {
-        current_pts /= this->sample_rate;
-    }
+    current_pts *= this->length_in_msec * 90;
+    if( file_size )
+        current_pts /= file_size;
 
     if (this->seek_flag) {
-        xine_demux_control_newpts (this->stream, current_pts, 0);
+#ifdef USE_ESTIMATED_PTS
+        _x_demux_control_newpts (this->stream, current_pts, BUF_FLAG_SEEK);
+#else
+        _x_demux_control_newpts (this->stream, 0, BUF_FLAG_SEEK);
+#endif
         this->seek_flag = 0;
     }
 
+
     while (remaining_sample_bytes)
     {
+        if(!this->audio_fifo) {
+          this->status = DEMUX_FINISHED;
+          break;
+        }
+
         buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
         buf->type = BUF_AUDIO_FLAC;
-        buf->extra_info->input_pos    = current_file_pos;
-        buf->extra_info->input_length = this->data_size;
+        if( file_size )
+          buf->extra_info->input_normpos = (int) ((double)current_file_pos * 65535 / file_size);
         buf->extra_info->input_time   = current_pts / 90;
-        //buf->pts = current_pts;
+#ifdef USE_ESTIMATED_PTS
+        buf->pts = current_pts;
+#else
+        buf->pts = 0;
+#endif
 
         if (remaining_sample_bytes > buf->max_size)
             buf->size = buf->max_size;
@@ -319,12 +326,11 @@ demux_flac_send_chunk (demux_plugin_t *this_gen) {
         remaining_sample_bytes -= buf->size;
 
         if (this->input->read (this->input,buf->content,buf->size)!=buf->size) {
-#ifdef LOG
-            printf("FLAC_DMXR: buf->size != input->read()\n");
-#endif
-            buf->free_buffer (buf);
-            this->status = DEMUX_FINISHED;
-            break;
+	  lprintf("buf->size != input->read()\n");
+
+	  buf->free_buffer (buf);
+	  this->status = DEMUX_FINISHED;
+	  break;
         }
 
         /*
@@ -345,27 +351,25 @@ demux_flac_send_headers (demux_plugin_t *this_gen) {
 
     buf_element_t *buf;
 
-#ifdef LOG
-    printf("FLAC_DMXR: demux_flac_send_headers\n");
-#endif
+    lprintf("demux_flac_send_headers\n");
 
     this->video_fifo = this->stream->video_fifo;
     this->audio_fifo = this->stream->audio_fifo;
 
     this->status = DEMUX_OK;
 
-    this->stream->stream_info[XINE_STREAM_INFO_HAS_VIDEO] = 0;
-    this->stream->stream_info[XINE_STREAM_INFO_HAS_AUDIO] = 1;
-    this->stream->stream_info[XINE_STREAM_INFO_AUDIO_CHANNELS] = this->channels;
-    this->stream->stream_info[XINE_STREAM_INFO_AUDIO_SAMPLERATE] = this->sample_rate;
-    this->stream->stream_info[XINE_STREAM_INFO_AUDIO_BITS] = this->bits_per_sample;
+    _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_VIDEO, 0);
+    _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_AUDIO, 1);
+    _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_CHANNELS, this->channels);
+    _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_SAMPLERATE, this->sample_rate);
+    _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_BITS, this->bits_per_sample);
 
-    xine_demux_control_start (this->stream);
+    _x_demux_control_start (this->stream);
 
     if (this->audio_fifo) {
         buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
         buf->type = BUF_AUDIO_FLAC;
-        buf->decoder_flags   = BUF_FLAG_HEADER;
+        buf->decoder_flags   = BUF_FLAG_HEADER|BUF_FLAG_STDHEADER|BUF_FLAG_FRAME_END;
         buf->decoder_info[0] = 0;
         buf->decoder_info[1] = this->sample_rate;
         buf->decoder_info[2] = this->bits_per_sample;
@@ -379,9 +383,8 @@ static void
 demux_flac_dispose (demux_plugin_t *this_gen) {
     demux_flac_t *this = (demux_flac_t *) this_gen;
 
-#ifdef LOG
-    printf("FLAC_DMXR: demux_flac_dispose\n");
-#endif
+    lprintf("demux_flac_dispose\n");
+
     if (this->flac_decoder)
         FLAC__seekable_stream_decoder_delete (this->flac_decoder);
 
@@ -393,52 +396,60 @@ static int
 demux_flac_get_status (demux_plugin_t *this_gen) {
     demux_flac_t *this = (demux_flac_t *) this_gen;
 
-#ifdef LOG
-    printf("FLAC_DMXR: demux_flac_get_status\n");
-#endif
+    lprintf("demux_flac_get_status\n");
 
     return this->status;
 }
 
 
 static int 
-demux_flac_seek (demux_plugin_t *this_gen, off_t start_pos, int start_time) {
+demux_flac_seek (demux_plugin_t *this_gen, off_t start_pos, int start_time, int playing) {
     demux_flac_t *this = (demux_flac_t *) this_gen;
 
-#ifdef LOG
-    printf("FLAC_DMXR: demux_flac_seek\n");
-#endif
+    lprintf("demux_flac_seek\n");
+
+    start_pos = (off_t) ( (double) start_pos / 65535 *
+                this->input->get_length (this->input) );
+
+    if (!start_pos && start_time) {
+        double distance = (double)start_time;
+
+        if (this->length_in_msec != 0)
+        {
+            distance /= (double)this->length_in_msec;
+        }
+        start_pos = (uint64_t)(distance * (this->data_size - this->data_start));
+    }
+    
     if (start_pos || !start_time) {
         
+        start_pos += this->data_start;
         this->input->seek (this->input, start_pos, SEEK_SET);
-#ifdef LOG
-        printf ("Seek to position: %lld\n", start_pos);
-#endif
+        lprintf ("Seek to position: %lld\n", start_pos);
     
     } else {
       
-        double distance = (double)start_time*1000.0;
-        uint64_t target_sample = (uint64_t)(distance * this->total_samples);
+        double distance = (double)start_time;
+        uint64_t target_sample;
         FLAC__bool s = false;
 
         if (this->length_in_msec != 0)
         {
             distance /= (double)this->length_in_msec;
         }
-
+        target_sample = (uint64_t)(distance * this->total_samples);
 
         s = FLAC__seekable_stream_decoder_seek_absolute (this->flac_decoder,
                                                          target_sample);
 
         if (s) {
-#ifdef LOG
-            printf ("Seek to: %d successfull!\n", start_time);
-#endif
+	  lprintf ("Seek to: %d successfull!\n", start_time);
         } else
             this->status = DEMUX_FINISHED;
     }
 
-    xine_demux_flush_engine (this->stream);
+    _x_demux_flush_engine (this->stream);
+    this->seek_flag = 1;
 
     return this->status;
 }
@@ -447,9 +458,7 @@ static int
 demux_flac_get_stream_length (demux_plugin_t *this_gen) {
     demux_flac_t *this = (demux_flac_t *) this_gen; 
 
-#ifdef LOG
-    printf("FLAC_DMXR: demux_flac_get_stream_length\n");
-#endif
+    lprintf("demux_flac_get_stream_length\n");
 
     if (this->flac_decoder)
         return this->length_in_msec;
@@ -459,18 +468,16 @@ demux_flac_get_stream_length (demux_plugin_t *this_gen) {
 
 static uint32_t 
 demux_flac_get_capabilities (demux_plugin_t *this_gen) {
-#ifdef LOG
-    printf("FLAC_DMXR: demux_flac_get_capabilities\n");
-#endif
-    return DEMUX_CAP_NOCAP;
+  lprintf("demux_flac_get_capabilities\n");
+
+  return DEMUX_CAP_NOCAP;
 }
 
 static int 
 demux_flac_get_optional_data (demux_plugin_t *this_gen, void *data, int dtype) {
-#ifdef LOG
-    printf("FLAC_DMXR: demux_flac_get_optional_data\n");
-#endif
-    return DEMUX_OPTIONAL_UNSUPPORTED;
+  lprintf("demux_flac_get_optional_data\n");
+
+  return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
 static demux_plugin_t *
@@ -479,39 +486,36 @@ open_plugin (demux_class_t *class_gen,
              input_plugin_t *input) {
     demux_flac_t *this;
 
-#ifdef LOG
-    printf("FLAC_DMXR: open_plugin\n");
-#endif
-#if 0
-    if ((input->get_capabilities (input) & INPUT_CAP_SEEKABLE) != 0)
-    {
-        printf("FLAC_DMXR: Input is not seekable, will not handle.\n");
-        return NULL;
-    }
-#endif
+    lprintf("open_plugin\n");
 
     switch (stream->content_detection_method) {
         case METHOD_BY_CONTENT:
         {
-            uint8_t buf[4096];
+          uint8_t      buf[MAX_PREVIEW_SIZE];
+          int          len;
 
-            /* Seek to the beginning */
-            input->seek(input, 0, SEEK_SET);
+          /* 
+           * try to get a preview of the data
+           */
+          len = input->get_optional_data (input, buf, INPUT_OPTIONAL_DATA_PREVIEW);
+          if (len == INPUT_OPTIONAL_UNSUPPORTED) {
+      
+            if (input->get_capabilities (input) & INPUT_CAP_SEEKABLE) {
+      
+              input->seek (input, 0, SEEK_SET);
+              if ( (len=input->read (input, buf, 1024)) <= 0)
+                return NULL;
+              input->seek (input, 0, SEEK_SET);
+      
+            } else
+              return NULL;
+          }      
 
-
-            /* FIXME: Skip id3v2 tag */
-            if (input->read (input, buf, 4)) {
-                input->seek(input, 0, SEEK_SET);
-                /* Look for fLaC tag at the beginning of file */
-                if ( (buf[0] != 'f') || (buf[1] != 'L') ||
-                     (buf[2] != 'a') || (buf[3] != 'C') )
-                    return NULL;
-            }
-            else {
-		printf("demux_flac: failed reading signature.\n");
-		/* FIXME: use preview buffer instead */
-		return NULL;
-            }
+          /* FIXME: Skip id3v2 tag */
+          /* Look for fLaC tag at the beginning of file */
+          if ( (buf[0] != 'f') || (buf[1] != 'L') ||
+               (buf[2] != 'a') || (buf[3] != 'C') )
+            return NULL;
         }
         break;
         case METHOD_BY_EXTENSION: {
@@ -549,8 +553,6 @@ open_plugin (demux_class_t *class_gen,
     this->demux_plugin.dispose           = demux_flac_dispose;
     this->demux_plugin.get_status        = demux_flac_get_status;
     this->demux_plugin.get_stream_length = demux_flac_get_stream_length;
-    this->demux_plugin.get_video_frame   = NULL;
-    this->demux_plugin.got_video_frame_cb= NULL;
     this->demux_plugin.get_capabilities  = demux_flac_get_capabilities;
     this->demux_plugin.get_optional_data = demux_flac_get_optional_data;
     this->demux_plugin.demux_class       = class_gen;
@@ -560,9 +562,8 @@ open_plugin (demux_class_t *class_gen,
 
     /* Get a new FLAC decoder and hook up callbacks */
     this->flac_decoder = FLAC__seekable_stream_decoder_new();
-#ifdef LOG
-    printf("FLAC_DMXR: this->flac_decoder: %p\n", this->flac_decoder);
-#endif
+    lprintf("this->flac_decoder: %p\n", this->flac_decoder);
+
     FLAC__seekable_stream_decoder_set_md5_checking  (this->flac_decoder, false);
     FLAC__seekable_stream_decoder_set_read_callback (this->flac_decoder,
                                                      flac_read_callback);
@@ -594,9 +595,15 @@ open_plugin (demux_class_t *class_gen,
      */
     this->status = DEMUX_OK;
     FLAC__seekable_stream_decoder_process_until_end_of_metadata (this->flac_decoder);
-#ifdef LOG
-    printf("FLAC_DMXR: Processed file until end of metadata\n");
-#endif
+
+    lprintf("Processed file until end of metadata: %s\n",
+	    this->status == DEMUX_OK ? "success" : "failure");
+
+    if (this->status != DEMUX_OK) {
+        FLAC__seekable_stream_decoder_delete (this->flac_decoder);
+        free (this);
+        return NULL;
+    }
 
     return &this->demux_plugin;
 }
@@ -628,9 +635,8 @@ static void
 class_dispose (demux_class_t *this_gen) {
     demux_flac_class_t *this = (demux_flac_class_t *) this_gen;
 
-#ifdef LOG
-    printf("FLAC_DMXR: class_dispose\n");
-#endif
+    lprintf("class_dispose\n");
+
     free (this);
 }
 
@@ -639,9 +645,8 @@ demux_flac_init_class (xine_t *xine, void *data) {
 
     demux_flac_class_t     *this;
   
-#ifdef LOG
-    printf("FLAC_DMXR: demux_flac_init_class\n");
-#endif
+    lprintf("demux_flac_init_class\n");
+
     this         = xine_xmalloc (sizeof (demux_flac_class_t));
     this->config = xine->config;
     this->xine   = xine;
@@ -655,14 +660,3 @@ demux_flac_init_class (xine_t *xine, void *data) {
 
     return this;
 }
-
-/*
- * exported plugin catalog entry
- */
-#if 0
-plugin_info_t xine_plugin_info[] = {
-  /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 20, "flac", XINE_VERSION_CODE, NULL, demux_flac_init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
-#endif
