@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.53.2.1 2005/07/17 21:50:02 miguelfreitas Exp $
+ * $Id: xine_decoder.c,v 1.56 2006/06/02 22:18:57 dsalt Exp $
  *
  * stuff needed to turn libmad into a xine decoder plugin
  */
@@ -363,12 +363,12 @@ static uint32_t audio_types[] = {
   BUF_AUDIO_MPEG, 0
 };
 
-static decoder_info_t dec_info_audio = {
+static const decoder_info_t dec_info_audio = {
   audio_types,         /* supported types */
   7                    /* priority        */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_AUDIO_DECODER, 15, "mad", XINE_VERSION_CODE, &dec_info_audio, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
