@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: audio_decoder.c,v 1.20 2003/01/23 12:47:24 miguelfreitas Exp $
+ * $Id: audio_decoder.c,v 1.48 2006/06/17 12:28:31 dgp85 Exp $
  *
  * thin layer to use real binary-only codecs in xine
  *
@@ -32,21 +32,26 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <dlfcn.h>
+#ifdef __x86_64__
+  #include <elf.h>
+#endif
+
+#define LOG_MODULE "real_audio_decoder"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
 
 #include "bswap.h"
 #include "xine_internal.h"
 #include "video_out.h"
 #include "buffer.h"
-
-/*
-#define LOG
-*/
+#include "xineutils.h"
 
 typedef struct {
   audio_decoder_class_t   decoder_class;
 
-  char                   *real_codec_path;
-
+  /* empty so far */
 } real_class_t;
 
 typedef struct realdec_decoder_s {
@@ -100,38 +105,12 @@ typedef struct {
     void  *extras;
 } ra_init_t;
 
-#ifdef LOG
-static void hexdump (char *buf, int length) {
+void *__builtin_new(unsigned long size);
+void __builtin_delete (void *foo);
+void *__builtin_vec_new(unsigned long size);
+void __builtin_vec_delete(void *mem);
+void __pure_virtual(void);
 
-  int i;
-
-  printf ("libareal: ascii contents>");
-  for (i = 0; i < length; i++) {
-    unsigned char c = buf[i];
-
-    if ((c >= 32) && (c <= 128))
-      printf ("%c", c);
-    else
-      printf (".");
-  }
-  printf ("\n");
-
-  printf ("libareal: complete hexdump of package follows:\nlibareal 0x0000:  ");
-  for (i = 0; i < length; i++) {
-    unsigned char c = buf[i];
-
-    printf ("%02x", c);
-
-    if ((i % 16) == 15)
-      printf ("\nlibareal 0x%04x: ", i+1);
-
-    if ((i % 2) == 1)
-      printf (" ");
-
-  }
-  printf ("\n");
-}
-#endif
 
 void *__builtin_new(unsigned long size) {
   return malloc(size);
@@ -142,21 +121,75 @@ void __builtin_delete (void *foo) {
   free (foo);
 }
 
-static int load_syms_linux (realdec_decoder_t *this, char *codec_name) {
+#ifdef __x86_64__
+/* (gb) quick-n-dirty check to be run natively */
+static int is_x86_64_object_(FILE *f)
+{
+  Elf64_Ehdr *hdr = malloc(sizeof(Elf64_Ehdr));
+  if (hdr == NULL)
+	return 0;
 
-  char path[1024];
+  if (fseek(f, 0, SEEK_SET) != 0) {
+	free(hdr);
+	return 0;
+  }
 
-  sprintf (path, "%s/%s", this->cls->real_codec_path, codec_name);
+  if (fread(hdr, sizeof(Elf64_Ehdr), 1, f) != 1) {
+	free(hdr);
+	return 0;
+  }
 
-#ifdef LOG
-  printf ("libareal: (audio) opening shared obj '%s'\n", path);
+  if (hdr->e_ident[EI_MAG0] != ELFMAG0 ||
+	  hdr->e_ident[EI_MAG1] != ELFMAG1 ||
+	  hdr->e_ident[EI_MAG2] != ELFMAG2 ||
+	  hdr->e_ident[EI_MAG3] != ELFMAG3) {
+	free(hdr);
+	return 0;
+  }
 
+  return hdr->e_machine == EM_X86_64;
+}
+
+static inline int is_x86_64_object(const char *filename)
+{
+  FILE *f;
+  int ret;
+
+  if ((f = fopen(filename, "r")) == NULL)
+	return 0;
+
+  ret = is_x86_64_object_(f);
+  fclose(f);
+  return ret;
+}
 #endif
 
+static int load_syms_linux (realdec_decoder_t *this, char *codec_name,
+			    const char *alt_codec_name) {
+
+  cfg_entry_t* entry = this->stream->xine->config->lookup_entry(
+			 this->stream->xine->config, "decoder.external.real_codecs_path");
+  char path[1024];
+  struct stat sb;
+
+  snprintf (path, sizeof(path), "%s/%s", entry->str_value, codec_name);
+  if (stat(path, &sb))
+    snprintf (path, sizeof(path), "%s/%s", entry->str_value, alt_codec_name);
+    
+#ifdef __x86_64__
+  /* check whether it's a real x86-64 library */
+  if (!is_x86_64_object(path))
+	return 0;
+#endif
+
+  lprintf ("(audio) opening shared obj '%s'\n", path);
+
   this->ra_handle = dlopen (path, RTLD_LAZY);
 
   if (!this->ra_handle) {
-    printf ("libareal: error: %s\n", dlerror());
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "libareal: error: %s\n", dlerror());
+    _x_message(this->stream, XINE_MSG_LIBRARY_LOAD_ERROR,
+                 codec_name, NULL);
     return 0;
   }
   
@@ -174,8 +207,8 @@ static int load_syms_linux (realdec_decoder_t *this, char *codec_name) {
   if (!this->raCloseCodec || !this->raDecode || !this->raFlush || !this->raFreeDecoder ||
       !this->raGetFlavorProperty || !this->raOpenCodec2 || !this->raSetFlavor ||
       /*!raSetDLLAccessPath ||*/ !this->raInitDecoder){
-    printf ("libareal: (audio) Cannot resolve symbols - incompatible dll: %s\n", 
-	    path);
+    xprintf (this->stream->xine, XINE_VERBOSITY_LOG, 
+	     _("libareal: (audio) Cannot resolve symbols - incompatible dll: %s\n"), path);
     return 0;
   }
 
@@ -183,7 +216,7 @@ static int load_syms_linux (realdec_decoder_t *this, char *codec_name) {
 
     char path[1024];
 
-    sprintf(path, "DT_Codecs=%s", this->cls->real_codec_path);
+    snprintf(path, sizeof(path) - 2, "DT_Codecs=%s", entry->str_value);
     if (path[strlen(path)-1]!='/'){
       path[strlen(path)+1]=0;
       path[strlen(path)]='/';
@@ -193,9 +226,7 @@ static int load_syms_linux (realdec_decoder_t *this, char *codec_name) {
     this->raSetDLLAccessPath(path);
   }
 
-#ifdef LOG
-  printf ("libareal: audio decoder loaded successfully\n");
-#endif
+  lprintf ("audio decoder loaded successfully\n");
 
   return 1;
 }
@@ -213,100 +244,93 @@ static int init_codec (realdec_decoder_t *this, buf_element_t *buf) {
    * extract header data
    */
 
-  version = BE_16 (buf->content);
+  version = BE_16 (buf->content+4);
 
+  lprintf ("header buffer detected, header version %d\n", version);
 #ifdef LOG
-  printf ("libareal: header buffer detected, header version %d\n", version);
-  hexdump (buf->content, buf->size);
+  xine_hexdump (buf->content, buf->size);
 #endif
     
-  flavor           = BE_16 (buf->content+18);
-  coded_frame_size = BE_32 (buf->content+20);
-  codec_data_length= BE_16 (buf->content+36);
-  coded_frame_size2= BE_16 (buf->content+38);
-  subpacket_size   = BE_16 (buf->content+40);
+  flavor           = BE_16 (buf->content+22);
+  coded_frame_size = BE_32 (buf->content+24);
+  codec_data_length= BE_16 (buf->content+40);
+  coded_frame_size2= BE_16 (buf->content+42);
+  subpacket_size   = BE_16 (buf->content+44);
     
   this->sps        = subpacket_size;
   this->w          = coded_frame_size2;
   this->h          = codec_data_length;
 
   if (version == 4) {
-    samples_per_sec = BE_16 (buf->content+44);
-    bits_per_sample = BE_16 (buf->content+48);
-    num_channels    = BE_16 (buf->content+50);
+    samples_per_sec = BE_16 (buf->content+48);
+    bits_per_sample = BE_16 (buf->content+52);
+    num_channels    = BE_16 (buf->content+54);
 
     /* FIXME: */
     if (buf->type==BUF_AUDIO_COOK) {
       
-      printf ("libareal: audio header version 4 for COOK audio not supported.\n");
-      abort();
+      xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, 
+	       "libareal: audio header version 4 for COOK audio not supported.\n");
+      return 0;
     }
     data_len        = 0; /* FIXME: COOK audio needs this */
-    extras          = buf->content+0x43;
+    extras          = buf->content+71;
 
   } else {
-    samples_per_sec = BE_16 (buf->content+50);
-    bits_per_sample = BE_16 (buf->content+54);
-    num_channels    = BE_16 (buf->content+56);
-    data_len        = BE_32 (buf->content+0x46);
-    extras          = buf->content+0x4a;
+    samples_per_sec = BE_16 (buf->content+54);
+    bits_per_sample = BE_16 (buf->content+58);
+    num_channels    = BE_16 (buf->content+60);
+    data_len        = BE_32 (buf->content+74);
+    extras          = buf->content+78;
   }
 
   this->block_align= coded_frame_size2;
 
-#ifdef LOG
-  printf ("libareal: 0x%04x 0x%04x 0x%04x 0x%04x data_len 0x%04x\n",
-	  subpacket_size, coded_frame_size, codec_data_length, 
-	  coded_frame_size2, data_len);
-  printf ("libareal: %d samples/sec, %d bits/sample, %d channels\n",
-	  samples_per_sec, bits_per_sample, num_channels);
-#endif
+  lprintf ("0x%04x 0x%04x 0x%04x 0x%04x data_len 0x%04x\n",
+	   subpacket_size, coded_frame_size, codec_data_length, 
+	   coded_frame_size2, data_len);
+  lprintf ("%d samples/sec, %d bits/sample, %d channels\n",
+	   samples_per_sec, bits_per_sample, num_channels);
 
   /* load codec, resolv symbols */
 
   switch (buf->type) {
   case BUF_AUDIO_COOK:
-    if (!load_syms_linux (this, "cook.so.6.0"))
+    _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "Cook");
+    if (!load_syms_linux (this, "cook.so", "cook.so.6.0"))
       return 0;
-
-    this->stream->meta_info[XINE_META_INFO_AUDIOCODEC] 
-      = strdup ("Cook");
     break;
     
   case BUF_AUDIO_ATRK:
-    if (!load_syms_linux (this, "atrc.so.6.0"))
+    _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "Atrac");
+    if (!load_syms_linux (this, "atrc.so", "atrc.so.6.0"))
       return 0;
     this->block_align = 384;
-    this->stream->meta_info[XINE_META_INFO_AUDIOCODEC] 
-      = strdup ("Atrac");
     break;
 
   case BUF_AUDIO_14_4:
-    if (!load_syms_linux (this, "14_4.so.6.0"))
+    _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "Real 14.4");
+    if (!load_syms_linux (this, "14_4.so", "14_4.so.6.0"))
       return 0;
-    this->stream->meta_info[XINE_META_INFO_AUDIOCODEC] 
-      = strdup ("Real 14.4");
     break;
 
   case BUF_AUDIO_28_8:
-    if (!load_syms_linux (this, "28_8.so.6.0"))
+    _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "Real 28.8");
+    if (!load_syms_linux (this, "28_8.so", "28_8.so.6.0"))
       return 0;
-    this->stream->meta_info[XINE_META_INFO_AUDIOCODEC] 
-      = strdup ("Real 28.8");
     break;
 
   case BUF_AUDIO_SIPRO:
-    if (!load_syms_linux (this, "sipr.so.6.0"))
+    _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "Sipro");
+    if (!load_syms_linux (this, "sipr.so", "sipr.so.6.0"))
       return 0;
     /* this->block_align = 19; */
-    this->stream->meta_info[XINE_META_INFO_AUDIOCODEC] 
-      = strdup ("Sipro");
     break;
 
   default:
-    printf ("libareal: error, i don't handle buf type 0x%08x\n",
-	    buf->type);
-    abort();
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, 
+	     "libareal: error, i don't handle buf type 0x%08x\n", buf->type);
+    return 0;
   }
 
   /*
@@ -315,7 +339,7 @@ static int init_codec (realdec_decoder_t *this, buf_element_t *buf) {
 
   result = this->raOpenCodec2 (&this->context);
   if (result) {
-    printf ("libareal: error in raOpenCodec2: %d\n", result);
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "libareal: error in raOpenCodec2: %d\n", result);
     return 0;
   }
 
@@ -333,15 +357,15 @@ static int init_codec (realdec_decoder_t *this, buf_element_t *buf) {
 
 #ifdef LOG
     printf ("libareal: init_data:\n");
-    hexdump (&init_data, sizeof (ra_init_t));
+    xine_hexdump ((char *) &init_data, sizeof (ra_init_t));
     printf ("libareal: extras :\n");
-    hexdump (init_data.extras, data_len);
+    xine_hexdump (init_data.extras, data_len);
 #endif
      
     result = this->raInitDecoder (this->context, &init_data);
     if(result){
-      printf ("libareal: decoder init failed, error code: 0x%x\n",
-	      result);
+      xprintf (this->stream->xine, XINE_VERBOSITY_LOG, 
+	       _("libareal: decoder init failed, error code: 0x%x\n"), result);
       return 0;
     }
   }
@@ -349,13 +373,13 @@ static int init_codec (realdec_decoder_t *this, buf_element_t *buf) {
   if (this->raSetPwd){
     /* used by 'SIPR' */
     this->raSetPwd (this->context, "Ardubancel Quazanga"); /* set password... lol. */
-    printf ("libareal: password set\n");
+    lprintf ("password set\n");
   }
 
   result = this->raSetFlavor (this->context, flavor);
   if (result){
-    printf ("libareal: decoder flavor setup failed, error code: 0x%x\n",
-	    result);
+    xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
+	     _("libareal: decoder flavor setup failed, error code: 0x%x\n"), result);
     return 0;
   }
 
@@ -391,8 +415,9 @@ static int init_codec (realdec_decoder_t *this, buf_element_t *buf) {
     mode = AO_CAP_MODE_STEREO;
     break;
   default:
-    printf ("libareal: oups, real can do more than 2 channels ?\n");
-    abort();
+    xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
+	     _("libareal: oups, real can do more than 2 channels ?\n"));
+    return 0;
   }
 
   this->stream->audio_out->open(this->stream->audio_out, 
@@ -418,10 +443,8 @@ static unsigned char sipr_swaps[38][2]={
 static void realdec_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
   realdec_decoder_t *this = (realdec_decoder_t *) this_gen;
 
-#ifdef LOG
-  printf ("libareal: decode_data %d bytes, flags=0x%08x, pts=%lld ...\n", 
-	  buf->size, buf->decoder_flags, buf->pts);
-#endif
+  lprintf ("decode_data %d bytes, flags=0x%08x, pts=%lld ...\n", 
+	   buf->size, buf->decoder_flags, buf->pts);
 
   if (buf->decoder_flags & BUF_FLAG_PREVIEW) {
 
@@ -431,15 +454,13 @@ static void realdec_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
 
     this->decoder_ok = init_codec (this, buf) ;
     if( !this->decoder_ok )
-      this->stream->stream_info[XINE_STREAM_INFO_AUDIO_HANDLED] = 0;
+      _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_HANDLED, 0);
 
   } else if( this->decoder_ok ) {
 
     int size;
 
-#ifdef LOG
-    printf ("libareal: content buffer detected, %d bytes\n", buf->size);
-#endif
+    lprintf ("content buffer detected, %d bytes\n", buf->size);
 
     if (buf->pts && !this->pts)
       this->pts = buf->pts;
@@ -457,9 +478,7 @@ static void realdec_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
 	memcpy (this->frame_buffer+this->frame_num_bytes, buf->content, size);
 	this->frame_num_bytes += size;
 
-#ifdef LOG
-	printf ("libareal: buffering %d/%d bytes\n", this->frame_num_bytes, this->frame_size);
-#endif
+	lprintf ("buffering %d/%d bytes\n", this->frame_num_bytes, this->frame_size);
 
 	size = 0;
 
@@ -473,19 +492,15 @@ static void realdec_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
 	int h       = this->h;
 	audio_buffer_t *audio_buffer;
 
-#ifdef LOG
-	printf ("libareal: buffering %d bytes\n", needed);
-#endif
+	lprintf ("buffering %d bytes\n", needed);
 
 	memcpy (this->frame_buffer+this->frame_num_bytes, buf->content, needed);
 
 	size -= needed;
 	this->frame_num_bytes = 0;
 
-#ifdef LOG
-	printf ("libareal: frame completed. reordering...\n");
-	printf ("libareal: bs=%d  sps=%d  w=%d h=%d \n",/*sh->wf->nBlockAlign*/-1,sps,w,h);
-#endif
+	lprintf ("frame completed. reordering...\n");
+	lprintf ("bs=%d  sps=%d  w=%d h=%d \n",/*sh->wf->nBlockAlign*/-1,sps,w,h);
 
 	if (!sps) {
 
@@ -533,10 +548,8 @@ static void realdec_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
 	    
 	    for (x=0; x<w; x++) {
 	      
-#ifdef LOG
-	      printf ("libareal: x=%d, y=%d, off %d\n",
-		      x, y, sps*(h*x+((h+1)/2)*(y&1)+(y>>1)));
-#endif
+	      lprintf ("x=%d, y=%d, off %d\n",
+		       x, y, sps*(h*x+((h+1)/2)*(y&1)+(y>>1)));
 	      
 	      memcpy (this->frame_reordered+sps*(h*x+((h+1)/2)*(y&1)+(y>>1)),
 		      s, sps);
@@ -553,7 +566,7 @@ static void realdec_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
 	}
 
 #ifdef LOG
-	hexdump (this->frame_reordered, buf->size);
+	xine_hexdump (this->frame_reordered, buf->size);
 #endif
   
 	n = 0;
@@ -564,11 +577,9 @@ static void realdec_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
 	  result = this->raDecode (this->context, 
 				   this->frame_reordered+n,
 				   this->block_align,
-				   audio_buffer->mem, &len, -1);
+				   (char *) audio_buffer->mem, &len, -1);
 
-#ifdef LOG
-	  printf ("libareal: raDecode result %d, len=%d\n", result, len);
-#endif
+	  lprintf ("raDecode result %d, len=%d\n", result, len);
 
 	  audio_buffer->vpts       = this->pts; 
 
@@ -584,10 +595,7 @@ static void realdec_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
     }
   }
 
-
-#ifdef LOG
-  printf ("libareal: decode_data...done\n");
-#endif
+  lprintf ("decode_data...done\n");
 }
 
 static void realdec_reset (audio_decoder_t *this_gen) {
@@ -606,9 +614,7 @@ static void realdec_dispose (audio_decoder_t *this_gen) {
 
   realdec_decoder_t *this = (realdec_decoder_t *) this_gen;
 
-#ifdef LOG
-  printf ("libareal: dispose\n");
-#endif
+  lprintf ("dispose\n");
 
   if (this->context)
     this->raCloseCodec (this->context);
@@ -620,9 +626,7 @@ static void realdec_dispose (audio_decoder_t *this_gen) {
     this->raFreeDecoder (this->context);
 #endif
 
-#ifdef LOG
-  printf ("libareal: dlclose...\n");
-#endif
+  lprintf ("dlclose...\n");
 
   if (this->ra_handle)
     dlclose (this->ra_handle);
@@ -635,9 +639,7 @@ static void realdec_dispose (audio_decoder_t *this_gen) {
 
   free (this);
 
-#ifdef LOG
-  printf ("libareal: dispose done\n");
-#endif
+  lprintf ("dispose done\n");
 }
 
 static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, 
@@ -646,8 +648,7 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen,
   real_class_t      *cls = (real_class_t *) class_gen;
   realdec_decoder_t *this ;
 
-  this = (realdec_decoder_t *) malloc (sizeof (realdec_decoder_t));
-  memset(this, 0, sizeof (realdec_decoder_t));
+  this = (realdec_decoder_t *) xine_xmalloc (sizeof (realdec_decoder_t));
 
   this->audio_decoder.decode_data         = realdec_decode_data;
   this->audio_decoder.reset               = realdec_reset;
@@ -687,7 +688,7 @@ void __builtin_vec_delete(void *mem) {
   free(mem);
 }
 void __pure_virtual(void) {
-  printf("libareal: FATAL: __pure_virtual() called!\n");
+  lprintf("libareal: FATAL: __pure_virtual() called!\n");
   /*      exit(1); */
 }
 
@@ -695,16 +696,13 @@ void __pure_virtual(void) {
  * real audio codec loader
  */
 
-static void codec_path_cb (void *data, xine_cfg_entry_t *cfg) {
-  real_class_t *this = (real_class_t *) data;
-  
-  this->real_codec_path = cfg->str_value;
-}
-
 static void *init_class (xine_t *xine, void *data) {
 
   real_class_t       *this;
   config_values_t    *config = xine->config;
+  char               *real_codec_path;
+  char               *default_real_codec_path = "";
+  struct stat s;
 
   this = (real_class_t *) xine_xmalloc (sizeof (real_class_t));
 
@@ -713,37 +711,44 @@ static void *init_class (xine_t *xine, void *data) {
   this->decoder_class.get_description = get_description;
   this->decoder_class.dispose         = dispose_class;
 
-  this->real_codec_path = config->register_string (config, "codec.real_codecs_path", 
-						   "unknown",
-						   _("path to real player codecs, if installed"),
-						   NULL, 10, codec_path_cb, (void *)this);
+  /* try some auto-detection */
+
+  if (!stat ("/usr/local/RealPlayer8/Codecs/drv3.so.6.0", &s)) 
+    default_real_codec_path = "/usr/local/RealPlayer8/Codecs";
+  if (!stat ("/usr/RealPlayer8/Codecs/drv3.so.6.0", &s)) 
+    default_real_codec_path = "/usr/RealPlayer8/Codecs";
+  if (!stat ("/usr/lib/RealPlayer8/Codecs/drv3.so.6.0", &s)) 
+    default_real_codec_path = "/usr/lib/RealPlayer8/Codecs";
+  if (!stat ("/opt/RealPlayer8/Codecs/drv3.so.6.0", &s)) 
+    default_real_codec_path = "/opt/RealPlayer8/Codecs";
+  if (!stat ("/usr/lib/RealPlayer9/users/Real/Codecs/drv3.so.6.0", &s)) 
+    default_real_codec_path = "/usr/lib/RealPlayer9/users/Real/Codecs";
+  if (!stat ("/usr/lib/RealPlayer10/codecs/drvc.so", &s)) 
+    default_real_codec_path = "/usr/lib/RealPlayer10/codecs";
+  if (!stat ("/usr/lib64/RealPlayer8/Codecs/drv3.so.6.0", &s)) 
+    default_real_codec_path = "/usr/lib64/RealPlayer8/Codecs";
+  if (!stat ("/usr/lib64/RealPlayer9/users/Real/Codecs/drv3.so.6.0", &s)) 
+    default_real_codec_path = "/usr/lib64/RealPlayer9/users/Real/Codecs";
+  if (!stat ("/usr/lib64/RealPlayer10/codecs/drvc.so", &s)) 
+    default_real_codec_path = "/usr/lib64/RealPlayer10/codecs";
+  if (!stat ("/usr/lib/codecs/drv3.so.6.0", &s)) 
+    default_real_codec_path = "/usr/lib/codecs";
+  if (!stat ("/usr/lib/win32/drv3.so.6.0", &s)) 
+    default_real_codec_path = "/usr/lib/win32";
   
-  if (!strcmp (this->real_codec_path, "unknown")) {
-
-    struct stat s;
-
-    /* try some auto-detection */
-
-    if (!stat ("/usr/local/RealPlayer8/Codecs/drv3.so.6.0", &s)) 
-      config->update_string (config, "codec.real_codecs_path", 
-			     "/usr/local/RealPlayer8/Codecs");
-    if (!stat ("/usr/RealPlayer8/Codecs/drv3.so.6.0", &s)) 
-      config->update_string (config, "codec.real_codecs_path", 
-			     "/usr/RealPlayer8/Codecs");
-    if (!stat ("/usr/lib/RealPlayer8/Codecs/drv3.so.6.0", &s)) 
-      config->update_string (config, "codec.real_codecs_path", 
-			     "/usr/lib/RealPlayer8/Codecs");
-    if (!stat ("/opt/RealPlayer8/Codecs/drv3.so.6.0", &s)) 
-      config->update_string (config, "codec.real_codecs_path", 
-			     "/opt/RealPlayer8/Codecs");
-    if (!stat ("/usr/lib/RealPlayer9/users/Real/Codecs/drv3.so.6.0", &s)) 
-      config->update_string (config, "codec.real_codecs_path", 
-			     "/usr/lib/RealPlayer9/users/Real/Codecs");
-  }
-
-#ifdef LOG
-  printf ("libareal: real codec path : %s\n",  this->real_codec_path);
-#endif
+  real_codec_path = config->register_string (config, "decoder.external.real_codecs_path", 
+					     default_real_codec_path,
+					     _("path to RealPlayer codecs"),
+					     _("If you have RealPlayer installed, specify the path "
+					       "to its codec directory here. You can easily find "
+					       "the codec directory by looking for a file named "
+					       "\"drv3.so.6.0\" in it. If xine can find the RealPlayer "
+					       "codecs, it will use them to decode RealPlayer content "
+					       "for you. Consult the xine FAQ for more information on "
+					       "how to install the codecs."),
+					     10, NULL, this);
+  
+  lprintf ("real codec path : %s\n",  real_codec_path);
 
   return this;
 }
@@ -753,16 +758,16 @@ static void *init_class (xine_t *xine, void *data) {
  */
 
 static uint32_t audio_types[] = { 
-  BUF_AUDIO_COOK, BUF_AUDIO_ATRK, BUF_AUDIO_14_4, BUF_AUDIO_28_8, BUF_AUDIO_SIPRO, 0
+  BUF_AUDIO_COOK, BUF_AUDIO_ATRK, /* BUF_AUDIO_14_4, BUF_AUDIO_28_8, */ BUF_AUDIO_SIPRO, 0
  };
 
-static decoder_info_t dec_info_audio = {
+static const decoder_info_t dec_info_audio = {
   audio_types,         /* supported types */
   5                    /* priority        */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_AUDIO_DECODER | PLUGIN_MUST_PRELOAD, 13, "realadec", XINE_VERSION_CODE, &dec_info_audio, init_class },
+  { PLUGIN_AUDIO_DECODER | PLUGIN_MUST_PRELOAD, 15, "realadec", XINE_VERSION_CODE, &dec_info_audio, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
