@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.88.2.1 2005/07/17 21:50:04 miguelfreitas Exp $
+ * $Id: xine_decoder.c,v 1.94 2006/06/17 12:50:18 hadess Exp $
  *
  */
 
@@ -696,7 +696,7 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
 
     _x_get_current_info (this->stream->master, &extra_info, sizeof(extra_info) );
    
-    lprintf("master: %d slave: %d input_normpos: %lld vo_discard: %d\n", 
+    lprintf("master: %d slave: %d input_normpos: %d vo_discard: %d\n", 
       master_status, slave_status, extra_info.input_normpos, vo_discard);
 
     if( !this->started && (master_status == XINE_STATUS_PLAY &&
@@ -855,7 +855,17 @@ static spu_decoder_t *sputext_class_open_plugin (spu_decoder_class_t *class_gen,
   return (spu_decoder_t *) this;
 }
 
-static void sputext_class_dispose (spu_decoder_class_t *this) {
+static void sputext_class_dispose (spu_decoder_class_t *class_gen) {
+  sputext_class_t *this = (sputext_class_t *)class_gen;
+
+  this->xine->config->unregister_callback(this->xine->config,
+					  "subtitles.separate.src_encoding");
+  this->xine->config->unregister_callback(this->xine->config,
+					  "subtitles.separate.subtitle_size");
+  this->xine->config->unregister_callback(this->xine->config,
+					  "subtitles.separate.vertical_offset");
+  this->xine->config->unregister_callback(this->xine->config,
+					  "subtitles.separate.use_unscaled_osd");
   free (this);
 }
 
@@ -944,12 +954,12 @@ static void *init_spu_decoder_plugin (xine_t *xine, void *data) {
 /* plugin catalog information */
 static uint32_t supported_types[] = { BUF_SPU_TEXT, BUF_SPU_OGM, 0 };
 
-static decoder_info_t spudec_info = {
+static const decoder_info_t spudec_info = {
   supported_types,     /* supported types */
   1                    /* priority        */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_SPU_DECODER | PLUGIN_MUST_PRELOAD, 16, "sputext", XINE_VERSION_CODE, &spudec_info, &init_spu_decoder_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
