@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.42 2004/12/16 13:59:10 mroi Exp $
+ * $Id: xine_decoder.c,v 1.46 2006/06/02 22:18:58 dsalt Exp $
  *
  * (ogg/)vorbis audio decoder plugin (libvorbis wrapper) for xine
  */
@@ -100,6 +100,7 @@ static struct {
   {"TITLE=", XINE_META_INFO_TITLE},
   {"GENRE=", XINE_META_INFO_GENRE},
   {"DESCRIPTION=", XINE_META_INFO_COMMENT},
+  {"COMMENT=", XINE_META_INFO_COMMENT},
   {"DATE=", XINE_META_INFO_YEAR},
   {"TRACKNUMBER=", XINE_META_INFO_TRACK_NUMBER},
   {NULL, 0}
@@ -165,28 +166,7 @@ static void vorbis_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 	
 	get_metadata (this);
 
-	
-	switch (this->vi.channels) {
-	case 1: 
-	  mode = AO_CAP_MODE_MONO;
-	  break;
-	case 2: 
-	  mode = AO_CAP_MODE_STEREO;
-	  break;
-	case 4: 
-	  mode = AO_CAP_MODE_4CHANNEL;
-	  break;
-	case 5: 
-	  mode = AO_CAP_MODE_5CHANNEL;
-	  break;
-	case 6: 
-	  mode = AO_CAP_MODE_5_1CHANNEL;
-	  break;
-	default:
-	  printf ("libvorbis: help, %d channels ?!\n",
-		  this->vi.channels);
-	  /* FIXME: handle error */
-	}
+	mode = _x_ao_channels2mode(this->vi.channels);
 	
 	this->convsize=MAX_NUM_SAMPLES/this->vi.channels;
 
@@ -359,12 +339,12 @@ static uint32_t audio_types[] = {
   BUF_AUDIO_VORBIS, 0
  };
 
-static decoder_info_t dec_info_audio = {
+static const decoder_info_t dec_info_audio = {
   audio_types,         /* supported types */
   5                    /* priority        */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_AUDIO_DECODER, 15, "vorbis", XINE_VERSION_CODE, &dec_info_audio, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
