@@ -1,5 +1,5 @@
-/* 
- * Copyright (C) 2000-2002 the xine project
+/*
+ * Copyright (C) 2000-2006 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: qt_decoder.c,v 1.13 2003/01/29 17:51:05 miguelfreitas Exp $
+ * $Id: qt_decoder.c,v 1.44 2006/06/11 15:09:47 valtri Exp $
  *
  * quicktime video/audio decoder plugin, using win32 dlls
  * most of this code comes directly from MPlayer
@@ -33,15 +33,24 @@
 #include <stdlib.h>
 #include <string.h>
 
+#define LOG_MODULE "qt_decoder"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
+
 #include "bswap.h"
 #include "xine_internal.h"
 #include "audio_out.h"
 #include "buffer.h"
 
 #include "qtx/qtxsdk/components.h"
+#include "wine/win32.h"
 #include "wine/windef.h"
 #include "wine/ldt_keeper.h"
 
+#include "common.c"
+
 /*
  * This version of the macro avoids compiler warnings about
  * multiple-character constants. It also does NOT assume
@@ -50,14 +59,7 @@
 #ifdef FOUR_CHAR_CODE
 #  undef FOUR_CHAR_CODE
 #endif
-#define FOUR_CHAR_CODE(a,b,c,d)  (uint32_t)( ((unsigned char)(a)<<24) | \
-                                             ((unsigned char)(b)<<16) | \
-                                             ((unsigned char)(c)<<8) | \
-                                             (unsigned char)(d) )
-
-/*
-#define LOG
-*/
+#define FOUR_CHAR_CODE BE_FOURCC
 
 /*
  *
@@ -75,19 +77,14 @@ int       WINAPI FreeLibrary(HMODULE);
  */
 static pthread_once_t   once_control = PTHREAD_ONCE_INIT;
 static pthread_mutex_t  win32_codec_mutex;
-extern char            *win32_def_path;
 
 static void init_routine(void) {
 
-#ifdef LOG
-  printf ("qt_decoder: init_routine\n");
-#endif
+  lprintf ("%s\n", __XINE_FUNCTION__);
 
   pthread_mutex_init (&win32_codec_mutex, NULL);
 
-#ifdef LOG
-  printf ("qt_decoder: init_routine completed\n");
-#endif
+  lprintf ("%s completed\n", __XINE_FUNCTION__);
 }
 
 #define BUFSIZE 1024*1024
@@ -178,7 +175,8 @@ typedef struct qta_decoder_s {
   uint8_t             data[BUFSIZE];
   int                 data_len;
   int                 num_frames;
-
+  
+  ldt_fs_t *ldt_fs;
 } qta_decoder_t;
 
 #ifdef LOG
@@ -224,88 +222,92 @@ static void qta_init_driver (qta_decoder_t *this, buf_element_t *buf) {
 
   this->FramesToGet = 0;
 
-#ifdef LOG
-  printf ("qt_audio: init_driver... (trying to lock mutex...)\n");
-#endif
+  lprintf ("audio: init_driver... (trying to lock mutex...)\n");
 
   pthread_mutex_lock(&win32_codec_mutex);
 
-#ifdef LOG
-  printf ("qt_audio: init_driver... (mutex locked)\n");
-#endif
+  lprintf ("audio: init_driver... (mutex locked)\n");
 
-  Setup_LDT_Keeper();
+  this->ldt_fs = Setup_LDT_Keeper();
   
   this->qtml_dll = LoadLibraryA("qtmlClient.dll");
   
   if (this->qtml_dll == NULL ) {
-    printf ("qt_audio: failed to load dll\n" );
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "qt_audio: failed to load dll\n" );
     pthread_mutex_unlock(&win32_codec_mutex);
+    _x_message(this->stream, XINE_MSG_LIBRARY_LOAD_ERROR,
+                 "qtmlClient.dll", NULL);
     return;
   }
 
   this->InitializeQTML = (LPFUNC1)GetProcAddress (this->qtml_dll, "InitializeQTML");
   if ( this->InitializeQTML == NULL )  {
-    printf ("qt_audio: failed geting proc address InitializeQTML\n");
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, 
+	     "qt_audio: failed geting proc address InitializeQTML\n");
     pthread_mutex_unlock(&win32_codec_mutex);
     return;
   }
   this->SoundConverterOpen = (LPFUNC2)GetProcAddress (this->qtml_dll, "SoundConverterOpen");
   if ( this->SoundConverterOpen == NULL ) {
-    printf ("qt_audio: failed getting proc address SoundConverterOpen\n");
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, 
+	     "qt_audio: failed getting proc address SoundConverterOpen\n");
     pthread_mutex_unlock(&win32_codec_mutex);
     return;
   }
   this->SoundConverterClose = (LPFUNC3)GetProcAddress (this->qtml_dll, "SoundConverterClose");
   if ( this->SoundConverterClose == NULL ) {
-    printf ("qt_audio: failed getting proc address SoundConverterClose\n");
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, 
+	     "qt_audio: failed getting proc address SoundConverterClose\n");
     pthread_mutex_unlock(&win32_codec_mutex);
     return;
   }
   this->TerminateQTML = (LPFUNC4)GetProcAddress (this->qtml_dll, "TerminateQTML");
   if ( this->TerminateQTML == NULL ) {
-    printf ("qt_audio: failed getting proc address TerminateQTML\n");
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, 
+	     "qt_audio: failed getting proc address TerminateQTML\n");
     pthread_mutex_unlock(&win32_codec_mutex);
     return;
   }
   this->SoundConverterSetInfo = (LPFUNC5)GetProcAddress (this->qtml_dll, "SoundConverterSetInfo");
   if ( this->SoundConverterSetInfo == NULL ) {
-    printf ("qt_audio: failed getting proc address SoundConverterSetInfo\n");
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, 
+	     "qt_audio: failed getting proc address SoundConverterSetInfo\n");
     pthread_mutex_unlock(&win32_codec_mutex);
     return;
   }
   this->SoundConverterGetBufferSizes = (LPFUNC6)GetProcAddress (this->qtml_dll, "SoundConverterGetBufferSizes");
   if ( this->SoundConverterGetBufferSizes == NULL ) {
-    printf ("qt_audio: failed getting proc address SoundConverterGetBufferSizes\n");
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, 
+	     "qt_audio: failed getting proc address SoundConverterGetBufferSizes\n");
     pthread_mutex_unlock(&win32_codec_mutex);
     return;
   }
   this->SoundConverterConvertBuffer = (LPFUNC7)GetProcAddress (this->qtml_dll, "SoundConverterConvertBuffer");
   if ( this->SoundConverterConvertBuffer == NULL ) {
-    printf ("qt_audio: failed getting proc address SoundConverterConvertBuffer1\n");
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, 
+	     "qt_audio: failed getting proc address SoundConverterConvertBuffer1\n");
     pthread_mutex_unlock(&win32_codec_mutex);
     return;
   }
   this->SoundConverterEndConversion = (LPFUNC8)GetProcAddress (this->qtml_dll, "SoundConverterEndConversion");
   if ( this->SoundConverterEndConversion == NULL ) {
-    printf ("qt_audio: failed getting proc address SoundConverterEndConversion\n");
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, 
+	     "qt_audio: failed getting proc address SoundConverterEndConversion\n");
     pthread_mutex_unlock(&win32_codec_mutex);
     return;
   }
   this->SoundConverterBeginConversion = (LPFUNC9)GetProcAddress (this->qtml_dll, "SoundConverterBeginConversion");
   if ( this->SoundConverterBeginConversion == NULL ) {
-    printf ("qt_audio: failed getting proc address SoundConverterBeginConversion\n");
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, 
+	     "qt_audio: failed getting proc address SoundConverterBeginConversion\n");
     pthread_mutex_unlock(&win32_codec_mutex);
     return;
   }
-#ifdef LOG
-  printf ("qt_audio: Standard init done you may now call supported functions\n");
-#endif
+  lprintf ("audio: Standard init done you may now call supported functions\n");
 
   error = this->InitializeQTML(6+16);
-#ifdef LOG
-  printf ("qt_audio: InitializeQTML:%i\n",error);
-#endif
+  lprintf ("audio: InitializeQTML:%i\n",error);
+
   if (error) {
     pthread_mutex_unlock(&win32_codec_mutex);
     return;
@@ -322,14 +324,23 @@ static void qta_init_driver (qta_decoder_t *this, buf_element_t *buf) {
 
   switch (buf->type) {
   case BUF_AUDIO_QDESIGN1:
-    this->InputFormatInfo.format = FOUR_CHAR_CODE('Q','D','M','1');
+    this->InputFormatInfo.format = FOUR_CHAR_CODE('Q','D','M','C');
+    _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, 
+      "QDesign Music Codec v1 (QT DLL)");
     break;
   case BUF_AUDIO_QDESIGN2:
     this->InputFormatInfo.format = FOUR_CHAR_CODE('Q','D','M','2');
+    _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, 
+      "QDesign Music Codec v2 (QT DLL)");
+    break;
+  case BUF_AUDIO_QCLP:
+    this->InputFormatInfo.format = FOUR_CHAR_CODE('Q','c','l','p');
+    _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, 
+      "Qualcomm Purevoice Codec (QT DLL)");
     break;
   default:
-    printf ("qt_audio: fourcc for buftype %08x ?\n", buf->type);
-    abort ();
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "qt_audio: fourcc for buftype %08x ?\n", buf->type);
+    _x_abort ();
   }
 
   this->OutputFormatInfo.format = FOUR_CHAR_CODE('N','O','N','E');
@@ -346,21 +357,18 @@ static void qta_init_driver (qta_decoder_t *this, buf_element_t *buf) {
   error = this->SoundConverterOpen (&this->InputFormatInfo, 
 				    &this->OutputFormatInfo, 
 				    &this->myConverter);
-#ifdef LOG
-  printf ("qt_audio: SoundConverterOpen:%i\n",error);
-#endif
+  lprintf ("audio: SoundConverterOpen:%i\n",error);
+
   if (error) {
     pthread_mutex_unlock(&win32_codec_mutex);
     return;
   }
 
-  if (buf->decoder_info[2] > 0x48) {
+  if ((buf->decoder_info[2] > 0x38) && (buf->decoder_info[2] != 0x64)) {
     error = this->SoundConverterSetInfo (this->myConverter,
 					 FOUR_CHAR_CODE('w','a','v','e'),
-					 ((unsigned char *)buf->decoder_info_ptr[2]) + 0x48);
-#ifdef LOG
-    printf ("qt_audio: SoundConverterSetInfo:%i\n",error);
-#endif
+					 ((unsigned char *)buf->decoder_info_ptr[2]) + 0x38);
+    lprintf ("audio: SoundConverterSetInfo:%i\n",error);
     if (error) {
       pthread_mutex_unlock(&win32_codec_mutex);
 
@@ -372,55 +380,28 @@ static void qta_init_driver (qta_decoder_t *this, buf_element_t *buf) {
   error = this->SoundConverterGetBufferSizes (this->myConverter,
 					      WantedBufferSize, &this->FramesToGet,
 					      &InputBufferSize, &OutputBufferSize);
-#ifdef LOG
-  printf ("qt_audio: SoundConverterGetBufferSizes:%i\n", error);
-  printf ("qt_audio: WantedBufferSize = %li\n", WantedBufferSize);
-  printf ("qt_audio: InputBufferSize  = %li\n", InputBufferSize);
-  printf ("qt_audio: OutputBufferSize = %li\n", OutputBufferSize);
-  printf ("qt_audio: this->FramesToGet = %li\n", this->FramesToGet);
-#endif    
+  lprintf ("audio: SoundConverterGetBufferSizes:%i\n", error);
+  lprintf ("audio: WantedBufferSize = %li\n", WantedBufferSize);
+  lprintf ("audio: InputBufferSize  = %li\n", InputBufferSize);
+  lprintf ("audio: OutputBufferSize = %li\n", OutputBufferSize);
+  lprintf ("audio: this->FramesToGet = %li\n", this->FramesToGet);
 
   this->InFrameSize   = (InputBufferSize+this->FramesToGet-1)/this->FramesToGet;
   this->OutFrameSize  = OutputBufferSize/this->FramesToGet;
 
-#ifdef LOG
-  printf ("qt_audio: FrameSize: %i -> %i\n", this->InFrameSize, this->OutFrameSize);
-#endif    
+  lprintf ("audio: FrameSize: %i -> %i\n", this->InFrameSize, this->OutFrameSize);
 
   error = this->SoundConverterBeginConversion (this->myConverter);
-#ifdef LOG
-  printf ("qt_audio: SoundConverterBeginConversion:%i\n",error);
-#endif    
+  lprintf ("audio: SoundConverterBeginConversion:%i\n",error);
+
   if (error) {    
     pthread_mutex_unlock(&win32_codec_mutex);
     return;
   }
 
-#ifdef LOG
-  printf ("qt_audio: opening output.\n");
-#endif    
+  lprintf ("audio: opening output.\n");
 
-  switch (this->wave.nChannels) {
-  case 1: 
-    mode = AO_CAP_MODE_MONO;
-    break;
-  case 2: 
-    mode = AO_CAP_MODE_STEREO;
-    break;
-  case 4: 
-    mode = AO_CAP_MODE_4CHANNEL;
-    break;
-  case 5: 
-    mode = AO_CAP_MODE_5CHANNEL;
-    break;
-  case 6: 
-    mode = AO_CAP_MODE_5_1CHANNEL;
-    break;
-  default:
-    printf ("qt_audio: help, %d channels ?!\n",
-	    this->wave.nChannels);
-    abort ();
-  }
+  mode = _x_ao_channels2mode(this->wave.nChannels);
 
   this->frame_size = this->wave.nChannels * this->wave.wBitsPerSample / 8;
 
@@ -432,9 +413,7 @@ static void qta_init_driver (qta_decoder_t *this, buf_element_t *buf) {
 
   this->codec_initialized = 1;
 
-#ifdef LOG
-  printf ("qt_audio: mutex unlock\n");
-#endif    
+  lprintf ("audio: mutex unlock\n");
 
   pthread_mutex_unlock(&win32_codec_mutex);
 }
@@ -443,40 +422,32 @@ static void qta_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 
   qta_decoder_t *this = (qta_decoder_t *) this_gen;
 
-#ifdef LOG
-  printf ("qt_audio: decode buf=%08x %d bytes flags=%08x pts=%lld\n",
-	  buf, buf->size, buf->decoder_flags, buf->pts);
-#endif
+  lprintf ("audio: decode buf=%08x %d bytes flags=%08x pts=%lld\n",
+	   buf, buf->size, buf->decoder_flags, buf->pts);
 
-  if (buf->decoder_flags & BUF_FLAG_HEADER) {
+  if (buf->decoder_flags & BUF_FLAG_STDHEADER) {
 
-    memcpy (&this->wave, buf->content, sizeof (xine_waveformatex));
+    if (buf->size >= sizeof(xine_waveformatex))
+      memcpy (&this->wave, buf->content, sizeof (xine_waveformatex));
 
     this->wave.nChannels      = buf->decoder_info[3];
     this->wave.wBitsPerSample = buf->decoder_info[2];
     this->wave.nSamplesPerSec = buf->decoder_info[1];
 
-#ifdef LOG 
-    printf ("qt_audio: header copied\n");
-#endif
+    lprintf ("audio: header copied\n");
     
   } else if (buf->decoder_flags & BUF_FLAG_SPECIAL) {
-#ifdef LOG 
-    printf ("qt_audio: special buffer\n");
-#endif
+    lprintf ("audio: special buffer\n");
     
     if (buf->decoder_info[1] == BUF_SPECIAL_STSD_ATOM) {
-      
-#ifdef LOG 
-      printf ("qt_audio: got stsd atom -> init codec\n");
-#endif
+      lprintf ("audio: got stsd atom -> init codec\n");
 
       if (!this->codec_initialized) {
 	qta_init_driver (this, buf);
       }
       
       if (!this->codec_initialized)
-        this->stream->stream_info[XINE_STREAM_INFO_AUDIO_HANDLED] = 0;
+        _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_HANDLED, 0);
     }
   } else if( this->codec_initialized ) {
 
@@ -488,6 +459,8 @@ static void qta_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
       int num_frames = this->data_len / this->InFrameSize;
       long out_frames, out_bytes;
       int error, frames_left, bytes_sent;
+    
+      Check_FS_Segment(this->ldt_fs);
 
       pthread_mutex_lock(&win32_codec_mutex);
       error = this->SoundConverterConvertBuffer (this->myConverter,
@@ -497,10 +470,8 @@ static void qta_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 						 &out_frames, &out_bytes);
       pthread_mutex_unlock(&win32_codec_mutex);
 
-#ifdef LOG
-      printf ("qt_audio: decoded %d frames => %d frames (error %d)\n",
-	      num_frames, out_frames, error);
-#endif
+      lprintf ("audio: decoded %d frames => %d frames (error %d)\n",
+	       num_frames, out_frames, error);
 
       this->data_len -= this->InFrameSize * num_frames;
       if (this->data_len>0)
@@ -527,10 +498,7 @@ static void qta_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 	buf->pts = 0;  /* only the first buffer gets the real pts */
 	audio_buffer->num_frames = nframes;
 
-#ifdef LOG
-	printf ("qt_audio: sending %d frames, %d frames left\n",
-		nframes, frames_left);
-#endif
+	lprintf ("audio: sending %d frames, %d frames left\n", nframes, frames_left);
 
 	this->stream->audio_out->put_buffer (this->stream->audio_out, 
 					     audio_buffer, this->stream);
@@ -545,6 +513,9 @@ static void qta_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 }
 
 static void qta_reset (audio_decoder_t *this_gen) {
+  qta_decoder_t *this = (qta_decoder_t *) this_gen;
+  
+  this->data_len = 0;
 }
 
 
@@ -562,13 +533,14 @@ static void qta_dispose (audio_decoder_t *this_gen) {
 
     error = this->SoundConverterEndConversion (this->myConverter,NULL,
 					       &ConvertedFrames,&ConvertedBytes);
-#ifdef LOG
-    printf ("qt_audio: SoundConverterEndConversion:%i\n",error);
-#endif
+    lprintf ("audio: SoundConverterEndConversion:%i\n",error);
+
     error = this->SoundConverterClose (this->myConverter);
-#ifdef LOG
-    printf ("qt_audio: SoundConverterClose:%i\n",error);
-#endif
+
+    lprintf ("audio: SoundConverterClose:%i\n",error);
+
+    Restore_LDT_Keeper(this->ldt_fs);
+    this->ldt_fs = NULL;
   }
 
   if (this->output_open)
@@ -617,31 +589,31 @@ static void *qta_init_class (xine_t *xine, void *data) {
   qta_class_t     *this;
   config_values_t *cfg;
 
+  cfg = xine->config;
+  if ((win32_def_path = get_win32_codecs_path(cfg)) == NULL) return NULL;
+
   pthread_once (&once_control, init_routine);
 
-  this = (qta_class_t *) malloc (sizeof (qta_class_t));
+  this = (qta_class_t *) xine_xmalloc (sizeof (qta_class_t));
 
   this->decoder_class.open_plugin     = qta_open_plugin;
   this->decoder_class.get_identifier  = qta_get_identifier;
   this->decoder_class.get_description = qta_get_description;
   this->decoder_class.dispose         = qta_dispose_class;
 
-  cfg = xine->config;
-  win32_def_path = cfg->register_string (cfg, "codec.win32_path", 
-					 "/usr/lib/win32",
-					 _("path to win32 codec dlls"),
-					 NULL, 0, NULL, NULL);
-
   return this;
 }
 
 static uint32_t audio_types[] = { 
-  BUF_AUDIO_QDESIGN1, BUF_AUDIO_QDESIGN2, 0
+  BUF_AUDIO_QDESIGN1,
+  BUF_AUDIO_QDESIGN2,
+  BUF_AUDIO_QCLP,
+  0
 };
 
-static decoder_info_t qta_dec_info = {
+static const decoder_info_t qta_dec_info = {
   audio_types,         /* supported types */
-  5                    /* priority        */
+  1                    /* priority        */
 };
 
 /*
@@ -669,6 +641,7 @@ typedef struct qtv_decoder_s {
   HINSTANCE        qtml_dll; 
 
   xine_bmiheader   bih;
+  double           ratio;
 
   int              codec_initialized;
 
@@ -731,6 +704,7 @@ typedef struct qtv_decoder_s {
 					  long rowBytes); 
   OSErr             (*NewHandleClear)(Size byteCount);
 
+  ldt_fs_t *ldt_fs;
 } qtv_decoder_t;
 
 #ifdef LOG
@@ -780,57 +754,52 @@ static void qtv_init_driver (qtv_decoder_t *this, buf_element_t *buf) {
   ImageSubCodecDecompressCapabilities icap; /* for ImageCodecInitialize() */
   ImageDescription                   *id;
 
-#ifdef LOG
-  printf ("qt_video: init_driver... (trying to lock mutex...)\n");
-#endif
+  lprintf ("video: init_driver... (trying to lock mutex...)\n");
 
   pthread_mutex_lock(&win32_codec_mutex);
 
-#ifdef LOG
-  printf ("qt_video: mutex locked\n");
-#endif
+  lprintf ("video: mutex locked\n");
 
-  Setup_LDT_Keeper();
+  this->ldt_fs = Setup_LDT_Keeper();
   
   this->qtml_dll = LoadLibraryA("qtmlClient.dll");
   
   if (this->qtml_dll == NULL ) {
-    printf ("qt_video: failed to load dll\n" );
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "qt_video: failed to load dll\n" );
     pthread_mutex_unlock(&win32_codec_mutex);
+    _x_message(this->stream, XINE_MSG_LIBRARY_LOAD_ERROR,
+                 "qtmlClient.dll", NULL);
     return;
   }
 
-  this->InitializeQTML = GetProcAddress (this->qtml_dll, "InitializeQTML");
-  this->EnterMovies = GetProcAddress (this->qtml_dll, "EnterMovies");
-  this->FindNextComponent = GetProcAddress (this->qtml_dll, "FindNextComponent");
-  this->CountComponents = GetProcAddress (this->qtml_dll, "CountComponents");
-  this->GetComponentInfo = GetProcAddress (this->qtml_dll, "GetComponentInfo");
-  this->OpenComponent = GetProcAddress (this->qtml_dll, "OpenComponent");
-  this->ImageCodecInitialize = GetProcAddress (this->qtml_dll, "ImageCodecInitialize");
-  this->ImageCodecGetCodecInfo = GetProcAddress (this->qtml_dll, "ImageCodecGetCodecInfo");
-  this->ImageCodecBeginBand = GetProcAddress (this->qtml_dll, "ImageCodecBeginBand");
-  this->ImageCodecPreDecompress = GetProcAddress (this->qtml_dll, "ImageCodecPreDecompress");
-  this->ImageCodecBandDecompress = GetProcAddress (this->qtml_dll, "ImageCodecBandDecompress");
-  this->GetGWorldPixMap = GetProcAddress (this->qtml_dll, "GetGWorldPixMap");
-  this->QTNewGWorldFromPtr = GetProcAddress (this->qtml_dll, "QTNewGWorldFromPtr");
-  this->NewHandleClear = GetProcAddress (this->qtml_dll, "NewHandleClear");
+  this->InitializeQTML = (OSErr(*)(long))GetProcAddress(this->qtml_dll, "InitializeQTML");
+  this->EnterMovies = (OSErr(*)(void))GetProcAddress(this->qtml_dll, "EnterMovies");
+  this->FindNextComponent = (Component(*)(Component,ComponentDescription*))GetProcAddress(this->qtml_dll, "FindNextComponent");
+  this->CountComponents = (long(*)(ComponentDescription*))GetProcAddress(this->qtml_dll, "CountComponents");
+  this->GetComponentInfo = (OSErr(*)(Component,ComponentDescription*,Handle,Handle,Handle))GetProcAddress(this->qtml_dll, "GetComponentInfo");
+  this->OpenComponent = (ComponentInstance(*)(Component))GetProcAddress(this->qtml_dll, "OpenComponent");
+  this->ImageCodecInitialize = (ComponentResult(*)(ComponentInstance,ImageSubCodecDecompressCapabilities*))GetProcAddress(this->qtml_dll, "ImageCodecInitialize");
+  this->ImageCodecGetCodecInfo = (ComponentResult(*)(ComponentInstance,CodecInfo*))GetProcAddress(this->qtml_dll, "ImageCodecGetCodecInfo");
+  this->ImageCodecBeginBand = (ComponentResult(*)(ComponentInstance,CodecDecompressParams*,ImageSubCodecDecompressRecord*,long))GetProcAddress(this->qtml_dll, "ImageCodecBeginBand");
+  this->ImageCodecPreDecompress = (ComponentResult(*)(ComponentInstance,CodecDecompressParams*))GetProcAddress(this->qtml_dll, "ImageCodecPreDecompress");
+  this->ImageCodecBandDecompress = (ComponentResult(*)(ComponentInstance,CodecDecompressParams*))GetProcAddress(this->qtml_dll, "ImageCodecBandDecompress");
+  this->GetGWorldPixMap = (PixMapHandle(*)(GWorldPtr))GetProcAddress(this->qtml_dll, "GetGWorldPixMap");
+  this->QTNewGWorldFromPtr = (OSErr(*)(GWorldPtr*,OSType,const Rect*,CTabHandle,void*,GWorldFlags,void*,long))GetProcAddress(this->qtml_dll, "QTNewGWorldFromPtr");
+  this->NewHandleClear = (OSErr(*)(Size))GetProcAddress(this->qtml_dll, "NewHandleClear");
     
   if (!this->InitializeQTML || !this->EnterMovies || !this->FindNextComponent 
       || !this->ImageCodecBandDecompress){
-    printf ("qt_video: invalid qt DLL!\n");
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "qt_video: invalid qt DLL!\n");
     pthread_mutex_unlock(&win32_codec_mutex);
     return;
   }
 
-#ifdef LOG
-  printf ("qt_video: calling InitializeQTML...\n");
-#endif
+  lprintf ("video: calling InitializeQTML...\n");
 
   result = this->InitializeQTML(6+16);
   /*    result=InitializeQTML(0); */
-#ifdef LOG
-  printf("qt_video: InitializeQTML returned %d\n",result);
-#endif
+  lprintf("video: InitializeQTML returned %d\n",result);
+
   /*    result=EnterMovies(); */
   /*    printf("EnterMovies->%d\n",result); */
 
@@ -842,38 +811,33 @@ static void qtv_init_driver (qtv_decoder_t *this, buf_element_t *buf) {
   desc.componentFlags=0;
   desc.componentFlagsMask=0;
 
-#ifdef LOG
-  printf("qt_video: Count = %d\n", this->CountComponents(&desc));
-#endif
+  lprintf("video: Count = %d\n", this->CountComponents(&desc));
 
   prev = this->FindNextComponent(NULL,&desc);
   if(!prev){
-    printf("Cannot find requested component\n");
+    xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "Cannot find requested component\n");
     pthread_mutex_unlock(&win32_codec_mutex);
     return;
   }
-#ifdef LOG
-  printf ("qt_video: Found it! ID = 0x%X\n",prev);
-#endif
+
+  lprintf ("video: Found it! ID = 0x%X\n",prev);
 
   this->ci = this->OpenComponent(prev);
-  printf ("qt_video: this->ci=%p\n",this->ci);
+  lprintf ("video: this->ci=%p\n",this->ci);
 
   memset (&icap,0,sizeof(icap));
   cres = this->ImageCodecInitialize (this->ci, &icap);
-#ifdef LOG
-  printf ("qt_video: ImageCodecInitialize->%p  size=%d (%d)\n",
-	  cres,icap.recordSize,icap.decompressRecordSize);
-#endif
+
+  lprintf ("video: ImageCodecInitialize->%p  size=%d (%d)\n",
+	   cres,icap.recordSize,icap.decompressRecordSize);
 
   memset(&cinfo,0,sizeof(cinfo));
   cres = this->ImageCodecGetCodecInfo (this->ci, &cinfo);
-#ifdef LOG
-  printf ("qt_video: Flags: compr: 0x%X  decomp: 0x%X format: 0x%X\n",
-	  cinfo.compressFlags, cinfo.decompressFlags, cinfo.formatFlags);
-  printf ("qt_video: Codec name: %.*s\n", ((unsigned char*)&cinfo.typeName)[0],
-	  ((unsigned char*)&cinfo.typeName)+1);
-#endif
+
+  lprintf ("video: Flags: compr: 0x%X  decomp: 0x%X format: 0x%X\n",
+	   cinfo.compressFlags, cinfo.decompressFlags, cinfo.formatFlags);
+  lprintf ("video: Codec name: %.*s\n", ((unsigned char*)&cinfo.typeName)[0],
+	   ((unsigned char*)&cinfo.typeName)+1);
 
   /* make a yuy2 gworld */
   this->OutBufferRect.top    = 0;
@@ -881,61 +845,56 @@ static void qtv_init_driver (qtv_decoder_t *this, buf_element_t *buf) {
   this->OutBufferRect.right  = this->bih.biWidth;
   this->OutBufferRect.bottom = this->bih.biHeight;
 
+  lprintf ("video: image size %d x %d\n",
+	   this->bih.biWidth,  this->bih.biHeight);
+
+  lprintf ("video: stsd (%d bytes):\n", buf->decoder_info[2]) ;
 #ifdef LOG
-  printf ("qt_video: image size %d x %d\n",
-	  this->bih.biWidth,  this->bih.biHeight);
-
-  printf ("qt_video: stsd (%d bytes):\n", buf->decoder_info[2]) ;
-
   qtv_hexdump ((unsigned char *)buf->decoder_info_ptr[2], buf->decoder_info[2]);
 #endif
 
   {
-    uint8_t *stdata = ((unsigned char *)buf->decoder_info_ptr[2]) + 20;
-    int      stdata_len = buf->decoder_info[2] - 24;
+    uint8_t *stdata = ((unsigned char *)buf->decoder_info_ptr[2]);
+    int      stdata_len = buf->decoder_info[2];
 
     id=malloc (8+stdata_len) ; /* trak->stdata_len); */
-    id->idSize          = 8+stdata_len;
+    id->idSize          = 4+stdata_len;
     id->cType           = FOUR_CHAR_CODE('S','V','Q','3');
-    id->version         = BE_16 (stdata+ 8);
-    id->revisionLevel   = BE_16 (stdata+10);
-    id->vendor          = BE_32 (stdata+12);
-    id->temporalQuality = BE_32 (stdata+16);
-    id->spatialQuality  = BE_32 (stdata+20);
-    id->width           = BE_16 (stdata+24);
-    id->height          = BE_16 (stdata+26);
-    id->hRes            = BE_32 (stdata+28);
-    id->vRes            = BE_32 (stdata+32);
-    id->dataSize        = BE_32 (stdata+36);
-    id->frameCount      = BE_16 (stdata+40);
-    memcpy(&id->name,stdata+42,32);
-    id->depth           = BE_16 (stdata+74);
-    id->clutID          = BE_16 (stdata+76);
-    if (stdata_len>78)	
-      memcpy (((char*)&id->clutID)+2, stdata+78, stdata_len-78);
+    id->version         = BE_16 (stdata+0x08);
+    id->revisionLevel   = BE_16 (stdata+0x0C);
+    id->vendor          = BE_32 (stdata+0x10);
+    id->temporalQuality = BE_32 (stdata+0x14);
+    id->spatialQuality  = BE_32 (stdata+0x18);
+    id->width           = BE_16 (stdata+0x1C);
+    id->height          = BE_16 (stdata+0x1E);
+    id->hRes            = BE_32 (stdata+0x20);
+    id->vRes            = BE_32 (stdata+0x24);
+    id->dataSize        = BE_32 (stdata+0x28);
+    id->frameCount      = BE_16 (stdata+0x2C);
+    memcpy(&id->name,stdata+0x2D,32);
+    id->depth           = BE_16 (stdata+0x4E);
+    id->clutID          = BE_16 (stdata+0x50);
+    if (stdata_len>0x56)	
+      memcpy (((char*)&id->clutID)+2, stdata+0x52, stdata_len-0x52);
 
+    lprintf ("video: id (%d bytes)\n", stdata_len);
 #ifdef LOG
-    printf ("qt_video: id (%d bytes)\n", stdata_len);
     qtv_hexdump (id, stdata_len);
 #endif
 
   }
 
+  lprintf ("video: ImageDescription size: %d\n", id->idSize);
 #ifdef LOG
-  printf ("qt_video: ImageDescription size: %d\n",
-	  id->idSize);
-
   qtv_hexdump (id, id->idSize);
 #endif
 
   this->framedescHandle = (ImageDescriptionHandle) this->NewHandleClear (id->idSize);
 
-#ifdef LOG
-  printf ("qt_video: framedescHandle = %x\n",
-	  this->framedescHandle);
-#endif
+  lprintf ("video: framedescHandle = %x\n", this->framedescHandle);
 
   memcpy (*this->framedescHandle, id, id->idSize);
+  free(id);
 
   /*
    * alloc video plane
@@ -952,10 +911,7 @@ static void qtv_init_driver (qtv_decoder_t *this, buf_element_t *buf) {
 				    this->plane,
 				    this->bih.biWidth*2);
 
-#ifdef LOG
-  printf ("qt_video: NewGWorldFromPtr returned:%d\n",
-	  65536-(result&0xffff));
-#endif
+  lprintf ("video: NewGWorldFromPtr returned:%d\n", 65536-(result&0xffff));
 
   this->decpar.imageDescription = this->framedescHandle;
   this->decpar.startLine        = 0;
@@ -966,16 +922,15 @@ static void qtv_init_driver (qtv_decoder_t *this, buf_element_t *buf) {
   this->decpar.matrix           = 0;
   this->decpar.capabilities     = &this->codeccap;
   this->decpar.accuracy         = codecNormalQuality;
-  this->decpar.port             = this->OutBufferGWorld;
+  this->decpar.port             = (CGrafPtr)this->OutBufferGWorld;
   this->decpar.srcRect          = this->OutBufferRect;
       
   this->decpar.transferMode     = srcCopy;
   this->decpar.dstPixMap        = **this->GetGWorldPixMap (this->OutBufferGWorld);/*destPixmap;  */
       
   cres = this->ImageCodecPreDecompress (this->ci, &this->decpar);
-#ifdef LOG
-  printf ("qt_video: ImageCodecPreDecompress cres=0x%X\n", cres);
-#endif
+
+  lprintf ("video: ImageCodecPreDecompress cres=0x%X\n", cres);
       
   this->data_len = 0;
 
@@ -990,53 +945,45 @@ static void qtv_init_driver (qtv_decoder_t *this, buf_element_t *buf) {
 static void qtv_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
   qtv_decoder_t *this = (qtv_decoder_t *) this_gen;
 
-#ifdef LOG
-  printf ("qt_video: decode_data, flags=0x%08x, len=%d, pts=%lld ...\n", 
-	  buf->decoder_flags, buf->size, buf->pts);
-#endif
+  lprintf ("video: decode_data, flags=0x%08x, len=%d, pts=%lld ...\n", 
+	   buf->decoder_flags, buf->size, buf->pts);
 
-  if (buf->decoder_flags & BUF_FLAG_HEADER) {
-
-#ifdef LOG 
-    printf ("qt_video: copying bih\n");
-#endif
+  if (buf->decoder_flags & BUF_FLAG_STDHEADER) {
+    lprintf ("video: copying bih\n");
 
     memcpy (&this->bih, buf->content, sizeof (xine_bmiheader));
+    this->ratio = (double)this->bih.biWidth / (double)this->bih.biHeight;
+
+    /* video decoder only handles SVQ3 at this point */
+    _x_meta_info_set_utf8(this->stream, XINE_META_INFO_VIDEOCODEC, "Sorenson Video 3 (QT DLL)");
 
   } else if (buf->decoder_flags & BUF_FLAG_SPECIAL) {
-#ifdef LOG 
-    printf ("qt_video: special buffer\n");
-#endif
+    lprintf ("video: special buffer\n");
     
     if (buf->decoder_info[1] == BUF_SPECIAL_STSD_ATOM) {
-      
-#ifdef LOG 
-      printf ("qt_video: got stsd atom -> init codec\n");
-#endif
+      lprintf ("video: got stsd atom -> init codec\n");
 
       if (!this->codec_initialized) {
 	qtv_init_driver (this, buf);
       }
       if (!this->codec_initialized)
-        this->stream->stream_info[XINE_STREAM_INFO_VIDEO_HANDLED] = 0;
+        _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_HANDLED, 0);
     }
   } else if (this->codec_initialized) {
 
-#ifdef LOG 
-    printf ("qt_video: actual image data\n");
-#endif
+    lprintf ("video: actual image data\n");
 
     memcpy (&this->data[this->data_len], buf->content, buf->size);
     this->data_len += buf->size;
 
-#ifdef LOG 
-    printf ("qt_video: got %d bytes in buffer\n", this->data_len);
-#endif
+    lprintf ("video: got %d bytes in buffer\n", this->data_len);
     
     if (buf->decoder_flags & BUF_FLAG_FRAME_END) {
 
       ComponentResult  cres;
       vo_frame_t      *img;
+    
+      Check_FS_Segment(this->ldt_fs);
 
       pthread_mutex_lock(&win32_codec_mutex);
 
@@ -1051,14 +998,14 @@ static void qtv_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
       pthread_mutex_unlock(&win32_codec_mutex);
 
       if (cres&0xFFFF){
-	printf("qt_video: ImageCodecBandDecompress cres=0x%X (-0x%X) %d :(\n",
-	       cres,-cres,cres);
+	xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
+		"qt_video: ImageCodecBandDecompress cres=0x%lX (-0x%lX) %ld :(\n", cres,-cres,cres);
       }
 
       img = this->stream->video_out->get_frame (this->stream->video_out,
 						this->bih.biWidth,
 						this->bih.biHeight,
-						42, 
+						this->ratio, 
 						XINE_IMGFMT_YUY2,
 						VO_BOTH_FIELDS);
 	
@@ -1076,23 +1023,19 @@ static void qtv_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
     }
   }
 
-#ifdef LOG
-  printf ("qt_video: decode_data...done\n");
-#endif
+  lprintf ("video: decode_data...done\n");
 }
 
 static void qtv_flush (video_decoder_t *this_gen) {
   /* qtv_decoder_t *this = (qtv_decoder_t *) this_gen; */
 
-#ifdef LOG
-  printf ("qt_video: flush\n");
-#endif
-
+  lprintf ("video: flush\n");
 }
 
 static void qtv_reset (video_decoder_t *this_gen) {
-  /* qtv_decoder_t *this = (qtv_decoder_t *) this_gen; */
-
+  qtv_decoder_t *this = (qtv_decoder_t *) this_gen;
+  
+  this->data_len = 0;
 }
 
 static void qtv_discontinuity (video_decoder_t *this_gen) {
@@ -1107,11 +1050,11 @@ static void qtv_dispose (video_decoder_t *this_gen) {
   if (this->codec_initialized) {
     this->stream->video_out->close(this->stream->video_out, this->stream);
     this->codec_initialized = 0;
+    Restore_LDT_Keeper(this->ldt_fs);
+    this->ldt_fs = NULL;
   }
 
-#ifdef LOG
-  printf ("qt_video: dispose\n");
-#endif
+  lprintf ("video: dispose\n");
 
   free (this);
 }
@@ -1167,15 +1110,11 @@ static void codec_path_cb (void *data, xine_cfg_entry_t *cfg) {
 static void *qtv_init_class (xine_t *xine, void *data) {
 
   qtv_class_t        *this;
-  config_values_t    *cfg = xine->config; 
+  config_values_t    *cfg = xine->config;
 
-  win32_def_path = cfg->register_string (cfg, "codec.win32_path", "/usr/lib/win32",
-					 _("path to win32 codec dlls"),
-					 NULL, 0, NULL, NULL);
+  if ((win32_def_path = get_win32_codecs_path(cfg)) == NULL) return NULL;
 
-#ifdef LOG
-  printf ("qtv_init_class...\n");
-#endif
+  lprintf ("%s...\n", __XINE_FUNCTION__);
 
   pthread_once (&once_control, init_routine);
 
@@ -1195,15 +1134,14 @@ static void *qtv_init_class (xine_t *xine, void *data) {
 
 static uint32_t qtv_supported_types[] = { BUF_VIDEO_SORENSON_V3, 0 };
 
-static decoder_info_t qtv_dec_info = {
+static const decoder_info_t qtv_dec_info = {
   qtv_supported_types,     /* supported types */
-  6                        /* priority        */
+  1                        /* priority        */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_VIDEO_DECODER | PLUGIN_MUST_PRELOAD, 14, "qtv", XINE_VERSION_CODE, &qtv_dec_info, qtv_init_class },
-  { PLUGIN_AUDIO_DECODER | PLUGIN_MUST_PRELOAD, 13, "qta", XINE_VERSION_CODE, &qta_dec_info, qta_init_class },
+  { PLUGIN_VIDEO_DECODER | PLUGIN_MUST_PRELOAD, 18, "qtv", XINE_VERSION_CODE, &qtv_dec_info, qtv_init_class },
+  { PLUGIN_AUDIO_DECODER | PLUGIN_MUST_PRELOAD, 15, "qta", XINE_VERSION_CODE, &qta_dec_info, qta_init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
-
