@@ -21,7 +21,7 @@
  * player. It really works too! It will output a continuous sine wave in
  * place of the data it should actually send.
  *
- * $Id: fooaudio.c,v 1.14 2004/12/16 13:59:12 mroi Exp $
+ * $Id: fooaudio.c,v 1.17 2006/06/02 22:13:45 dsalt Exp $
  */
 
 #include <stdio.h>
@@ -116,7 +116,7 @@ static void fooaudio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
 /*      this->bits_per_sample, */
       16,
       this->sample_rate,
-/*      (this->channels == 2) ? AO_CAP_MODE_STEREO : AO_CAP_MODE_MONO);*/
+/*      _x_ao_channels2mode(this->channels));*/
       AO_CAP_MODE_MONO);
   }
 
@@ -335,14 +335,14 @@ static uint32_t audio_types[] = {
  * the priority that the plugin should be given with respect to other
  * plugins that handle the same buffer type. A plugin with priority (n+1) 
  * will be used instead of a plugin with priority (n). */
-static decoder_info_t dec_info_audio = {
+static const decoder_info_t dec_info_audio = {
   audio_types,         /* supported types */
   5                    /* priority        */
 };
 
 /* The plugin catalog entry. This is the only information that this plugin
  * will export to the public. */
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* { type, API version, "name", version, special_info, init_function }, */
   { PLUGIN_AUDIO_DECODER, 15, "fooaudio", XINE_VERSION_CODE, &dec_info_audio, &init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
