@@ -21,7 +21,7 @@
  * Actually, this decoder just reorganizes chunks of raw YUV data in such
  * a way that xine can display them.
  * 
- * $Id: yuv.c,v 1.34 2004/12/16 13:59:12 mroi Exp $
+ * $Id: yuv.c,v 1.36 2006/06/02 22:18:58 dsalt Exp $
  */
 
 #include <stdio.h>
@@ -371,7 +371,7 @@ static void *init_plugin (xine_t *xine, void *data) {
  * exported plugin catalog entry
  */
 
-static uint32_t video_types[] = { 
+static const uint32_t video_types[] = { 
   BUF_VIDEO_YUY2, 
   BUF_VIDEO_YV12,
   BUF_VIDEO_YVU9,
@@ -380,12 +380,12 @@ static uint32_t video_types[] = {
   0
  };
 
-static decoder_info_t dec_info_video = {
+static const decoder_info_t dec_info_video = {
   video_types,         /* supported types */
   1                    /* priority        */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_VIDEO_DECODER, 18, "yuv", XINE_VERSION_CODE, &dec_info_video, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
