@@ -23,7 +23,7 @@
  * process. It simply paints the screen a solid color and rotates through
  * colors on each iteration.
  *
- * $Id: fooviz.c,v 1.23 2004/05/29 14:45:26 mroi Exp $
+ * $Id: fooviz.c,v 1.26 2006/06/02 22:18:58 dsalt Exp $
  *
  */
 
@@ -270,7 +270,7 @@ static post_plugin_t *fooviz_open_plugin(post_class_t *class_gen, int inputs,
   outputv->xine_out.data   = (xine_video_port_t **)&this->vo_port;
   outputv->xine_out.rewire = fooviz_rewire_video;
   outputv->post            = &this->post;
-  xine_list_append_content(this->post.output, outputv);
+  xine_list_push_back(this->post.output, outputv);
   
   this->post.xine_post.audio_input[0] = &port->new_port;
   
@@ -313,9 +313,9 @@ static void *fooviz_init_plugin(xine_t *xine, void *data)
 }
 
 /* plugin catalog information */
-post_info_t fooviz_special_info = { XINE_POST_TYPE_AUDIO_VISUALIZATION };
+static const post_info_t fooviz_special_info = { XINE_POST_TYPE_AUDIO_VISUALIZATION };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_POST, 9, "fooviz", XINE_VERSION_CODE, &fooviz_special_info, &fooviz_init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
