@@ -20,7 +20,7 @@
  * Basic Oscilloscope Visualization Post Plugin For xine
  *   by Mike Melanson (melanson@pcisys.net)
  *
- * $Id: oscope.c,v 1.19 2004/05/29 14:45:26 mroi Exp $
+ * $Id: oscope.c,v 1.20 2006/01/27 07:46:14 tmattern Exp $
  *
  */
 
@@ -341,7 +341,7 @@ static post_plugin_t *oscope_open_plugin(post_class_t *class_gen, int inputs,
   outputv->xine_out.data   = (xine_video_port_t **)&this->vo_port;
   outputv->xine_out.rewire = oscope_rewire_video;
   outputv->post            = &this->post;
-  xine_list_append_content(this->post.output, outputv);
+  xine_list_push_back(this->post.output, outputv);
   
   this->post.xine_post.audio_input[0] = &port->new_port;
   
