@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_out_xvmc.c,v 1.20.2.2 2005/07/17 21:50:07 miguelfreitas Exp $
+ * $Id: video_out_xvmc.c,v 1.27 2006/06/02 22:18:59 dsalt Exp $
  * 
  * video_out_xvmc.c, X11 video motion compensation extension interface for xine
  *
@@ -73,7 +73,6 @@
 #include "accel_xvmc.h"
 
 /* TODO - delete these? */
-#include "alphablend.h"
 #include "deinterlace.h"
 
 #include "xineutils.h"
@@ -837,6 +836,9 @@ static void xvmc_overlay_blend (vo_driver_t *this_gen,
   xvmc_frame_t   *frame = (xvmc_frame_t *) frame_gen;
 
   lprintf ("xvmc_overlay_blend\n");
+  
+  this->alphablend_extra_data.offset_x = frame_gen->overlay_offset_x;
+  this->alphablend_extra_data.offset_y = frame_gen->overlay_offset_y;
 
   /* Alpha Blend here
    * As XV drivers improve to support Hardware overlay, we will change this function.
@@ -844,11 +846,11 @@ static void xvmc_overlay_blend (vo_driver_t *this_gen,
   
   if (overlay->rle) {
     if (frame->format == XINE_IMGFMT_YV12)
-      blend_yuv(frame->vo_frame.base, overlay, 
+      _x_blend_yuv(frame->vo_frame.base, overlay, 
 		frame->width, frame->height, frame->vo_frame.pitches,
                 &this->alphablend_extra_data);
     else if (frame->format != XINE_IMGFMT_XVMC)
-      blend_yuy2(frame->vo_frame.base[0], overlay, 
+      _x_blend_yuy2(frame->vo_frame.base[0], overlay, 
 		 frame->width, frame->height, frame->vo_frame.pitches[0],
                  &this->alphablend_extra_data);
   }
@@ -1784,7 +1786,7 @@ static void *init_class (xine_t *xine, void *visual_gen) {
   return this;
 }
 
-static vo_info_t vo_info_xvmc = {
+static const vo_info_t vo_info_xvmc = {
   /* priority must be low until it supports displaying non-accelerated stuff */
   0,                    /* priority    */
   XINE_VISUAL_TYPE_X11  /* visual type */
@@ -1794,9 +1796,9 @@ static vo_info_t vo_info_xvmc = {
  * exported plugin catalog entry
  */
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_VIDEO_OUT, 20, "xvmc", XINE_VERSION_CODE, &vo_info_xvmc, init_class },
+  { PLUGIN_VIDEO_OUT, 21, "xvmc", XINE_VERSION_CODE, &vo_info_xvmc, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
 
