@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2005 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: buffer_types.c,v 1.52 2003/01/23 16:12:19 miguelfreitas Exp $
+ * $Id: buffer_types.c,v 1.104 2006/03/06 00:19:50 hadess Exp $
  *
  *
  * contents:
@@ -40,17 +40,9 @@
 
 /* FOURCC will be manipulated using machine endian */
 #ifdef WORDS_BIGENDIAN
-#define meFOURCC( ch0, ch1, ch2, ch3 )              \
-        ( (uint32_t)(unsigned char)(ch3) |          \
-        ( (uint32_t)(unsigned char)(ch2) << 8 ) |   \
-        ( (uint32_t)(unsigned char)(ch1) << 16 ) |  \
-        ( (uint32_t)(unsigned char)(ch0) << 24 ) )
+#define meFOURCC BE_FOURCC
 #else
-#define meFOURCC( ch0, ch1, ch2, ch3 )              \
-        ( (uint32_t)(unsigned char)(ch0) |          \
-        ( (uint32_t)(unsigned char)(ch1) << 8 ) |   \
-        ( (uint32_t)(unsigned char)(ch2) << 16 ) |  \
-        ( (uint32_t)(unsigned char)(ch3) << 24 ) )
+#define meFOURCC LE_FOURCC
 #endif
 
 
@@ -71,7 +63,10 @@ static video_db_t video_db[] = {
 {
   {
     meFOURCC('m', 'p', 'e', 'g'),
+    meFOURCC('M', 'P', 'E', 'G'),
     meFOURCC('P', 'I', 'M', '1'),
+    meFOURCC('m', 'p', 'g', '2'),
+    meFOURCC('m', 'p', 'g', '1'),
     0
   },
   BUF_VIDEO_MPEG,
@@ -87,18 +82,20 @@ static video_db_t video_db[] = {
     meFOURCC('m', 'p', '4', 'v'),
     meFOURCC('M', '4', 'S', '2'),
     meFOURCC('m', '4', 's', '2'),
+    meFOURCC('F', 'M', 'P', '4'),
     0
   },
   BUF_VIDEO_MPEG4,
-  "ISO-MPEG4/OpenDivx format"
+  "ISO-MPEG4/OpenDivx"
 },
 {
   {
     meFOURCC('X', 'V', 'I', 'D'),
+    meFOURCC('x', 'v', 'i', 'd'),
     0
   },
   BUF_VIDEO_XVID,
-  "XviD format"
+  "XviD"
 },
 {
   {
@@ -106,7 +103,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_DIVX5,
-  "DivX 5 format"
+  "DivX 5"
 },
 {
   {
@@ -114,7 +111,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_CINEPAK,
-  "Cinepak format"
+  "Cinepak"
 },
 {
   {
@@ -144,7 +141,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_MSMPEG4_V1,
-  "Microsoft MPEG-4 format v1"
+  "Microsoft MPEG-4 v1"
 },
 {
   {
@@ -157,7 +154,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_MSMPEG4_V2,
-  "Microsoft MPEG-4 format v2"
+  "Microsoft MPEG-4 v2"
 },
 {
   {
@@ -174,14 +171,16 @@ static video_db_t video_db[] = {
     meFOURCC('A', 'P', '4', '1'),
     meFOURCC('M', 'P', 'G', '3'),
     meFOURCC('C', 'O', 'L', '1'),
+    meFOURCC('3', 'I', 'V', 'D'),
     0
   },
   BUF_VIDEO_MSMPEG4_V3,
-  "Microsoft MPEG-4 format v3"
+  "Microsoft MPEG-4 v3"
 },
 {
   {
     meFOURCC('3', 'I', 'V', '1'),
+    meFOURCC('3', 'I', 'V', '2'),
     0
   },
   BUF_VIDEO_3IVX,
@@ -192,11 +191,20 @@ static video_db_t video_db[] = {
     meFOURCC('d', 'm', 'b', '1'),
     meFOURCC('M', 'J', 'P', 'G'),
     meFOURCC('m', 'j', 'p', 'a'),
-    meFOURCC('m', 'j', 'p', 'b'),
+    meFOURCC('A', 'V', 'R', 'n'),
+    meFOURCC('A', 'V', 'D', 'J'),
     0
   },
   BUF_VIDEO_MJPEG,
-  "motion jpeg format"
+  "Motion JPEG"
+},
+{
+  {
+    meFOURCC('m', 'j', 'p', 'b'),
+    0
+  },
+  BUF_VIDEO_MJPEG_B,
+  "Motion JPEG B"
 },
 {
   {
@@ -205,7 +213,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_IV50,
-  "Indeo Video 5.0 format"
+  "Indeo Video 5.0"
 },
 {
   {
@@ -214,7 +222,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_IV41,
-  "Indeo Video 4.1 format"
+  "Indeo Video 4.1"
 },
 {
   {
@@ -223,7 +231,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_IV32,
-  "Indeo Video 3.2 format"
+  "Indeo Video 3.2"
 },
 {
   {
@@ -232,7 +240,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_IV31,
-  "Indeo Video 3.1 format"
+  "Indeo Video 3.1"
 },
 {
   {
@@ -240,7 +248,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_ATIVCR1,
-  "ATI VCR1 format"
+  "ATI VCR1"
 },
 {
   {
@@ -248,7 +256,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_ATIVCR2,
-  "ATI VCR2 format"
+  "ATI VCR2"
 },
 {
   {
@@ -260,7 +268,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_I263,
-  "I263 format"
+  "I263"
 },
 {
   {
@@ -277,7 +285,8 @@ static video_db_t video_db[] = {
     else if (!strncasecmp (video, "yuv2", 4))
     this->video_type = BUF_VIDEO_YUY2;
     */
-    meFOURCC('y','u','v','2'),
+    meFOURCC('y', 'u', 'v', '2'),
+    meFOURCC('Y', 'U', 'Y', '2'),
     0
   },
   BUF_VIDEO_YUY2,
@@ -290,7 +299,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_JPEG,
-  "jpeg"
+  "JPEG"
 },
 {
   {
@@ -311,6 +320,7 @@ static video_db_t video_db[] = {
 {
   {
     meFOURCC('W','M','V','3'),
+    meFOURCC('W','M','V','P'),
     0
   },
   BUF_VIDEO_WMV9,
@@ -341,6 +351,7 @@ static video_db_t video_db[] = {
 },
 {
   {
+    meFOURCC('V','P','3',' '),
     meFOURCC('V','P','3','0'),
     meFOURCC('v','p','3','0'),
     meFOURCC('V','P','3','1'),
@@ -348,17 +359,26 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_VP31,
-  "On2 VP3.1 Codec"
+  "On2 VP3.1"
+},
+{
+  {
+    meFOURCC('V','P','4','0'),
+    0,
+  },
+  BUF_VIDEO_VP4,
+  "On2 VP4"
 },
 {
   {
     meFOURCC('H', '2', '6', '3'),
     meFOURCC('h', '2', '6', '3'),
     meFOURCC('U', '2', '6', '3'),
+    meFOURCC('s', '2', '6', '3'),
     0
   },
   BUF_VIDEO_H263,
-  "H263 format"
+  "H263"
 },
 {
   {
@@ -367,7 +387,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_CYUV,
-  "Creative YUV format"
+  "Creative YUV"
 },
 {
   {
@@ -384,7 +404,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_RPZA,
-  "Apple Quicktime Video (RPZA)"
+  "Apple Quicktime (RPZA)"
 },
 {
   {
@@ -423,7 +443,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_PGVV,
-  "Radius Studio Codec"
+  "Radius Studio"
 },
 {
   {
@@ -455,7 +475,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_GREY,
-  "Raw greyscale"
+  "Raw Greyscale"
 },
 {
   {
@@ -467,6 +487,24 @@ static video_db_t video_db[] = {
   BUF_VIDEO_XXAN,
   "Wing Commander IV Video Codec"
 },
+{
+  {
+    meFOURCC('Y', 'V', '1', '2'),
+    meFOURCC('y', 'v', '1', '2'),
+    0
+  },
+  BUF_VIDEO_YV12,
+  "Raw Planar YV12"
+},
+{
+  {
+    meFOURCC('I', '4', '2', '0'),
+    meFOURCC('I', 'Y', 'U', 'V'),
+    0
+  },
+  BUF_VIDEO_I420,
+  "Raw Planar I420"
+},
 {
   {
     meFOURCC('S', 'E', 'G', 'A'),
@@ -526,6 +564,181 @@ static video_db_t video_db[] = {
   BUF_VIDEO_RV40,
   "Real Video 4.0"
 },
+{
+  {
+    meFOURCC('H', 'F', 'Y', 'U'),
+    0,
+  },
+  BUF_VIDEO_HUFFYUV,
+  "HuffYUV"
+},
+{
+  {
+    meFOURCC('I', 'M', 'G', ' '),
+    meFOURCC('g', 'i', 'f', ' '),
+    0,
+  },
+  BUF_VIDEO_IMAGE,
+  "Image"
+},
+{
+  {
+    0,
+  },
+  BUF_VIDEO_THEORA,
+  "Ogg Theora"
+},
+{
+  {
+    meFOURCC('V','P','5','0'),
+    0
+  },
+  BUF_VIDEO_VP5,
+  "On2 VP5"
+},
+{
+  {
+    meFOURCC('V','P','6','0'),
+    meFOURCC('V','P','6','1'),
+    meFOURCC('V','P','6','2'),
+    0
+  },
+  BUF_VIDEO_VP6,
+  "On2 VP6"
+},
+{
+  {
+    meFOURCC('8','B', 'P','S'),
+    0
+  },
+  BUF_VIDEO_8BPS,
+  "Planar RGB"
+},
+{
+  {
+    meFOURCC('Z','L','I','B'),
+    0
+  },
+  BUF_VIDEO_ZLIB,
+  "ZLIB Video"
+},
+{
+  {
+    meFOURCC('M','S','Z','H'),
+    0
+  },
+  BUF_VIDEO_MSZH,
+  "MSZH Video"
+},
+{
+  {
+    meFOURCC('A','S','V','1'),
+    0
+  },
+  BUF_VIDEO_ASV1,
+  "ASV v1 Video"
+},
+{
+  {
+    meFOURCC('A','S','V','2'),
+    0
+  },
+  BUF_VIDEO_ASV2,
+  "ASV v2 Video"
+},
+{
+  {
+    meFOURCC('a','v','c','1'),
+    meFOURCC('h','2','6','4'),
+    meFOURCC('H','2','6','4'),
+    0
+  },
+  BUF_VIDEO_H264,
+  "Advanced Video Coding (H264)"
+},
+{
+  {
+    meFOURCC('A','A','S','C'),
+    0
+  },
+  BUF_VIDEO_AASC,
+  "Autodesk Animator Studio Codec"
+},
+{
+  {
+    meFOURCC('q','d','r','w'),
+    0
+  },
+  BUF_VIDEO_QDRW,
+  "QuickDraw"
+},
+{
+  {
+    meFOURCC('L','O','C','O'),
+    0
+  },
+  BUF_VIDEO_LOCO,
+  "LOCO"
+},
+{
+  {
+    meFOURCC('U','L','T','I'),
+    0
+  },
+  BUF_VIDEO_ULTI,
+  "IBM UltiMotion"
+},
+{
+  {
+    meFOURCC('W','N','V','1'),
+    0
+  },
+  BUF_VIDEO_WNV1,
+  "Winnow Video"
+},
+{
+  {
+    meFOURCC('P','I','X','L'),
+    meFOURCC('X','I','X','L'),
+    0
+  },
+  BUF_VIDEO_XL,
+  "Miro/Pinnacle VideoXL"
+},
+{
+  {
+    meFOURCC('Q','P','E','G'),
+    meFOURCC('Q','1','.','0'),
+    meFOURCC('Q','1','.','1'),
+    0
+  },
+  BUF_VIDEO_QPEG,
+  "Q-Team QPEG Video"
+},
+{
+  {
+    meFOURCC('R','T','2','1'),
+    0
+  },
+  BUF_VIDEO_RT21,
+  "Winnow Video"
+},
+{
+  {
+    meFOURCC('F','P','S','1'),
+    0
+  },
+  BUF_VIDEO_FPS1,
+  "Fraps FPS1"
+},
+{
+  {
+    meFOURCC('T','M','2','0'),
+    0
+  },
+  BUF_VIDEO_DUCKTM2,
+  "Duck TrueMotion 2"
+},
 { { 0 }, 0, "last entry" }
 };
 
@@ -533,7 +746,9 @@ static video_db_t video_db[] = {
 static audio_db_t audio_db[] = {
 {
   {
-    0x2000, 0
+    0x2000,
+    meFOURCC('m', 's', 0x20, 0x00),
+    0
   },
   BUF_AUDIO_A52,
   "AC3"
@@ -543,6 +758,7 @@ static audio_db_t audio_db[] = {
     0x50, 0x55,
     meFOURCC('.','m','p','3'),
     meFOURCC('m', 's', 0, 0x55),
+    meFOURCC('M','P','3',' '),
     0
   },
   BUF_AUDIO_MPEG,
@@ -551,6 +767,7 @@ static audio_db_t audio_db[] = {
 {
   {
     meFOURCC('t','w','o','s'),
+    meFOURCC('i','n','2','4'),
     0
   },
   BUF_AUDIO_LPCM_BE,
@@ -589,7 +806,21 @@ static audio_db_t audio_db[] = {
 },
 {
   {
-    0
+    0x163, 0
+  },
+  BUF_AUDIO_WMALL,
+  "Windows Media Audio Lossless"
+},
+{
+  {
+    0xA, 0
+  },
+  BUF_AUDIO_WMAV,
+  "Windows Media Audio Voice"
+},
+{
+  {
+    0x2001, 0
   },
   BUF_AUDIO_DTS,
   "DTS"
@@ -691,6 +922,7 @@ static audio_db_t audio_db[] = {
 {
   {
     meFOURCC('m', 'a', 'c', '3'),
+    meFOURCC('M', 'A', 'C', '3'),
     0
   },
   BUF_AUDIO_MAC3,
@@ -699,6 +931,7 @@ static audio_db_t audio_db[] = {
 {
   {
     meFOURCC('m', 'a', 'c', '6'),
+    meFOURCC('M', 'A', 'C', '6'),
     0
   },
   BUF_AUDIO_MAC6,
@@ -722,8 +955,12 @@ static audio_db_t audio_db[] = {
 },
 {
   {
+    0xFF,
     meFOURCC('m', 'p', '4', 'a'),
     meFOURCC('M', 'P', '4', 'A'),
+    meFOURCC('r', 'a', 'a', 'c'),
+    meFOURCC('r', 'a', 'c', 'p'),
+    meFOURCC('A', 'A', 'C', ' '), 
     0
   },
   BUF_AUDIO_AAC,
@@ -802,11 +1039,55 @@ static audio_db_t audio_db[] = {
   BUF_AUDIO_FLAC,
   "Free Lossless Audio Codec (FLAC)"
 },
+{
+  {
+    0
+  },
+  BUF_AUDIO_DV,
+  "DV Audio"
+},
+{
+  {
+    meFOURCC('l', 'p', 'c', 'J'),
+    0
+  },
+  BUF_AUDIO_14_4,
+  "Real 14.4"
+},
+{
+  {
+    meFOURCC('2', '8', '_', '8'),
+    0
+  },
+  BUF_AUDIO_28_8,
+  "Real 28.8"
+},
+{
+  {
+    0
+  },
+  BUF_AUDIO_SPEEX,
+  "Speex"
+},
+{
+  {
+    meFOURCC('a', 'l', 'a', 'c'),
+  },
+  BUF_AUDIO_ALAC,
+  "Apple Lossless Audio Codec"
+},
+{
+  {
+    0
+  },
+  BUF_AUDIO_MPC,
+  "Musepack"
+},
 { { 0 }, 0, "last entry" }
 };
 
 
-uint32_t fourcc_to_buf_video( uint32_t fourcc_int ) {
+uint32_t _x_fourcc_to_buf_video( uint32_t fourcc_int ) {
 int i, j;
 static uint32_t cached_fourcc=0;
 static uint32_t cached_buf_type=0;
@@ -826,7 +1107,7 @@ static uint32_t cached_buf_type=0;
   return 0;
 }
 
-char * buf_video_name( uint32_t buf_type ) {
+char * _x_buf_video_name( uint32_t buf_type ) {
 int i;
   
   buf_type &= 0xffff0000;
@@ -840,7 +1121,7 @@ int i;
   return "";
 }
 
-uint32_t formattag_to_buf_audio( uint32_t formattag ) {
+uint32_t _x_formattag_to_buf_audio( uint32_t formattag ) {
 int i, j;
 static uint16_t cached_formattag=0;
 static uint32_t cached_buf_type=0;
@@ -860,7 +1141,7 @@ static uint32_t cached_buf_type=0;
   return 0;
 }
 
-char * buf_audio_name( uint32_t buf_type ) {
+char * _x_buf_audio_name( uint32_t buf_type ) {
 int i;
   
   buf_type &= 0xffff0000;
@@ -874,7 +1155,7 @@ int i;
   return "";
 }
 
-void xine_bmiheader_le2me( xine_bmiheader *bih ) {
+void _x_bmiheader_le2me( xine_bmiheader *bih ) {
   /* OBS: fourcc must be read using machine endianness
    *      so don't play with biCompression here!
    */
@@ -891,7 +1172,7 @@ void xine_bmiheader_le2me( xine_bmiheader *bih ) {
   bih->biClrImportant = le2me_32(bih->biClrImportant);
 }
 
-void xine_waveformatex_le2me( xine_waveformatex *wavex ) {
+void _x_waveformatex_le2me( xine_waveformatex *wavex ) {
   
   wavex->wFormatTag = le2me_16(wavex->wFormatTag);
   wavex->nChannels = le2me_16(wavex->nChannels);
