@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2005 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -31,7 +31,9 @@
 #include <stdio.h>
 #include <zlib.h>
 #include <sys/types.h>
+#ifdef HAVE_DIRENT_H
 #include <dirent.h>
+#endif
 #include <errno.h>
 
 #ifdef HAVE_ICONV
@@ -47,7 +49,6 @@
 #define XINE_ENGINE_INTERNAL
 
 #include "xine_internal.h"
-#include "video_out/alphablend.h"
 #include "xine-engine/bswap.h"
 #include "xineutils.h"
 #include "video_out.h"
@@ -236,10 +237,10 @@ static int _osd_show (osd_object_t *osd, int64_t vpts, int unscaled ) {
     this->event.object.overlay->width = osd->x2 - osd->x1;
     this->event.object.overlay->height = osd->y2 - osd->y1;
  
-    this->event.object.overlay->clip_top    = 0;
-    this->event.object.overlay->clip_bottom = this->event.object.overlay->height;
-    this->event.object.overlay->clip_left   = 0;
-    this->event.object.overlay->clip_right  = this->event.object.overlay->width;
+    this->event.object.overlay->hili_top    = 0;
+    this->event.object.overlay->hili_bottom = this->event.object.overlay->height;
+    this->event.object.overlay->hili_left   = 0;
+    this->event.object.overlay->hili_right  = this->event.object.overlay->width;
    
     /* there will be at least that many rle objects (one for each row) */
     required = osd->y2 - osd->y1;
@@ -296,8 +297,8 @@ static int _osd_show (osd_object_t *osd, int64_t vpts, int unscaled ) {
 #endif
     lprintf("num_rle = %d\n", this->event.object.overlay->num_rle);
   
-    memcpy(this->event.object.overlay->clip_color, osd->color, sizeof(osd->color)); 
-    memcpy(this->event.object.overlay->clip_trans, osd->trans, sizeof(osd->trans)); 
+    memcpy(this->event.object.overlay->hili_color, osd->color, sizeof(osd->color)); 
+    memcpy(this->event.object.overlay->hili_trans, osd->trans, sizeof(osd->trans)); 
     memcpy(this->event.object.overlay->color, osd->color, sizeof(osd->color)); 
     memcpy(this->event.object.overlay->trans, osd->trans, sizeof(osd->trans)); 
   
@@ -399,6 +400,11 @@ static void osd_point (osd_object_t *osd, int x, int y, int color) {
   
   lprintf("osd=%p (%d x %d)\n", osd, x, y);
   
+  if (x < 0 || x >= osd->width)
+    return;
+  if (y < 0 || y >= osd->height)
+    return;
+  
   /* update clipping area */
   osd->x1 = MIN(osd->x1, x);
   osd->x2 = MAX(osd->x2, (x + 1));
@@ -406,8 +412,7 @@ static void osd_point (osd_object_t *osd, int x, int y, int color) {
   osd->y2 = MAX(osd->y2, (y + 1));
 
   c = osd->area + y * osd->width + x;
-  if(c <= (osd->area + (osd->width * osd->height)))
-    *c = color;
+  *c = color;
 }
 
 /*
@@ -419,21 +424,66 @@ static void osd_line (osd_object_t *osd,
      
   uint8_t *c;
   int dx, dy, t, inc, d, inc1, inc2;
+  int swap_x = 0;
+  int swap_y = 0;
 
   lprintf("osd=%p (%d,%d)-(%d,%d)\n",osd, x1,y1, x2,y2 );
-     
-  /* update clipping area */
-  t = MIN( x1, x2 );
-  osd->x1 = MIN( osd->x1, t );
-  t = MAX( x1, x2 );
-  osd->x2 = MAX( osd->x2, t );
-  t = MIN( y1, y2 );
-  osd->y1 = MIN( osd->y1, t );
-  t = MAX( y1, y2 );
-  osd->y2 = MAX( osd->y2, t );
+  
+  /* sort line */
+  if (x2 < x1) {
+    t  = x1;
+    x1 = x2;
+    x2 = t;
+    swap_x = 1;
+  }
+  if (y2 < y1) {
+    t  = y1;
+    y1 = y2;
+    y2 = t;
+    swap_y = 1;
+  }
 
-  dx = abs(x1-x2);
-  dy = abs(y1-y2);
+  /* clip line */ 
+  if (x1 < 0) {
+    y1 = y1 + (y2-y1) * -x1 / (x2-x1);
+    x1 = 0;
+  }
+  if (y1 < 0) {
+    x1 = x1 + (x2-x1) * -y1 / (y2-y1);
+    y1 = 0;
+  }
+  if (x2 > osd->width) {
+    y2 = y1 + (y2-y1) * (osd->width-x1) / (x2-x1);
+    x2 = osd->width;
+  }
+  if (y2 > osd->height) {
+    x2 = x1 + (x2-x1) * (osd->height-y1) / (y2-y1);
+    y2 = osd->height;
+  }
+  
+  if (x1 >= osd->width || y1 >= osd->height)
+    return;
+ 
+  /* update clipping area */
+  osd->x1 = MIN( osd->x1, x1 );
+  osd->x2 = MAX( osd->x2, x2 );
+  osd->y1 = MIN( osd->y1, y1 );
+  osd->y2 = MAX( osd->y2, y2 );
+  
+  dx = x2 - x1;
+  dy = y2 - y1;
+  
+  /* unsort line */
+  if (swap_x) {
+    t  = x1;
+    x1 = x2;
+    x2 = t;
+  }
+  if (swap_y) {
+    t  = y1;
+    y1 = y2;
+    y2 = t;
+  }
 
   if( dx>=dy ) {
     if( x1>x2 )
@@ -452,11 +502,7 @@ static void osd_line (osd_object_t *osd,
     
     while(x1<x2)
     {
-
-      if(c <= (osd->area + (osd->width * osd->height)))
-	*c = color;
-
-      c++;
+      *c++ = color;
       
       x1++;
       if( d<0 ) {
@@ -482,9 +528,7 @@ static void osd_line (osd_object_t *osd,
     c = osd->area + y1 * osd->width + x1;
 
     while(y1<y2) {
-
-      if(c <= (osd->area + (osd->width * osd->height)))
-	*c = color;
+      *c = color; 
       
       c += osd->width;
       y1++;
@@ -510,15 +554,33 @@ static void osd_filled_rect (osd_object_t *osd,
   int x, y, dx, dy;
 
   lprintf("osd=%p (%d,%d)-(%d,%d)\n",osd, x1,y1, x2,y2 );
-
-  /* update clipping area */
-  x = MIN( x1, x2 );
-  osd->x1 = MIN( osd->x1, x );
+  
+  /* sort rectangle */
+  x  = MIN( x1, x2 );
   dx = MAX( x1, x2 );
-  osd->x2 = MAX( osd->x2, dx );
-  y = MIN( y1, y2 );
-  osd->y1 = MIN( osd->y1, y );
+  y  = MIN( y1, y2 );
   dy = MAX( y1, y2 );
+  
+  /* clip rectangle */
+  if (x >= osd->width || y >= osd->height)
+    return;
+ 
+  if (x < 0) {
+    dx += x;
+    x = 0;
+  }
+  if (y < 0) {
+    dy += y;
+    y = 0;
+  }
+
+  dx = MIN( dx, osd->width );
+  dy = MIN( dy, osd->height );
+   
+  /* update clipping area */
+  osd->x1 = MIN( osd->x1, x );
+  osd->x2 = MAX( osd->x2, dx );
+  osd->y1 = MIN( osd->y1, y );
   osd->y2 = MAX( osd->y2, dy );
 
   dx -= x;
@@ -966,7 +1028,7 @@ static int osd_set_encoding (osd_object_t *osd, const char *encoding) {
 	      _("osd: can't find out current locale character set\n"));
       return 0;
     }
-    lprintf("locale encoding='%s'\n", osd, enc);
+    lprintf("locale encoding='%s'\n", enc);
   } else
     enc = strdup(encoding);
 
@@ -1066,7 +1128,7 @@ static int osd_render_text (osd_object_t *osd, int x1, int y1,
       }
       previous = i;
 
-      if (FT_Load_Glyph(osd->ft2->face, i, FT_LOAD_DEFAULT)) {
+      if (FT_Load_Glyph(osd->ft2->face, i, FT_LOAD_DEFAULT | FT_LOAD_NO_HINTING)) {
         xprintf(this->stream->xine, XINE_VERBOSITY_LOG, _("osd: error loading glyph\n"));
         continue;
       }
@@ -1232,7 +1294,7 @@ static int osd_get_text_size(osd_object_t *osd, const char *text, int *width, in
       }
       previous = i;
 
-      if (FT_Load_Glyph(osd->ft2->face, i, FT_LOAD_DEFAULT)) {
+      if (FT_Load_Glyph(osd->ft2->face, i, FT_LOAD_DEFAULT | FT_LOAD_NO_HINTING)) {
         xprintf(this->stream->xine, XINE_VERBOSITY_LOG, _("osd: error loading glyph %i\n"), i);
         text++;
         continue;
@@ -1273,7 +1335,8 @@ static int osd_get_text_size(osd_object_t *osd, const char *text, int *width, in
      * to also add the left bearing because the letter might be shifted left or
      * right and then the right edge is also shifted
      */
-    *width -= osd->ft2->face->glyph->advance.x / 64;
+    if (osd->ft2->face->glyph->bitmap.width)
+        *width -= osd->ft2->face->glyph->advance.x / 64;
     *width += osd->ft2->face->glyph->bitmap.width;
     *width += osd->ft2->face->glyph->bitmap_left;
     *height = osd->ft2->face->size->metrics.height / 64;
