@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: plugin_catalog.h,v 1.16 2004/06/23 19:45:54 mroi Exp $
+ * $Id: plugin_catalog.h,v 1.20 2006/01/27 07:46:15 tmattern Exp $
  *
  * xine-internal header: Definitions for plugin lists
  *
@@ -46,24 +46,26 @@
 
 typedef struct {
   char            *filename;
-  plugin_info_t   *info;
-  void            *plugin_class;
-  int              ref; /* count instances created of this plugin */
   off_t            filesize;
   time_t           filemtime;
+  int              ref;          /* count number of classes */
+  void            *lib_handle;
+  int              no_unload;    /* set if the file can't be unloaded */
+} plugin_file_t ;
+
+typedef struct {
+  plugin_file_t   *file;
+  plugin_info_t   *info;
+  void            *plugin_class;
+  int              ref;          /* count intances of plugins */
+  int              priority;
 } plugin_node_t ;
 
 struct plugin_catalog_s {
-  xine_list_t     *input;
-  xine_list_t     *demux;
-  xine_list_t     *spu;
-  xine_list_t     *audio;
-  xine_list_t     *video;
-  xine_list_t     *aout;
-  xine_list_t     *vout;
-  xine_list_t     *post;
+  xine_sarray_t   *plugin_lists[PLUGIN_TYPE_MAX];
   
-  xine_list_t     *cache;
+  xine_sarray_t   *cache_list;
+  xine_list_t     *file_list;
 
   plugin_node_t   *audio_decoder_map[DECODER_MAX][PLUGINS_PER_TYPE];
   plugin_node_t   *video_decoder_map[DECODER_MAX][PLUGINS_PER_TYPE];
@@ -75,6 +77,9 @@ struct plugin_catalog_s {
   char            *prio_desc[DECODER_MAX];
 
   pthread_mutex_t  lock;
+
+  int              plugin_count;
+  int              decoder_count;
 };
 typedef struct plugin_catalog_s plugin_catalog_t;
 
